@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.49 2006/03/18 15:05:31 ryo_saeba Exp $";
+ *   "$Id: init.c 5555 2007-02-16 00:06:50Z ryo_saeba $";
  */
 
 /*
@@ -44,7 +44,6 @@ CSPORT,				/* Client/server port */
 #else
   llevInfo,
 #endif
-
 0, NULL, 0,    /* dumpvalues, dumparg, daemonmode */
 0, /* argc */
 NULL, /* argv */
@@ -96,6 +95,8 @@ ARMOR_WEIGHT_LINEAR,
 ARMOR_SPEED_IMPROVEMENT,
 ARMOR_SPEED_LINEAR,
 1, /* no_player_stealing */
+0, /* create_home_portals */
+0, /* allow_denied_spells_writing */
 };
 
 /** perhaps not the best place for this, but needs to be
@@ -248,8 +249,8 @@ void init_globals(void) {
     first_treasurelist=NULL;
     first_artifactlist=NULL;
     first_archetype=NULL;
+    *first_map_ext_path=0;
     warn_archetypes=0;
-    first_map=NULL;
     nroftreasures = 0;
     nrofartifacts = 0;
     nrofallowedstr=0;
@@ -313,9 +314,14 @@ void init_defaults(void) {
 void init_dynamic (void) {
     archetype *at = first_archetype;
     while (at) {
-	if (at->clone.type == MAP && EXIT_PATH (&at->clone)) {
-	    strcpy (first_map_path, EXIT_PATH (&at->clone));
-	    return;
+	if (at->clone.type == MAP) {
+            if (at->clone.race) {
+                strcpy (first_map_ext_path, at->clone.race);
+            }
+            if (EXIT_PATH (&at->clone)) {
+                strcpy (first_map_path, EXIT_PATH (&at->clone));
+                return;
+            }
 	}
 	at = at->next;
     }
