@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.63 2006/06/05 06:28:55 mwedel Exp $";
+ *   "$Id: item.c 5544 2007-02-15 19:44:29Z ryo_saeba $";
  */
 
 /*
@@ -592,7 +592,7 @@ char *query_name(const object *op) {
     use_buf %=5;
 
 #ifdef NEW_MATERIAL_CODE
-    if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname) {
+    if ((IS_ARMOR(op) || IS_WEAPON(op) || IS_SHIELD(op)) && op->materialname) {
 	mt = name_to_material(op->materialname);
 	if (mt) {
 	    safe_strcat(buf[use_buf], mt->description, &len, HUGE_BUF);
@@ -981,11 +981,11 @@ char *describe_item(const object *op, const object *owner) {
 
 	case POWER_CRYSTAL:
 	    if (op->stats.maxsp>1000){ /*higher capacity crystals*/
-	        i = (op->stats.maxsp%100)/10;
+	        i = (op->stats.maxsp%1000)/100;
 	        if (i)
-	            snprintf(buf,MAX_BUF,"(capacity %d.%dk). It is ",op->stats.maxsp/100,i);
+	            snprintf(buf,MAX_BUF,"(capacity %d.%dk). It is ",op->stats.maxsp/1000,i);
                 else
-	            snprintf(buf,MAX_BUF,"(capacity %dk). It is ",op->stats.maxsp/100);
+	            snprintf(buf,MAX_BUF,"(capacity %dk). It is ",op->stats.maxsp/1000);
 	    }else
 	        snprintf(buf,MAX_BUF,"(capacity %d). It is ",op->stats.maxsp);
 	    strcat(retbuf,buf);
@@ -1061,12 +1061,16 @@ char *describe_item(const object *op, const object *owner) {
 	    }
 	}
 
-	if(op->stats.exp) {
-	    sprintf(buf,"(speed %+" FMT64 ")",op->stats.exp);
-	    strcat(retbuf,buf);
-	}
-
-
+        switch (op->type) {
+            case FLESH:
+                break;
+            default:
+                if(op->stats.exp) {
+                    sprintf(buf,"(speed %+" FMT64 ")",op->stats.exp);
+                    strcat(retbuf,buf);
+                }
+                break;
+        }
 	switch(op->type) {
 	    case BOW:
 	    case ARROW:
@@ -1386,7 +1390,7 @@ void identify(object *op) {
     if (op->map)
 	update_object(op,UP_OBJ_FACE);
     else {
-	pl = is_player_inv(op->env);
+	pl = get_player_container(op->env);
 	if (pl)
 	    /* A lot of the values can change from an update - might as well send
 	     * it all.
