@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c,v 1.78 2006/04/06 21:18:34 tchize Exp $";
+ *   "$Id: living.c 5138 2006-11-26 18:23:36Z ryo_saeba $";
  */
 
 /*
@@ -765,7 +765,6 @@ void add_statbonus(object *op) {
 
 void fix_player(object *op) {
     int i,j;
-    event *evt;
     float f,max=9,added_speed=0,bonus_speed=0, sp_tmp,speed_reduce_from_disease=1;
     int weapon_weight=0,weapon_speed=0;
     int best_wc=0, best_ac=0, wc=0, ac=0;
@@ -1082,12 +1081,6 @@ void fix_player(object *op) {
 		     * extra strength damage, this is where the code should
 		     * go.
 		     */
-		    evt = find_event(tmp,EVENT_ATTACK);
-		    if (evt != NULL) {
-			if (op->current_weapon_script)
-			    free_string(op->current_weapon_script);
-			op->current_weapon_script=add_string(query_name(tmp));
-		    }
 		    op->current_weapon = tmp;
 		    if(settings.spell_encumbrance == TRUE && op->type==PLAYER)
 			op->contr->encumbrance+=(int)3*tmp->weight/1000;
@@ -1290,9 +1283,7 @@ void fix_player(object *op) {
     else /* Something wrong here...: */
 	op->speed /= (float)(1.0-added_speed);
 
-    /* Max is determined by armour */
-    if(op->speed>max)
-	op->speed=max;
+    op->speed+=bonus_speed/10.0; /* Not affected by limits */
 
     if(op->type == PLAYER) {
 	/* f is a number the represents the number of kg above (positive num)
@@ -1301,10 +1292,13 @@ void fix_player(object *op) {
 	 * much above he is, and what is max carry is
 	 */
 	f=(op->carrying/1000)-max_carry[op->stats.Str];
-	if(f>0) op->speed=op->speed/(1.0+f/max_carry[op->stats.Str]);
+	if(f>0)
+	    op->speed=op->speed/(1.0+f/max_carry[op->stats.Str]);
     }
 
-    op->speed+=bonus_speed/10.0; /* Not affected by limits */
+    /* Max is determined by armour */
+    if(op->speed>max)
+        op->speed=max;
 
     /* Put a lower limit on speed.  Note with this speed, you move once every
      * 100 ticks or so.  This amounts to once every 12 seconds of realtime.
