@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.109 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: map.c 5806 2007-03-17 22:45:24Z ryo_saeba $";
  */
 
 /*
@@ -309,7 +309,7 @@ int get_map_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sin
  */
 
 int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
-    object *tmp;
+    object *tmp, *tmp_head;
     int mflags, blocked;
 
     /* Make sure the coordinates are valid - they should be, as caller should
@@ -351,9 +351,16 @@ int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
      * ob is blocking it, so return 0.
      */
     for(tmp = GET_MAP_OB(m,sx,sy); tmp!= NULL; tmp = tmp->above) {
-
-	/* This must be before the checks below.  Code for inventory checkers. */
-	if (tmp->type==CHECK_INV && OB_MOVE_BLOCK(ob, tmp)) {
+        /* Never block part of self. */
+        if (tmp->head)
+            tmp_head = tmp->head;
+        else
+            tmp_head = tmp;
+        if (tmp_head == ob) {
+            continue;
+	}
+        /* This must be before the checks below.  Code for inventory checkers. */
+	else if (tmp->type==CHECK_INV && OB_MOVE_BLOCK(ob, tmp)) {
 	    /* If last_sp is set, the player/monster needs an object,
 	     * so we check for it.  If they don't have it, they can't
 	     * pass through this space.
@@ -377,7 +384,7 @@ int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
 	    /* Broke apart a big nasty if into several here to make
 	     * this more readable.  first check - if the space blocks
 	     * movement, can't move here.
-	     * second - if a monster, can't move there, unles it is a 
+	     * second - if a monster, can't move there, unless it is a 
 	     * hidden dm
 	     */
 	    if (OB_MOVE_BLOCK(ob, tmp)) return 1;
@@ -480,7 +487,8 @@ void fix_container(object *container)
     sum_weight(container);
 }
 
-/* link_multipart_objects go through all the objects on the map looking
+/**
+ *link_multipart_objects go through all the objects on the map looking
  * for objects whose arch says they are multipart yet according to the
  * info we have, they only have the head (as would be expected when
  * they are saved).  We do have to look for the old maps that did save
@@ -490,8 +498,7 @@ void fix_container(object *container)
 static void link_multipart_objects(mapstruct *m)
 {
     int x,y;
-    object *tmp, *op, *last, *above;
-    archetype	*at;
+    object *tmp, *above;
 
     for(x=0;x<MAP_WIDTH(m);x++)
 	for(y=0;y<MAP_HEIGHT(m);y++)
@@ -501,33 +508,7 @@ static void link_multipart_objects(mapstruct *m)
 		/* already multipart - don't do anything more */
 		if (tmp->head || tmp->more) continue;
 
-		/* If there is nothing more to this object, this for loop
-		 * won't do anything.
-		 */
-		for (at = tmp->arch->more, last=tmp; at != NULL; at=at->more, last=op) {
-		    op = arch_to_object(at);
-
-		    /* update x,y coordinates */
-		    op->x += tmp->x;
-		    op->y += tmp->y;
-		    op->head = tmp;
-		    op->map = m;
-		    last->more = op;
-		    if (tmp->name != op->name) {
-			if (op->name) free_string(op->name);
-			op->name = add_string(tmp->name);
-		    }
-		    if (tmp->title != op->title) {
-			if (op->title) free_string(op->title);
-			op->title = add_string(tmp->title);
-		    }
-		    /* we could link all the parts onto tmp, and then just
-		     * call insert_ob_in_map once, but the effect is the same,
-		     * as insert_ob_in_map will call itself with each part, and
-		     * the coding is simpler to just to it here with each part.
-		     */
-		    insert_ob_in_map(op, op->map, tmp,INS_NO_MERGE|INS_ABOVE_FLOOR_ONLY|INS_NO_WALK_ON);
-		} /* for at = tmp->arch->more */
+        fix_multipart_object(tmp);
 	    } /* for objects on this space */
 }
 		    
@@ -676,6 +657,7 @@ mapstruct *get_linked_map(void) {
 	/*set part to -1 indicating conversion to weather map not yet done*/
 	MAP_WORLDPARTX(map)=-1;
 	MAP_WORLDPARTY(map)=-1;
+    map->last_reset_time.tv_sec = 0;
     return map;
 }
 
@@ -972,6 +954,9 @@ static int load_map_header(FILE *fp, mapstruct *m)
 	} else if (!strcmp(key, "nosmooth")) {
 	    m->nosmooth = atoi(value);
 	}
+    else if (!strcmp(key, "first_load")) {
+        m->last_reset_time.tv_sec = atoi(value);
+    }
 	else if (!strncmp(key,"tile_path_", 10)) {
 	    int tile=atoi(key+10);
 
@@ -1075,6 +1060,10 @@ mapstruct *load_original_map(const char *filename, int flags) {
     if (!MAP_DIFFICULTY(m)) 
 	MAP_DIFFICULTY(m)=calculate_difficulty(m);
     set_map_reset_time(m);
+    
+    /* Handle for map load event */
+    execute_global_event(EVENT_MAPLOAD, m);
+    
     return (m);
 }
 
@@ -1321,6 +1310,7 @@ int new_save_map(mapstruct *m, int flag) {
     if (m->winddir) fprintf(fp, "winddir %d\n", m->winddir);
     if (m->sky) fprintf(fp, "sky %d\n", m->sky);
     if (m->nosmooth) fprintf(fp, "nosmooth %d\n", m->nosmooth);
+    if (m->last_reset_time.tv_sec) fprintf(fp, "first_load %d\n", (int)m->last_reset_time.tv_sec);
 
     /* Save any tiling information, except on overlays */
     if (flag != 2)
@@ -1341,8 +1331,11 @@ int new_save_map(mapstruct *m, int flag) {
 	if ((fp2 = fopen (buf, "w")) == NULL) {
 	    LOG(llevError, "Can't open unique items file %s\n", buf);
 	}
-	if (flag == 2)
+    if (flag == 2) {
+        // 2 is non destructive save, so map is still valid.
 	    save_objects(m, fp, fp2, 2);
+        m->in_memory = MAP_IN_MEMORY;
+    }
 	else
 	    save_objects (m, fp, fp2, 0);
 	if (fp2 != NULL) {
@@ -1443,6 +1436,10 @@ void free_map(mapstruct *m,int flag) {
 	LOG(llevError,"Trying to free freed map.\n");
 	return;
     }
+    
+    /* Handle for plugin map unload event. */
+    execute_global_event(EVENT_MAPUNLOAD, m);
+    
     if (flag && m->spaces) free_all_objects(m);
     if (m->name) FREE_AND_CLEAR(m->name);
     if (m->spaces) FREE_AND_CLEAR(m->spaces);
@@ -1616,6 +1613,11 @@ mapstruct *ready_map_name(const char *name, int flags) {
 	    set_darkness_map(m);
     /* run the weather over this map */
     weather_effect(name);
+    if (!(flags & (MAP_FLUSH)))
+    {
+        if (m->last_reset_time.tv_sec==0)
+            gettimeofday(&(m->last_reset_time),NULL);
+    }
     return m;
 }
 
@@ -1832,12 +1834,16 @@ void update_position (mapstruct *m, int x, int y) {
 
     for (tmp = get_map_ob (m, x, y); tmp; tmp = tmp->above) {
 
-	/* This could be made additive I guess (two lights better than
-	 * one).  But if so, it shouldn't be a simple additive - 2
-	 * light bulbs do not illuminate twice as far as once since
-	 * it is a dissipation factor that is squared (or is it cubed?)
-	 */
-	if (tmp->glow_radius > light) light = tmp->glow_radius;
+        /* DMs just don't do anything when hidden, including no light. */
+        if (QUERY_FLAG(tmp, FLAG_WIZ) && tmp->contr->hidden)
+            continue;
+
+        /* This could be made additive I guess (two lights better than
+         * one).  But if so, it shouldn't be a simple additive - 2
+         * light bulbs do not illuminate twice as far as once since
+         * it is a dissipation factor that is squared (or is it cubed?)
+         */
+        if (tmp->glow_radius > light) light = tmp->glow_radius;
 
 	/* if this object is visible and not a blank face,
 	 * update the objects that show how this space
@@ -1934,7 +1940,10 @@ void set_map_reset_time(mapstruct *map) {
 static mapstruct *load_and_link_tiled_map(mapstruct *orig_map, int tile_num)
 {
     int dest_tile = (tile_num +2) % 4;
-    char *path = path_combine_and_normalize(orig_map->path, orig_map->tile_path[tile_num]);
+    char path[HUGE_BUF];
+    path[0] = '\0';
+    strncpy(path, path_combine_and_normalize(orig_map->path, orig_map->tile_path[tile_num]), HUGE_BUF - 1);
+    path[HUGE_BUF-1] = '\0';
 
     orig_map->tile_map[tile_num] = ready_map_name(path, 0);
 
