@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_c =
- *   "$Id: object.c,v 1.133 2006/06/24 10:41:44 ryo_saeba Exp $";
+ *   "$Id: object.c 5159 2006-12-09 18:12:21Z ryo_saeba $";
  */
 
 /*
@@ -48,6 +48,7 @@ static void expand_objects(void);
 static void free_object2(object *, int);
 static void permute(int *, int, int);
 static int set_ob_key_value_s(object *, const char *, const char *, int);
+static void get_multi_size(object *, int *, int *, int *, int *);
 
 
 
@@ -188,10 +189,11 @@ int can_merge(object *ob1, object *ob2) {
 	SET_FLAG(ob2, FLAG_BEEN_APPLIED);
 
 
+    /* Note: FLAG_INV_LOCKED is ignored for merging purposes */
     if ((ob1->arch != ob2->arch) || 
 	(ob1->flags[0] != ob2->flags[0]) || 
 	(ob1->flags[1] != ob2->flags[1]) ||
-    ((ob1->flags[2] & ~( 1 << ( FLAG_INV_LOCKED % 32 ) )) != (ob2->flags[2] & ~ ( 1 << ( FLAG_INV_LOCKED % 32 )))) ||
+	(ob1->flags[2] != ob2->flags[2]) ||
 	((ob1->flags[3] & ~0x4) != (ob2->flags[3] & ~0x4)) || /* ignore CLIENT_SENT */
 	(ob1->name != ob2->name) || 
 	(ob1->title != ob2->title) ||
@@ -289,7 +291,7 @@ object *object_get_env_recursive (object *op) {
  * or find a player.
  */
 /*TODO this function is badly named*/
-object *is_player_inv (object *op) { 
+object *get_player_container(object *op) { 
     for (;op!=NULL&&op->type!=PLAYER; op=op->env)
       /*TODO this is patching the structure on the flight as side effect. Shoudln't be needed in clean code */
       if (op->env==op)
@@ -324,6 +326,21 @@ static void dump_object2(object *op) {
       for (tmp=op->inv; tmp; tmp=tmp->below)
         dump_object2(tmp);
 #endif
+      if (op->more) {
+          strcat(errmsg, "more ");
+          strcat(errmsg, ltostr10(op->more->count));
+          strcat(errmsg, "\n");
+      }
+      if (op->head) {
+          strcat(errmsg, "head ");
+          strcat(errmsg, ltostr10(op->head->count));
+          strcat(errmsg, "\n");
+      }
+      if (op->env) {
+          strcat(errmsg, "env ");
+          strcat(errmsg, ltostr10(op->env->count));
+          strcat(errmsg, "\n");
+      }
       strcat(errmsg,"end\n");
   } else {
       strcat(errmsg,"Object ");
@@ -539,7 +556,7 @@ void set_owner (object *op, object *owner)
      */
     if (owner->owner){
       LOG(llevError,"owner id %d could not be resolved to a parent owner sin set_owner(). This is bad!"
-          "owner=%p owner->owner=%p owner->ownercount=%d owner->owner->count=%d ",
+          "owner=%p owner->owner=%p owner->ownercount=%d owner->owner->count=%d\n",
           owner,owner->owner,owner->ownercount, owner->owner->count);
       return;
     }
@@ -627,26 +644,12 @@ static void free_key_values(object * op) {
 
 void clear_object(object *op) {
 
-    event *evt;
-    event *evt2;
-
     /*TODO this comment must be investigated*/
     /* redo this to be simpler/more efficient. Was also seeing
      * crashes in the old code.  Move this to the top - am
      * seeing periodic crashes in this code, and would like to have
      * as much info available as possible (eg, object name).
      */
-    for (evt = op->events; evt; evt=evt2) {
-	evt2 = evt->next;
-
-        if (evt->hook != NULL) FREE_AND_CLEAR_STR(evt->hook);
-        if (evt->plugin != NULL) FREE_AND_CLEAR_STR(evt->plugin);
-        if (evt->options != NULL) FREE_AND_CLEAR_STR(evt->options);
-
-        free(evt);
-    }
-    op->events = NULL;
-
     free_key_values(op);
 
     /* the memset will clear all these values for us, but we need
@@ -676,7 +679,6 @@ void clear_object(object *op) {
     op->below=NULL;
     op->above=NULL;
     op->inv=NULL;
-    op->events=NULL;
     op->container=NULL;
     op->env=NULL;
     op->more=NULL;
@@ -706,7 +708,6 @@ void clear_object(object *op) {
 
 void copy_object(object *op2, object *op) {
     int is_freed=QUERY_FLAG(op,FLAG_FREED),is_removed=QUERY_FLAG(op,FLAG_REMOVED);
-    event *evt, *evt2, *evt_new;
 
     /* Decrement the refcounts, but don't bother zeroing the fields;
     they'll be overwritten by memcpy. */
@@ -722,16 +723,6 @@ void copy_object(object *op2, object *op) {
     if(op->custom_name != NULL)		free_string(op->custom_name);
 
     /* Basically, same code as from clear_object() */
-    for (evt = op->events; evt; evt=evt2) {
-      evt2 = evt->next;
-
-      if (evt->hook != NULL) FREE_AND_CLEAR_STR(evt->hook);
-      if (evt->plugin != NULL) FREE_AND_CLEAR_STR(evt->plugin);
-      if (evt->options != NULL) FREE_AND_CLEAR_STR(evt->options);
-
-      free(evt);
-    }
-    op->events = NULL;
 
     free_key_values(op);
 
@@ -752,31 +743,16 @@ void copy_object(object *op2, object *op) {
     if(op->msg!=NULL)	    add_refcount(op->msg);
     if(op->custom_name!=NULL)	    add_refcount(op->custom_name);
     if (op->materialname != NULL)   add_refcount(op->materialname);
+    
+    /* If archetype is a temporary one, we need to update reference count, because
+     * that archetype will be freed by free_object when the last object is removed.
+     */
+    if (op->arch->reference_count > 0)
+        op->arch->reference_count++;
 
     if((op2->speed<0) && !editor)
 	op->speed_left=op2->speed_left-RANDOM()%200/100.0;
 
-    /* Copy over event information */
-    evt2 = NULL;
-    for (evt = op2->events; evt; evt=evt->next) {
-	evt_new = malloc(sizeof(event));
-	memcpy(evt_new, evt, sizeof(event));
-	if (evt_new->hook)  add_refcount(evt_new->hook);
-	if (evt_new->plugin)  add_refcount(evt_new->plugin);
-	if (evt_new->options)  add_refcount(evt_new->options);
-	evt_new->next = NULL;
-
-	/* Try to be a little clever here, and store away the
-	 * last event we copied, so that its simpler to update the
-	 * pointer.
-	 */
-	if (evt2)
-	    evt2->next = evt_new;
-	else
-	    op->events = evt_new;
-
-	evt2 = evt_new;
-    }
     /* Copy over key_values, if any. */
     if (op2->key_values != NULL) {
 	key_value * tail = NULL;
@@ -1113,91 +1089,127 @@ void free_object(object *ob) {
   free_object2(ob, 0);
 }
 static void free_object2(object *ob, int free_inventory) {
-  object *tmp,*op;
+    object *tmp,*op;
 
-  if (!QUERY_FLAG(ob,FLAG_REMOVED)) {
-    LOG(llevDebug,"Free object called with non removed object\n");
-    dump_object(ob);
+    if (!QUERY_FLAG(ob,FLAG_REMOVED)) {
+      LOG(llevDebug,"Free object called with non removed object\n");
+      dump_object(ob);
 #ifdef MANY_CORES
-    abort();
+      abort();
 #endif
-  }
-  if(QUERY_FLAG(ob,FLAG_FRIENDLY)) {
-    LOG(llevMonster,"Warning: tried to free friendly object.\n");
-    remove_friendly_object(ob);
-  }
-  if(QUERY_FLAG(ob,FLAG_FREED)) {
-    dump_object(ob);
-    LOG(llevError,"Trying to free freed object.\n%s\n",errmsg);
-    return;
-  }
-  if(ob->more!=NULL) {
-    free_object2(ob->more, free_inventory);
-    ob->more=NULL;
-  }
-  if (ob->inv) {
-  /* Only if the space blocks everything do we not process -
-   * if some form of movemnt is allowed, let objects
-   * drop on that space.
-   */
-    if (free_inventory || ob->map==NULL || ob->map->in_memory!=MAP_IN_MEMORY ||
-        (GET_MAP_MOVE_BLOCK(ob->map, ob->x, ob->y) == MOVE_ALL)) 
-    {
-      op=ob->inv;
-      while(op!=NULL) {
-        tmp=op->below;
-        remove_ob(op);
-        free_object2(op, free_inventory);
-        op=tmp;
-      }
-    } else { /* Put objects in inventory onto this space */
-      op=ob->inv;
-      while(op!=NULL) {
-        tmp=op->below;
-        remove_ob(op);
-        if(QUERY_FLAG(op,FLAG_STARTEQUIP)||QUERY_FLAG(op,FLAG_NO_DROP) ||
-           op->type==RUNE || op->type==TRAP || QUERY_FLAG(op,FLAG_IS_A_TEMPLATE))
-          free_object(op);
-        else {
-          op->x=ob->x;
-          op->y=ob->y;
-          insert_ob_in_map(op,ob->map,NULL,0); /* Insert in same map as the envir */
-        }
-      op=tmp;
-      }
     }
-  }
-  /* Remove object from the active list */
-  ob->speed = 0;
-  update_ob_speed(ob);
+    if(QUERY_FLAG(ob,FLAG_FRIENDLY)) {
+        LOG(llevMonster,"Warning: tried to free friendly object.\n");
+        remove_friendly_object(ob);
+    }
+    if(QUERY_FLAG(ob,FLAG_FREED)) {
+        dump_object(ob);
+        LOG(llevError,"Trying to free freed object.\n%s\n",errmsg);
+        return;
+    }
 
-  SET_FLAG(ob, FLAG_FREED);
-  ob->count = 0;
+    /* Handle for plugin destroy event */
+    execute_event(ob, EVENT_DESTROY,NULL,NULL,NULL,SCRIPT_FIX_NOTHING);
 
-  /* Remove this object from the list of used objects */
-  if(ob->prev==NULL) {
-    objects=ob->next;
-    if(objects!=NULL)
-    objects->prev=NULL;
-  } else {
-    ob->prev->next=ob->next;
-    if(ob->next!=NULL)
-      ob->next->prev=ob->prev;
-  }
+    if (ob->inv) {
+        /* Only if the space blocks everything do we not process -
+         * if some form of movemnt is allowed, let objects
+         * drop on that space.
+         */
+        if (free_inventory || ob->map==NULL || ob->map->in_memory!=MAP_IN_MEMORY ||
+            (GET_MAP_MOVE_BLOCK(ob->map, ob->x, ob->y) == MOVE_ALL)) {
+            op=ob->inv;
+            while(op!=NULL) {
+                tmp=op->below;
+                remove_ob(op);
+                free_object2(op, free_inventory);
+                op=tmp;
+            }
+        } else { /* Put objects in inventory onto this space */
+            op=ob->inv;
+            while(op!=NULL) {
+                tmp=op->below;
+                remove_ob(op);
+                if(QUERY_FLAG(op,FLAG_STARTEQUIP)||QUERY_FLAG(op,FLAG_NO_DROP) ||
+                   op->type==RUNE || op->type==TRAP || QUERY_FLAG(op,FLAG_IS_A_TEMPLATE))
+                    free_object(op);
+                else {
+                    object *part;
+                    /* If it's a multi-tile object, scatter dropped items randomly */
+                    if (ob->more) {
+                        int partcount = 0;
+                        /* Get the number of non-head parts */
+                        for(part = ob; part; part = part->more) {
+                            partcount++;
+                        }
+                        /* Select a random part */
+                        partcount = RANDOM()%partcount;
+                        for(part = ob; partcount>0; partcount--) {
+                            part = part->more;
+                        }
+                    } else {
+                        part = ob;
+                    }
 
-  if(ob->name!=NULL)	    FREE_AND_CLEAR_STR(ob->name);
-  if(ob->name_pl!=NULL)   FREE_AND_CLEAR_STR(ob->name_pl);
-  if(ob->title!=NULL)	    FREE_AND_CLEAR_STR(ob->title);
-  if(ob->race!=NULL)	    FREE_AND_CLEAR_STR(ob->race);
-  if(ob->slaying!=NULL)   FREE_AND_CLEAR_STR(ob->slaying);
-  if(ob->skill!=NULL)	    FREE_AND_CLEAR_STR(ob->skill);
-  if(ob->lore!=NULL)	    FREE_AND_CLEAR_STR(ob->lore);
-  if(ob->msg!=NULL)	    FREE_AND_CLEAR_STR(ob->msg);
-  if(ob->materialname!=NULL) FREE_AND_CLEAR_STR(ob->materialname);
+                    if ( QUERY_FLAG(op,FLAG_ALIVE) ) {
+                        int pos;
+
+                        pos = find_free_spot(op,part->map,part->x, part->y, 0, SIZEOFFREE);
+                        if ( pos == -1 )
+                            free_object(op);
+                        else {
+                            op->x=part->x + freearr_x[ pos ];
+                            op->y=part->y + freearr_y[ pos ];
+                            insert_ob_in_map(op,part->map,NULL,0); /* Insert in same map as the envir */
+                        }
+                    }
+                    else {
+                        op->x=part->x;
+                        op->y=part->y;
+                        insert_ob_in_map(op,part->map,NULL,0); /* Insert in same map as the envir */
+                    }
+                }
+                op=tmp;
+            }
+        }
+    }
+    
+    if(ob->more!=NULL) {
+        free_object2(ob->more, free_inventory);
+        ob->more=NULL;
+    }
+    
+    /* Remove object from the active list */
+    ob->speed = 0;
+    update_ob_speed(ob);
+
+    SET_FLAG(ob, FLAG_FREED);
+    ob->count = 0;
+
+    /* Remove this object from the list of used objects */
+    if(ob->prev==NULL) {
+        objects=ob->next;
+        if(objects!=NULL)
+            objects->prev=NULL;
+    } else {
+        ob->prev->next=ob->next;
+        if(ob->next!=NULL)
+            ob->next->prev=ob->prev;
+    }
+
+    if(ob->name!=NULL)	    FREE_AND_CLEAR_STR(ob->name);
+    if(ob->name_pl!=NULL)   FREE_AND_CLEAR_STR(ob->name_pl);
+    if(ob->title!=NULL)	    FREE_AND_CLEAR_STR(ob->title);
+    if(ob->race!=NULL)	    FREE_AND_CLEAR_STR(ob->race);
+    if(ob->slaying!=NULL)   FREE_AND_CLEAR_STR(ob->slaying);
+    if(ob->skill!=NULL)	    FREE_AND_CLEAR_STR(ob->skill);
+    if(ob->lore!=NULL)	    FREE_AND_CLEAR_STR(ob->lore);
+    if(ob->msg!=NULL)	    FREE_AND_CLEAR_STR(ob->msg);
+    if(ob->materialname!=NULL) FREE_AND_CLEAR_STR(ob->materialname);
 
 
-  /* Why aren't events freed? */
-  free_key_values(ob);
+    /* Why aren't events freed? */
+    free_key_values(ob);
 
 #if 0 /* MEMORY_DEBUG*/
   /* This is a nice idea.  Unfortunately, a lot of the code in crossfire
@@ -1210,16 +1222,23 @@ static void free_object2(object *ob, int free_inventory) {
    * and not freeing it, but the original one that malloc'd the
    * object will get the blame.
    */
-  free(ob);
+      free(ob);
 #else
 
+    /* Test whether archetype is a temporary one, and if so look whether it should be trashed. */
+    if (ob->arch && ob->arch->reference_count > 0) {
+        if (--ob->arch->reference_count == 0) {
+            free(ob->arch);
+        }
+    }
+
     /* Now link it with the free_objects list: */
-  ob->prev=NULL;
-  ob->next=free_objects;
-  if(free_objects!=NULL)
-  free_objects->prev=ob;
-  free_objects=ob;
-  nroffreeobjects++;
+    ob->prev=NULL;
+    ob->next=free_objects;
+    if(free_objects!=NULL)
+    free_objects->prev=ob;
+    free_objects=ob;
+    nroffreeobjects++;
 #endif
 }
 
@@ -1282,179 +1301,165 @@ void sub_weight (object *op, signed long weight) {
  *   the previous environment.
  *   Beware: This function is called from the editor as well!
  */
-
+/* TODO this function is a piece of overbloated crap or at lest
+ * look like need cleanup it does to much different things.
+ */
 void remove_ob(object *op) {
-    object *tmp,*last=NULL;
-    object *otmp;
-    tag_t tag;
-    int check_walk_off;
-    mapstruct *m;
-    sint16 x,y;
-    
+  object *tmp,*last=NULL;
+  object *otmp;
+  tag_t tag;
+  int check_walk_off;
+  mapstruct *m;
+  sint16 x,y;
 
-    if(QUERY_FLAG(op,FLAG_REMOVED)) {
-	dump_object(op);
-	LOG(llevError,"Trying to remove removed object.\n%s\n",errmsg);
+  if(QUERY_FLAG(op,FLAG_REMOVED)) {
+    dump_object(op);
+    LOG(llevError,"Trying to remove removed object.\n%s\n",errmsg);
+    abort();
+  }
+  if(op->more!=NULL)
+    remove_ob(op->more);
 
-	/* Changed it to always dump core in this case.  As has been learned
-	 * in the past, trying to recover from errors almost always
-	 * make things worse, and this is a real error here - something
-	 * that should not happen.
-	 * Yes, if this was a mission critical app, trying to do something
-	 * to recover may make sense, but that is because failure of the app    
-	 * may have other disastrous problems.  Cf runs out of a script
-	 * so is easily enough restarted without any real problems.
-	 * MSW 2001-07-01
-	 */
-	abort();
-    }
-    if(op->more!=NULL)
-	remove_ob(op->more);
+  SET_FLAG(op, FLAG_REMOVED);
 
-    SET_FLAG(op, FLAG_REMOVED);
-
-    /* 
-     * In this case, the object to be removed is in someones
-     * inventory.
-     */
-    if(op->env!=NULL) {
-	if(op->nrof)
-	    sub_weight(op->env, op->weight*op->nrof);
-	else
-	    sub_weight(op->env, op->weight+op->carrying);
-
-	/* NO_FIX_PLAYER is set when a great many changes are being
-	 * made to players inventory.  If set, avoiding the call
-	 * to save cpu time.
-	 */
-	if ((otmp=is_player_inv(op->env))!=NULL && otmp->contr && 
-	    !QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
-	    fix_player(otmp);
-
-	if(op->above!=NULL)
-	    op->above->below=op->below;
-	else
-	    op->env->inv=op->below;
-
-	if(op->below!=NULL)
-	    op->below->above=op->above;
-
-	/* we set up values so that it could be inserted into
-	 * the map, but we don't actually do that - it is up
-	 * to the caller to decide what we want to do.
-	 */
-	op->x=op->env->x,op->y=op->env->y;
-	op->ox=op->x,op->oy=op->y;
-	op->map=op->env->map;
-	op->above=NULL,op->below=NULL;
-	op->env=NULL;
-	return;
-    }
-
-    /* If we get here, we are removing it from a map */
-    if (op->map == NULL) return;
-
-    x = op->x;
-    y = op->y;
-    m = get_map_from_coord(op->map, &x, &y);
-
-    if (!m) {
-	LOG(llevError,"remove_ob called when object was on map but appears to not be within valid coordinates? %s (%d,%d)\n",
-	    op->map->path, op->x, op->y);
-	/* in old days, we used to set x and y to 0 and continue.
-	 * it seems if we get into this case, something is probablye
-	 * screwed up and should be fixed.
-	 */
-	abort();
-    }
-    if (op->map != m) {
-	LOG(llevDebug,"remove_ob: Object not really on map it claimed to be on? %s != %s, %d,%d != %d,%d\n",
-	    op->map->path, m->path, op->x, op->y, x, y);
-    }
-
-    /* Re did the following section of code - it looks like it had
-     * lots of logic for things we no longer care about
-     */
-
-    /* link the object above us */
-    if (op->above)
-	op->above->below=op->below;
+  /*
+   * In this case, the object to be removed is in someones
+   * inventory.
+   */
+  /* TODO try to call a generic inventory weight adjusting function like sub_weight */
+  if(op->env!=NULL) {
+    if(op->nrof)
+      sub_weight(op->env, op->weight*op->nrof);
     else
-	SET_MAP_TOP(m,x,y,op->below); /* we were top, set new top */
-    
-    /* Relink the object below us, if there is one */
-    if(op->below) {
-	op->below->above=op->above;
-    } else {
-	/* Nothing below, which means we need to relink map object for this space 
-	 * use translated coordinates in case some oddness with map tiling is
-	 * evident
-	 */
-	if(GET_MAP_OB(m,x,y)!=op) {
-	    dump_object(op);
-	    LOG(llevError,"remove_ob: GET_MAP_OB does not return object to be removed even though it appears to be on the bottom?\n%s\n", errmsg);
-	    dump_object(GET_MAP_OB(m,x,y));
-	    LOG(llevError,"%s\n",errmsg);
-	}
-	SET_MAP_OB(m,x,y,op->above);  /* goes on above it. */
-    }
-    op->above=NULL;                      
+      sub_weight(op->env, op->weight+op->carrying);
+
+  /* NO_FIX_PLAYER is set when a great many changes are being
+   * made to players inventory.  If set, avoiding the call
+   * to save cpu time.
+   */
+    if ((otmp=get_player_container(op->env))!=NULL && otmp->contr && 
+        !QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
+        fix_player(otmp);
+
+    if(op->above!=NULL)
+        op->above->below=op->below;
+    else
+        op->env->inv=op->below;
+
+    if(op->below!=NULL)
+      op->below->above=op->above;
+
+    /* we set up values so that it could be inserted into
+     * the map, but we don't actually do that - it is up
+     * to the caller to decide what we want to do.
+     */
+    op->x=op->env->x;
+    op->y=op->env->y;
+    op->ox=op->x;
+    op->oy=op->y;
+    op->map=op->env->map;
+    op->above=NULL;
     op->below=NULL;
+    op->env=NULL;
+    return;
+  }
 
-    if (op->map->in_memory == MAP_SAVING)
-	return;
+  /* If we get here, we are removing it from a map */
+  if (op->map == NULL) return;
 
-    tag = op->count;
-    check_walk_off = ! QUERY_FLAG (op, FLAG_NO_APPLY);
-    for(tmp=GET_MAP_OB(m,x,y);tmp!=NULL;tmp=tmp->above) {
-	/* No point updating the players look faces if he is the object
-	 * being removed.
-	 */
+  x = op->x;
+  y = op->y;
+  m = get_map_from_coord(op->map, &x, &y);
 
-	if(tmp->type==PLAYER && tmp!=op) {
-	    /* If a container that the player is currently using somehow gets
-	     * removed (most likely destroyed), update the player view
-	     * appropriately.
-	     */
-	    if (tmp->container==op) {
-		CLEAR_FLAG(op, FLAG_APPLIED);
-		tmp->container=NULL;
-	    }
-	    tmp->contr->socket.update_look=1;
-	}
-	/* See if player moving off should effect something */
-	if (check_walk_off && ((op->move_type & tmp->move_off) &&
-	      (op->move_type & ~tmp->move_off & ~tmp->move_block)==0)) {
+  if (!m) {
+    LOG(llevError,"remove_ob called when object was on map but appears to not be within valid coordinates? %s (%d,%d)\n",
+        op->map->path, op->x, op->y);
+    abort();
+  }
+  if (op->map != m) {
+    LOG(llevError,"remove_ob: Object not really on map it claimed to be on? %s != %s, %d,%d != %d,%d\n",
+        op->map->path, m->path, op->x, op->y, x, y);
+  }
 
-	    move_apply(tmp, op, NULL);
-	    if (was_destroyed (op, tag)) {
-		LOG (llevError, "BUG: remove_ob(): name %s, archname %s destroyed "
-		     "leaving object\n", tmp->name, tmp->arch->name);
-	    }
-	}
+  /* link the object above us */
+  if (op->above)
+    op->above->below=op->below;
+  else
+    SET_MAP_TOP(m,x,y,op->below); /* we were top, set new top */
 
-	/* Eneq(@csd.uu.se): Fixed this to skip tmp->above=tmp */
-
-	if(tmp->above == tmp)
-	    tmp->above = NULL;
-	last=tmp;
+  /* Relink the object below us, if there is one */
+  if(op->below) {
+    op->below->above=op->above;
+  } else {
+  /* Nothing below, which means we need to relink map object for this space 
+   * use translated coordinates in case some oddness with map tiling is
+   * evident
+   */
+    /*TODO is this check really needed?*/
+    if(GET_MAP_OB(m,x,y)!=op) {
+      dump_object(op);
+      LOG(llevError,"remove_ob: GET_MAP_OB does not return object to be removed even though it appears to be on the bottom?\n%s\n", errmsg);
+      dump_object(GET_MAP_OB(m,x,y));
+      LOG(llevError,"%s\n",errmsg);
     }
-    /* last == NULL of there are no objects on this space */
-    if (last==NULL) {
-	/* set P_NEED_UPDATE, otherwise update_position will complain.  In theory,
-	 * we could preserve the flags (GET_MAP_FLAGS), but update_position figures
-	 * those out anyways, and if there are any flags set right now, they won't
-	 * be correct anyways.
-	 */
-	SET_MAP_FLAGS(op->map, op->x, op->y,  P_NEED_UPDATE);
-	update_position(op->map, op->x, op->y);
+    SET_MAP_OB(m,x,y,op->above);  /* goes on above it. */
+  }
+  op->above=NULL;
+  op->below=NULL;
+
+  if (op->map->in_memory == MAP_SAVING)
+    return;
+
+  tag = op->count;
+  check_walk_off = ! QUERY_FLAG (op, FLAG_NO_APPLY);
+  for(tmp=GET_MAP_OB(m,x,y);tmp!=NULL;tmp=tmp->above) {
+    /* No point updating the players look faces if he is the object
+     * being removed.
+     */
+
+    if(tmp->type==PLAYER && tmp!=op) {
+      /* If a container that the player is currently using somehow gets
+       * removed (most likely destroyed), update the player view
+       * appropriately.
+       */
+      if (tmp->container==op) {
+        CLEAR_FLAG(op, FLAG_APPLIED);
+        tmp->container=NULL;
+      }
+      tmp->contr->socket.update_look=1;
     }
-    else
-	update_object(last, UP_OBJ_REMOVE);
+    /* See if player moving off should effect something */
+    if (check_walk_off && ((op->move_type & tmp->move_off) &&
+          (op->move_type & ~tmp->move_off & ~tmp->move_block)==0)) {
 
-    if(QUERY_FLAG(op,FLAG_BLOCKSVIEW)|| (op->glow_radius != 0)) 
-	update_all_los(op->map, op->x, op->y);
+      move_apply(tmp, op, NULL);
+      if (was_destroyed (op, tag)) {
+        LOG (llevError, "BUG: remove_ob(): name %s, archname %s destroyed "
+            "leaving object\n", tmp->name, tmp->arch->name);
+      }
+    }
 
+    /* Eneq(@csd.uu.se): Fixed this to skip tmp->above=tmp */
+
+    if(tmp->above == tmp)
+        tmp->above = NULL;
+    last=tmp;
+  }
+  /* last == NULL or there are no objects on this space */
+  if (last==NULL) {
+    /* set P_NEED_UPDATE, otherwise update_position will complain.  In theory,
+     * we could preserve the flags (GET_MAP_FLAGS), but update_position figures
+     * those out anyways, and if there are any flags set right now, they won't
+     * be correct anyways.
+     */
+    SET_MAP_FLAGS(op->map, op->x, op->y,  P_NEED_UPDATE);
+    update_position(op->map, op->x, op->y);
+  }
+  else
+    update_object(last, UP_OBJ_REMOVE);
+
+  if(QUERY_FLAG(op,FLAG_BLOCKSVIEW)|| (op->glow_radius != 0))
+    update_all_los(op->map, op->x, op->y);
 }
 
 /**
@@ -1475,7 +1480,6 @@ object *merge_ob(object *op, object *top) {
     if (can_merge(op,top))
     {
       top->nrof+=op->nrof;
-/*      CLEAR_FLAG(top,FLAG_STARTEQUIP);*/
       op->weight = 0; /* Don't want any adjustements now */
       remove_ob(op);
       free_object(op);
@@ -1636,7 +1640,10 @@ object *insert_ob_in_map (object *op, mapstruct *m, object *originator, int flag
 	originator->below = op;
     } else {
 	/* If there are other objects, then */
-	if((! (flag & INS_MAP_LOAD)) && ((top=GET_MAP_OB(op->map,op->x,op->y))!=NULL)) {
+        /* This test is incorrect i think, as ins_above_floor_only needs the floor variable
+        if((! (flag & INS_MAP_LOAD)) && ((top=GET_MAP_OB(op->map,op->x,op->y))!=NULL)) {
+        */
+        if(((top=GET_MAP_OB(op->map,op->x,op->y))!=NULL)) {
 	    object *last=NULL;
 	    /*
 	     * If there are multiple objects on this space, we do some trickier handling.
@@ -1860,7 +1867,7 @@ object *decrease_ob_nr (object *op, uint32 i)
 	/* is this object in the players inventory, or sub container
 	 * therein?
 	 */
-        tmp = is_player_inv (op->env);
+        tmp = get_player_container (op->env);
 	/* nope.  Is this a container the player has opened?
 	 * If so, set tmp to that player.
 	 * IMO, searching through all the players will mostly
@@ -1998,7 +2005,7 @@ object *insert_ob_in_ob(object *op,object *where) {
   } else
     add_weight (where, (op->weight+op->carrying));
 
-  otmp=is_player_inv(where);
+  otmp=get_player_container(where);
   if (otmp&&otmp->contr!=NULL) {
     if (!QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
       fix_player(otmp);
@@ -2271,6 +2278,109 @@ void set_cheat(object *op) {
     flag_inv(op, FLAG_WAS_WIZ);
 }
 
+/*
+ * find_multi_free_spot_around() sets hx and hy to the coords to insert a possibly
+ * multi-tile ob at, around gen. Returns 0 for success and -1 for failure. This function
+ * assumes that multi-tile objects are rectangular.
+ */
+int find_multi_free_spot_around(object *ob, object *gen, int *hx, int *hy) {
+    int genx, geny, genx2, geny2, sx, sy, sx2, sy2, ix, iy, nx, ny, i, flag;
+    int freecount=0;
+    
+    if (ob->head)
+        ob = ob->head;
+    
+    get_multi_size(ob, &sx, &sy, &sx2, &sy2);
+    get_multi_size(gen, &genx, &geny, &genx2, &geny2);
+    /*
+     * sx and sy are now the coords of the bottom right corner of ob relative to the head.
+     * genx and geny are now the coords of the bottom right corner of gen relative to the head.
+     * sx2 and sy2 are now the coords of the head of ob relative to the top left corner.
+     * genx2 and geny2 are now the coords of the head of gen relative to the top left corner.
+     */
+
+    sx++;
+    sy++;
+    genx++;
+    geny++;
+    /*
+     * sx, sy, genx, and geny, are now the size of the object, excluding parts left and above
+     * the head.
+     */
+    
+    ix = gen->x - sx - genx2;
+    iy = gen->y - sy - geny2;
+    sx += genx + sx2;
+    sy += geny + sy2;
+    /*
+     * ix and iy are the map coords of the top left square where the head of ob could possibly
+     * be placed. sx and sy are now the size of the square to search for placement of the head
+     * relative to ix and iy.
+     */
+    
+    /*
+     * Loop around the square of possible potitions for the head of ob object:
+     */
+    for(i = 0; i < (sx+sx+sy+sy); i++) {
+        if (i <= sx) {
+            nx = i+ix;
+            ny = iy;
+        } else if (i <= (sx + sy)) {
+            nx = ix + sx;
+            ny = iy + i - sx;
+        } else if (i <= (sx + sy + sx)) {
+            nx = ix + sx - (i - (sx + sy));
+            ny = iy + sy;
+        } else {
+            nx = ix;
+            ny = iy + sy  - (i - (sx + sy + sx));
+        }
+        /* Check if the spot is free. */
+        flag = ob_blocked(ob,gen->map,nx,ny);
+        if (!flag) {
+            freecount++;
+        }
+    }
+    /* If no free spaces, return. */
+    if (!freecount)
+        return -1;
+        
+    /* Choose a random valid position */
+    freecount = RANDOM()%freecount;
+    for(i = 0; i < (sx+sx+sy+sy); i++) {
+        if (i <= sx) {
+            nx = i+ix;
+            ny = iy;
+        } else if (i <= (sx + sy)) {
+            nx = ix + sx;
+            ny = iy + i - sx;
+        } else if (i <= (sx + sy + sx)) {
+            nx = ix + sx - (i - (sx + sy));
+            ny = iy + sy;
+        } else {
+            nx = ix;
+            ny = iy + sy  - (i - (sx + sy + sx));
+        }
+        
+        /* Make sure it's within map. */
+        if (nx < 0 || nx >= MAP_WIDTH(gen->map) ||
+            ny < 0 || ny >= MAP_HEIGHT(gen->map))
+            continue;
+            
+        /* Check if the spot is free.*/
+        flag = ob_blocked(ob,gen->map,nx,ny);
+        if (!flag) {
+            freecount--;
+            if (freecount <= 0) {
+                *hx = nx;
+                *hy = ny;
+                 return 0;
+            }
+        }
+    }
+    return -1;
+}
+
 /**
  * find_free_spot(object, map, x, y, start, stop) will search for
  * a spot at the given map and coordinates which will be able to contain
@@ -2980,3 +3090,83 @@ int item_matched_string(object *pl, object *op, const char *name)
     }
    return 0;
 }
+
+/**
+ * This just ensures specified object has its more parts correctly inserted in map.
+ * The object *must* be on a map.
+ *
+ * Extracted from common/map.c:link_multipart_objects
+ **/
+void fix_multipart_object(object* tmp)
+{
+    archetype	*at;
+    object *op, *last;
+
+    if (!tmp->map)
+    {
+        LOG(llevError, "fix_multipart_object: not on a map!\n");
+        return;
+    }
+    
+    /* already multipart - don't do anything more */
+    if (tmp->head || tmp->more)
+        return;
+
+    /* If there is nothing more to this object, this for loop
+     * won't do anything.
+     */
+    for (at = tmp->arch->more, last=tmp; at != NULL; at=at->more, last=op) {
+        op = arch_to_object(at);
+
+        /* update x,y coordinates */
+        op->x += tmp->x;
+        op->y += tmp->y;
+        op->head = tmp;
+        op->map = tmp->map;
+        last->more = op;
+        if (tmp->name != op->name) {
+            if (op->name) free_string(op->name);
+            op->name = add_string(tmp->name);
+        }
+        if (tmp->title != op->title) {
+            if (op->title) free_string(op->title);
+            op->title = add_string(tmp->title);
+        }
+        /* we could link all the parts onto tmp, and then just
+        * call insert_ob_in_map once, but the effect is the same,
+        * as insert_ob_in_map will call itself with each part, and
+        * the coding is simpler to just to it here with each part.
+        */
+        insert_ob_in_map(op, op->map, tmp,INS_NO_MERGE|INS_ABOVE_FLOOR_ONLY|INS_NO_WALK_ON);
+    } /* for at = tmp->arch->more */
+}
+
+/*
+ * Fills sx and sy with the coords of the bottom right tail relative to the head and
+ * fills hx and hy with the coords of the head tile relative to the top left tile. If
+ * you don't want a value, it's safe to pass NULL in place of the pointer.
+ */
+static void get_multi_size(object *ob, int *sx, int *sy, int *hx, int *hy) {
+    archetype *part;
+    int maxx=0, maxy=0, minx=0, miny=0;
+    if (ob->head)
+        ob = ob->head;
+    *sx = 1;
+    *sy = 1;
+    if (ob->arch->more){
+        for(part = ob->arch; part; part = part->more) {
+            if (part->clone.x > maxx)
+                maxx = part->clone.x;
+            if (part->clone.y > maxy)
+                maxy = part->clone.y;
+            if (part->clone.x < minx)
+                minx = part->clone.x;
+            if (part->clone.y < miny)
+                miny = part->clone.y;
+        }  
+    }
+    if (sx) *sx = maxx;
+    if (sy) *sy = maxy;
+    if (hx) *hx = -minx;
+    if (hy) *hy = -miny;
+}
