@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: region.c,v 1.10 2006/02/09 00:48:36 akirschbaum Exp $";
+ *   "$Id: region.c 5127 2006-11-19 20:15:44Z akirschbaum $";
  */
 
 /*
@@ -228,7 +228,7 @@ void init_regions(void) {
     if(first_region!=NULL) /* Only do this once */
     return;
         
-    sprintf(filename,"%s/%s",settings.datadir,settings.regions);
+    sprintf(filename,"%s/%s/%s",settings.datadir,settings.mapdir,settings.regions);
     LOG(llevDebug,"Reading regions from %s...\n",filename);
     if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
 	LOG(llevError," Can't open regions file %s in init_regions.\n", filename);
@@ -255,7 +255,7 @@ region *get_region_struct(void) {
     if(new==NULL)
         fatal(OUT_OF_MEMORY);
 
-    memset(new, sizeof(region), '\0');
+    memset(new, '\0', sizeof(region));
   
     return new;
 }
@@ -386,5 +386,5 @@ void assign_region_parents(void) {
 	}
 	region_count++;
     }
-    LOG(llevDebug, "Assigned %u regions with %u parents.", region_count, parent_count);
+    LOG(llevDebug, "Assigned %u regions with %u parents.\n", region_count, parent_count);
 }
