@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c,v 1.66 2006/05/21 12:50:17 akirschbaum Exp $";
+ *   "$Id: treasure.c 5295 2007-01-03 21:03:42Z ryo_saeba $";
  */
 
 /*
@@ -38,7 +38,7 @@
 
 /* TREASURE_VERBOSE enables copious output concerning artifact generation */
 /* #define TREASURE_VERBOSE */
- 
+
 #include <global.h>
 #include <treasure.h>
 #include <funcpoint.h>
@@ -47,6 +47,7 @@
 
 static void change_treasure(treasure *t, object *op); /* overrule default values */
 extern char *spell_mapping[];
+int artifact_init;  /* 1 if doing archetypes initialization */
 
 /*
  * Initialize global archtype pointers:
@@ -292,7 +293,7 @@ static void put_treasure (object *op, object *creator, int flags)
         op = insert_ob_in_ob (op, creator);
         if ((flags & GT_APPLY) && QUERY_FLAG (creator, FLAG_MONSTER))
             monster_check_apply(creator, op);
-        if ((flags & GT_UPDATE_INV) && (tmp = is_player_inv (creator)) != NULL)
+        if ((flags & GT_UPDATE_INV) && (tmp = get_player_container (creator)) != NULL)
             esrv_send_item(tmp, op);
     }
 }
@@ -382,6 +383,14 @@ void create_one_treasure(treasurelist *tl, object *op, int flag, int difficulty,
 	    create_one_treasure(tl, op, flag, difficulty, tries);
 	return;
     }
+    if((t->item)&&(flag & GT_ONLY_GOOD)) { /* Generate only good items, damnit !*/
+        if (QUERY_FLAG(&(t->item->clone),FLAG_CURSED)||
+            QUERY_FLAG(&(t->item->clone), FLAG_DAMNED))
+        {
+            create_one_treasure(tl, op, flag, difficulty, tries+1);
+            return;
+        }
+    }
     if((t->item && t->item->clone.invisible != 0) || flag != GT_INVISIBLE) {
 	object *tmp=arch_to_object(t->item);
 	if (!tmp) return;
@@ -513,7 +522,7 @@ int level_for_item(const object *op, int difficulty, int retmult)
 	return mult;
 
     olevel = mult * rndm(0, difficulty) + level;
-    if (olevel > MAXLEVEL) olevel = MAXLEVEL;
+    if (olevel > MAX_SPELLITEM_LEVEL) olevel = MAX_SPELLITEM_LEVEL;
 
     return olevel;
 }
@@ -762,271 +771,320 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 {
     int was_magic = op->magic, num_enchantments=0, save_item_power;
 
-    if(!creator||creator->type==op->type) creator=op; /*safety & to prevent polymorphed
-							* objects giving attributes */ 
+    if(!creator||creator->type==op->type)
+        creator=op; /* safety & to prevent polymorphed objects giving
+                     * attributes
+                     */
 
     /* If we make an artifact, this information will be destroyed */
     save_item_power = op->item_power;
     op->item_power = 0;
 
-    if (op->randomitems && op->type != SPELL) {
-	create_treasure(op->randomitems, op,flags ,difficulty, 0);
-	if (!op->inv) LOG(llevDebug,"fix_generated_item: Unable to generate treasure for %s\n",
-			  op->name);
-	/* So the treasure doesn't get created again */
-	op->randomitems = NULL;
+    if (op->randomitems && op->type != SPELL)
+    {
+        create_treasure(op->randomitems, op,flags ,difficulty, 0);
+        if (!op->inv)
+            LOG(llevDebug,
+                "fix_generated_item: Unable to generate treasure for %s\n",
+                op->name);
+        /* So the treasure doesn't get created again */
+        op->randomitems = NULL;
     }
 
     if (difficulty<1) difficulty=1;
-    if (!(flags & GT_MINIMAL)) {
-	if (op->arch == crown_arch) {
-	    set_magic(difficulty>25?30:difficulty+5, op, max_magic, flags);
-	    num_enchantments = calc_item_power(op, 1);
-	    generate_artifact(op,difficulty);
-	} else {
-	    if(!op->magic && max_magic)
-		set_magic(difficulty,op,max_magic, flags);
-	    num_enchantments = calc_item_power(op, 1);
-	    if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT)) || op->type == HORN ||
-		difficulty >= 999 )
-	    generate_artifact(op, difficulty);
-	}
+    if (!(flags & GT_MINIMAL))
+    {
+        if (op->arch == crown_arch)
+        {
+            set_magic(difficulty>25?30:difficulty+5, op, max_magic, flags);
+            num_enchantments = calc_item_power(op, 1);
+            generate_artifact(op,difficulty);
+        }
+        else
+        {
+            if(!op->magic && max_magic)
+                set_magic(difficulty,op,max_magic, flags);
+            num_enchantments = calc_item_power(op, 1);
+            if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT))
+                || op->type == HORN || difficulty >= 999 )
+                generate_artifact(op, difficulty);
+        }
 
-	/* Object was made an artifact.  Calculate its item_power rating.
-	 * the item_power in the object is what the artfiact adds.
-	 */
-	if (op->title) {
-	    /* if save_item_power is set, then most likely we started with an
-	     * artifact and have added new abilities to it - this is rare, but
-	     * but I have seen things like 'strange rings of fire'.  So just figure
-	     * out the power from the base power plus what this one adds.  Note
-	     * that since item_power is not quite linear, this actually ends up
-	     * being somewhat of a bonus
-	     */
-	    if (save_item_power) {
-		op->item_power = save_item_power + get_power_from_ench(op->item_power);
-	    } else {
-		op->item_power = get_power_from_ench(op->item_power + num_enchantments);
-	    }
-	} else if (save_item_power) {
-	    /* restore the item_power field to the object if we haven't changed it.
-	     * we don't care about num_enchantments - that will basically just
-	     * have calculated some value from the base attributes of the archetype.
-	     */
-	    op->item_power = save_item_power;
-	}
+        /* Object was made an artifact.  Calculate its item_power rating.
+         * the item_power in the object is what the artifact adds.
+         */
+        if (op->title)
+        {
+            /* if save_item_power is set, then most likely we started with an
+             * artifact and have added new abilities to it - this is rare, but
+             * but I have seen things like 'strange rings of fire'.  So just
+             * figure out the power from the base power plus what this one adds.
+             * Note that since item_power is not quite linear, this actually
+             * ends up being somewhat of a bonus.
+             */
+            if (save_item_power)
+                op->item_power = save_item_power +
+                    get_power_from_ench(op->item_power);
+            else
+                op->item_power += get_power_from_ench(num_enchantments);
+        }
+        else if (save_item_power)
+        {
+            /* restore the item_power field to the object if we haven't changed
+             * it. we don't care about num_enchantments - that will basically
+             * just have calculated some value from the base attributes of the
+             * archetype.
+             */
+            op->item_power = save_item_power;
+        }
+        else
+        {
+            /* item_power was zero. This is suspicious, as it may be because it
+             * was never previously calculated. Let's compute a value and see if
+             * it is non-zero. If it indeed is, then assign it as the new
+             * item_power value.
+             * - gros, 21th of July 2006.
+             */
+            op->item_power = calc_item_power(op,0);
+            save_item_power = op->item_power; /* Just in case it would get used
+                                               * again below */
+        }
     }
 
     /* materialtype modifications.  Note we allow this on artifacts. */
 
     set_materialname(op, difficulty, NULL);
 
-    if (flags & GT_MINIMAL) {
-	if (op->type == POTION)
-	    /* Handle healing and magic power potions */
-	    if (op->stats.sp && !op->randomitems) {
-		object *tmp;
-
-		tmp = create_archetype(spell_mapping[op->stats.sp]);
-		insert_ob_in_ob(tmp, op);
-		op->stats.sp=0;
-	    }
+    if (flags & GT_MINIMAL)
+    {
+        if (op->type == POTION)
+        /* Handle healing and magic power potions */
+            if (op->stats.sp && !op->randomitems)
+            {
+                object *tmp;
+                tmp = create_archetype(spell_mapping[op->stats.sp]);
+                insert_ob_in_ob(tmp, op);
+                op->stats.sp=0;
+            }
     }
     else if (!op->title) /* Only modify object if not special */
-	switch(op->type) {
-	    case WEAPON:
-	    case ARMOUR:
-	    case SHIELD:
-	    case HELMET:
-	    case CLOAK:
-		if (QUERY_FLAG(op, FLAG_CURSED) && !(RANDOM()%4))
-		    set_ring_bonus(op, -DICE2);
-		break;
+    {
+        switch(op->type) {
+            case WEAPON:
+            case ARMOUR:
+            case SHIELD:
+            case HELMET:
+            case CLOAK:
+                if (QUERY_FLAG(op, FLAG_CURSED) && !(RANDOM()%4))
+                    set_ring_bonus(op, -DICE2);
+                break;
 
-	    case BRACERS:
-		if (!(RANDOM()%(QUERY_FLAG(op, FLAG_CURSED)?5:20))) {
-		    set_ring_bonus(op,QUERY_FLAG(op, FLAG_CURSED)?-DICE2:DICE2);
-		    if (!QUERY_FLAG(op, FLAG_CURSED))
-			op->value*=3;
-		}
-		break;
+            case BRACERS:
+                if (!(RANDOM()%(QUERY_FLAG(op, FLAG_CURSED)?5:20)))
+                {
+                    set_ring_bonus(op,QUERY_FLAG(op, FLAG_CURSED)?-DICE2:DICE2);
+                    if (!QUERY_FLAG(op, FLAG_CURSED))
+                        op->value*=3;
+                }
+                break;
 
-	    case POTION: {
-		int too_many_tries=0,is_special=0;
+            case POTION:
+            {
+                int too_many_tries=0,is_special=0;
 
-		/* Handle healing and magic power potions */
-		if (op->stats.sp && !op->randomitems) {
-		    object *tmp;
+                /* Handle healing and magic power potions */
+                if (op->stats.sp && !op->randomitems)
+                {
+                    object *tmp;
+                    tmp = create_archetype(spell_mapping[op->stats.sp]);
+                    insert_ob_in_ob(tmp, op);
+                    op->stats.sp=0;
+                }
 
-		    tmp = create_archetype(spell_mapping[op->stats.sp]);
-		    insert_ob_in_ob(tmp, op);
-		    op->stats.sp=0;
-		}
+                while(!(is_special=special_potion(op)) && !op->inv)
+                {
+                    generate_artifact(op,difficulty);
+                    if(too_many_tries++ > 10) break;
+                }
+                /* don't want to change value for healing/magic power potions,
+                 * since the value set on those is already correct.
+                 */
+                if (op->inv && op->randomitems)
+                {
+                    /* value multiplier is same as for scrolls */
+                    op->value=(op->value*op->inv->value);
+                    op->level = op->inv->level/2+ RANDOM()%difficulty
+                        + RANDOM()%difficulty;
+                }
+                else
+                {
+                    FREE_AND_COPY(op->name, "potion");
+                    FREE_AND_COPY(op->name_pl, "potions");
+                }
+                if ( ! (flags & GT_ONLY_GOOD) && RANDOM() % 2)
+                    SET_FLAG(op, FLAG_CURSED);
+                break;
+            }
+            case AMULET:
+                if(op->arch==amulet_arch)
+                    op->value*=5; /* Since it's not just decoration */
+            case RING:
+                if(op->arch==NULL)
+                {
+                    remove_ob(op);
+                    free_object(op);
+                    op=NULL;
+                    break;
+                }
+                if(op->arch!=ring_arch&&op->arch!=amulet_arch)
+                /* It's a special artifact!*/
+                    break;
 
-		while(!(is_special=special_potion(op)) && !op->inv) {
-		    generate_artifact(op,difficulty);
-		    if(too_many_tries++ > 10) break;
-		}
-		/* don't want to change value for healing/magic power potions,
-		 * since the value set on those is already correct.
-		 */
-		if (op->inv && op->randomitems) { 
-		    /* value multiplier is same as for scrolls */
-		    op->value=(op->value*op->inv->value);
-		    op->level = op->inv->level/2+ RANDOM()%difficulty + RANDOM()%difficulty;
-		} else {
-		    FREE_AND_COPY(op->name, "potion");
-		    FREE_AND_COPY(op->name_pl, "potions");
-		}
-		if ( ! (flags & GT_ONLY_GOOD) && RANDOM() % 2)
-		    SET_FLAG(op, FLAG_CURSED);
-		break; 		
-	    }
+                if ( ! (flags & GT_ONLY_GOOD) && ! (RANDOM() % 3))
+                    SET_FLAG(op, FLAG_CURSED);
+                set_ring_bonus(op,QUERY_FLAG(op, FLAG_CURSED)?-DICE2:DICE2);
+                if(op->type!=RING) /* Amulets have only one ability */
+                    break;
+                if(!(RANDOM()%4))
+                {
+                    int d=(RANDOM()%2 || QUERY_FLAG(op, FLAG_CURSED))?-DICE2:DICE2;
+                    if(d>0)
+                        op->value*=3;
+                    set_ring_bonus(op,d);
+                    if(!(RANDOM()%4))
+                    {
+                        int d=(RANDOM()%3 || QUERY_FLAG(op, FLAG_CURSED))?-DICE2:DICE2;
+                        if(d>0)
+                            op->value*=5;
+                        set_ring_bonus(op,d);
+                    }
+                }
+                if(GET_ANIM_ID(op))
+                    SET_ANIMATION(op, RANDOM()%((int) NUM_ANIMATIONS(op)));
+                break;
 
-	    case AMULET:
-		if(op->arch==amulet_arch)
-		    op->value*=5; /* Since it's not just decoration */
+            case BOOK:
+                /* Is it an empty book?, if yes lets make a special
+                 * msg for it, and tailor its properties based on the
+                 * creator and/or map level we found it on.
+                 */
+                if(!op->msg&&RANDOM()%10)
+                {
+                    /* set the book level properly */
+                    if(creator->level==0 || QUERY_FLAG(creator,FLAG_ALIVE))
+                    {
+                        if(op->map&&op->map->difficulty)
+                            op->level=RANDOM()%(op->map->difficulty)
+                                +RANDOM()%10+1;
+                        else
+                            op->level=RANDOM()%20+1;
+                    }
+                    else
+                        op->level=RANDOM()%creator->level;
 
-	    case RING:
-		if(op->arch==NULL) {
-		    remove_ob(op);
-		    free_object(op);
-		    op=NULL;
-		    break;
-		}
-		if(op->arch!=ring_arch&&op->arch!=amulet_arch) /* It's a special artifact!*/
-		    break;
+                    tailor_readable_ob(op,(creator&&creator->stats.sp)?
+                        creator->stats.sp:-1);
+                    /* books w/ info are worth more! */
+                    op->value*=((op->level>10?op->level:(op->level+1)/2)*
+                        ((strlen(op->msg)/250)+1));
+                    /* creator related stuff */
 
-		if ( ! (flags & GT_ONLY_GOOD) && ! (RANDOM() % 3))
-		    SET_FLAG(op, FLAG_CURSED);
-		set_ring_bonus(op,QUERY_FLAG(op, FLAG_CURSED)?-DICE2:DICE2);
-		if(op->type!=RING) /* Amulets have only one ability */
-		    break;
-		if(!(RANDOM()%4)) {
-		    int d=(RANDOM()%2 || QUERY_FLAG(op, FLAG_CURSED))?-DICE2:DICE2;
-		    if(d>0)
-			op->value*=3;
-		    set_ring_bonus(op,d);
-		    if(!(RANDOM()%4)) {
-			int d=(RANDOM()%3 || QUERY_FLAG(op, FLAG_CURSED))?-DICE2:DICE2;
-			if(d>0)
-			    op->value*=5;
-			set_ring_bonus(op,d);
-		    }
-		}
-		if(GET_ANIM_ID(op))
-		    SET_ANIMATION(op, RANDOM()%((int) NUM_ANIMATIONS(op)));
-		break;
+                    /* for library, chained books.  Note that some monsters have
+                     * no_pick set - we don't want to set no pick in that case.
+                     */
+                    if(QUERY_FLAG(creator,FLAG_NO_PICK) &&
+                        !QUERY_FLAG(creator, FLAG_MONSTER))
+                        SET_FLAG(op,FLAG_NO_PICK);
+                    if(creator->slaying&&!op->slaying) /* for check_inv floors */
+                        op->slaying = add_string(creator->slaying);
 
-	    case BOOK:
-		/* Is it an empty book?, if yes lets make a special 
-		 * msg for it, and tailor its properties based on the 
-		 * creator and/or map level we found it on.
-		 */
-		if(!op->msg&&RANDOM()%10) { 
-		    /* set the book level properly */
-		    if(creator->level==0 || QUERY_FLAG(creator,FLAG_ALIVE)) {
-			if(op->map&&op->map->difficulty)
-			    op->level=RANDOM()%(op->map->difficulty)+RANDOM()%10+1;
-			else
-			    op->level=RANDOM()%20+1;
-		    } else 
-			op->level=RANDOM()%creator->level;
+                    /* add exp so reading it gives xp (once)*/
+                    op->stats.exp = op->value>10000?op->value/5:op->value/10;
+                }
+                break;
 
-		    tailor_readable_ob(op,(creator&&creator->stats.sp)?creator->stats.sp:-1);
-		    /* books w/ info are worth more! */
-		    op->value*=((op->level>10?op->level:(op->level+1)/2)*((strlen(op->msg)/250)+1));
-		    /* creator related stuff */
+            case SPELLBOOK:
+                op->value=op->value* op->inv->value;
+                /* add exp so learning gives xp */
+                op->level = op->inv->level;
+                op->stats.exp = op->value;
+                break;
 
-		    /* for library, chained books.  Note that some monsters have no_pick
-		     * set - we don't want to set no pick in that case.
-		     */
-		    if(QUERY_FLAG(creator,FLAG_NO_PICK) &&
-		       !QUERY_FLAG(creator, FLAG_MONSTER))
-			SET_FLAG(op,FLAG_NO_PICK);
-		    if(creator->slaying&&!op->slaying) /* for check_inv floors */
-			op->slaying = add_string(creator->slaying);
+            case WAND:
+                /* nrof in the treasure list is number of charges,
+                 * not number of wands.  So copy that into food (charges),
+                 * and reset nrof.
+                 */
+                op->stats.food=op->inv->nrof;
+                op->nrof=1;
+                /* If the spell changes by level, choose a random level
+                 * for it, and adjust price.  If the spell doesn't
+                 * change by level, just set the wand to the level of
+                 * the spell, and value calculation is simpler.
+                 */
+                if (op->inv->duration_modifier || op->inv->dam_modifier ||
+                    op->inv->range_modifier)
+                {
+                    op->level = level_for_item(op, difficulty, 0);
+                    op->value= op->value* op->inv->value * (op->level +50)/
+                        (op->inv->level + 50);
+                }
+                else
+                {
+                    op->level = op->inv->level;
+                    op->value = op->value * op->inv->value;
+                }
+                break;
 
-		    /* add exp so reading it gives xp (once)*/
-		    op->stats.exp = op->value>10000?op->value/5:op->value/10;
-		}
-		break;
+            case ROD:
+                op->level = level_for_item(op, difficulty, 0);
+                /* Add 50 to both level an divisor to keep prices a little
+                 * more reasonable.  Otherwise, a high level version of a
+                 * low level spell can be worth tons a money (eg, level 20
+                 * rod, level 2 spell = 10 time multiplier).  This way, the
+                 * value are a bit more reasonable.
+                 */
+                op->value= op->value * op->inv->value * (op->level +50) /
+                    (op->inv->level + 50);
+                /* maxhp is used to denote how many 'charges' the rod holds
+                 * before */
+                if (op->stats.maxhp)
+                    op->stats.maxhp *= MAX(op->inv->stats.sp, op->inv->stats.grace);
+                else
+                    op->stats.maxhp = 2 * MAX(op->inv->stats.sp, op->inv->stats.grace);
 
-	    case SPELLBOOK:
-		op->value=op->value* op->inv->value;
-		/* add exp so learning gives xp */
-		op->level = op->inv->level;
-		op->stats.exp = op->value;
-		break;
+                op->stats.hp = op->stats.maxhp;
+                break;
 
-	    case WAND:
-		/* nrof in the treasure list is number of charges,
-		 * not number of wands.  So copy that into food (charges),
-		 * and reset nrof.
-		 */
-		op->stats.food=op->inv->nrof;
-		op->nrof=1;
-		/* If the spell changes by level, choose a random level
-		 * for it, and adjust price.  If the spell doesn't
-		 * change by level, just set the wand to the level of
-		 * the spell, and value calculation is simpler.
-		 */
-		if (op->inv->duration_modifier || op->inv->dam_modifier ||
-		  op->inv->range_modifier) {
-		    op->level = level_for_item(op, difficulty, 0);
-		    op->value= op->value* op->inv->value * (op->level +50)/ 
-			(op->inv->level + 50);
-		}
-		else {
-		    op->level = op->inv->level;
-		    op->value = op->value * op->inv->value;
-		}
-		break;
+            case SCROLL:
+                op->level = level_for_item(op, difficulty, 0);
+                op->value= op->value * op->inv->value * (op->level +50) /
+                    (op->inv->level + 50);
+                /* add exp so reading them properly gives xp */
+                op->stats.exp = op->value/5;
+                op->nrof = op->inv->nrof;
+                break;
 
-	    case ROD:
-		op->level = level_for_item(op, difficulty, 0);
-		/* Add 50 to both level an divisor to keep prices a little more
-		 * reasonable.  Otherwise, a high level version of a low level
-		 * spell can be worth tons a money (eg, level 20 rod, level 2 spell =
-		 * 10 time multiplier).  This way, the value are a bit more reasonable.
-		 */
-		op->value= op->value * op->inv->value * (op->level +50) / (op->inv->level + 50);
-		/* maxhp is used to denote how many 'charges' the rod holds before */
-		if (op->stats.maxhp)
-		    op->stats.maxhp *= MAX(op->inv->stats.sp, op->inv->stats.grace);
-		else
-		    op->stats.maxhp = 2 * MAX(op->inv->stats.sp, op->inv->stats.grace);
+            case RUNE:
+                trap_adjust(op,difficulty);
+                break;
 
-		op->stats.hp = op->stats.maxhp;
-		break;
-
-	    case SCROLL:
-		op->level = level_for_item(op, difficulty, 0);
-		op->value= op->value * op->inv->value * (op->level +50) / (op->inv->level + 50);
-		/* add exp so reading them properly gives xp */ 
-		op->stats.exp = op->value/5;
-		op->nrof = op->inv->nrof;
-		break;
-
-	    case RUNE:
-		trap_adjust(op,difficulty);
-		break;
-		
-		case TRAP:
-		trap_adjust(op,difficulty);
-		break;
-	} /* switch type */
-
-    if (flags & GT_STARTEQUIP) {
-	if (op->nrof < 2 && op->type != CONTAINER
-	    && op->type != MONEY && ! QUERY_FLAG (op, FLAG_IS_THROWN))
-	    SET_FLAG (op, FLAG_STARTEQUIP);
-	else if (op->type != MONEY)
-	    op->value = 0;
+            case TRAP:
+                trap_adjust(op,difficulty);
+                break;
+        } /* switch type */
+    }
+    if (flags & GT_STARTEQUIP)
+    {
+        if (op->nrof < 2 && op->type != CONTAINER
+            && op->type != MONEY && ! QUERY_FLAG (op, FLAG_IS_THROWN))
+            SET_FLAG (op, FLAG_STARTEQUIP);
+        else if (op->type != MONEY)
+            op->value = 0;
     }
 
     if ( ! (flags & GT_ENVIRONMENT))
-	fix_flesh_item (op, creator);
+        fix_flesh_item (op, creator);
 }
 
 /*
@@ -1201,6 +1259,8 @@ void init_artifacts(void) {
     if (has_been_inited) return;
     else has_been_inited = 1;
 
+    artifact_init = 1;
+    
     sprintf(filename, "%s/artifacts", settings.datadir);
     LOG(llevDebug, "Reading artifacts from %s...",filename);
     if ((fp = open_and_uncompress(filename, 0, &comp)) == NULL) {
@@ -1276,6 +1336,7 @@ void init_artifacts(void) {
     }
 
     LOG(llevDebug,"done.\n");
+    artifact_init = 0;
 }
 
 
@@ -1285,7 +1346,7 @@ void init_artifacts(void) {
  */
 
 void add_abilities(object *op, object *change) {
-    int i,j, tmp;
+    int i, tmp;
 
     if (change->face != blank_face) {
 #ifdef TREASURE_VERBOSE
@@ -1445,59 +1506,6 @@ void add_abilities(object *op, object *change) {
 	    free_string(op->msg);
 	op->msg = add_refcount(change->msg);
     }
-    /* GROS: Added support for event_... in artifact file */
-    for(j=0;j<NR_EVENTS;j++) {
-	event *evt;
-	event *evt2;
-	event *evtn;
-	event *evtp;
-
-	evt = find_event(change,j);
-	evt2= find_event(op,j);
-
-	if ((evt) && (evt->hook)) {
-	    if ((evt2)&&(evt2->hook)) {
-		free_string(evt2->hook);
-		free_string(evt2->plugin);
-		free_string(evt2->options);
-		evtp = NULL;
-		evtn = evt2->next;
-		if (evt2 == op->events) {
-		    free(evt2);
-		    op->events = evtn;
-		}
-		else {
-		    evtp = op->events;
-		    while (evtp->next != evt2)
-			evtp = evtp->next;
-		    free(evt2);
-		    evtp->next = evtn;
-		}
-	    }
-	    else if (evt2 == NULL) {
-		if (op->events == NULL) {
-		    evt2 = (event *)malloc(sizeof(event));
-		    op->events = evt2;
-		}
-		else {
-		    evtp = op->events;
-		    while (evtp->next != NULL)
-			evtp = evtp->next;
-		    evtp->next = (event *)malloc(sizeof(event));
-		    evt2 = evtp->next;
-		}
-	    }
-	    evt2->next = NULL;
-	    evt2->hook = add_refcount(evt->hook);
-	    evt2->plugin = add_refcount(evt->plugin);
-	    evt2->type = j;
-
-	    if (evt->options)
-		evt2->options = add_refcount(evt->options);
-	    else
-		evt2->options = NULL;
-	}
-    }
 }
 
 static int legal_artifact_combination(object *op, artifact *art) {
@@ -1626,13 +1634,36 @@ void fix_flesh_item(object *item, object *donor) {
     char tmpbuf[MAX_BUF];
     int i;
 
-    if(item->type==FLESH && donor) {
+    if(item->type==FLESH && donor && QUERY_FLAG(donor, FLAG_MONSTER)) {
 	/* change the name */
 	sprintf(tmpbuf,"%s's %s",donor->name,item->name);
 	FREE_AND_COPY(item->name, tmpbuf);
 	sprintf(tmpbuf,"%s's %s",donor->name,item->name_pl);
 	FREE_AND_COPY(item->name_pl, tmpbuf);
 
+        /* store original arch in other_arch */
+        if(!item->other_arch) {
+	    if (!donor->arch->reference_count) {
+		item->other_arch = donor->arch;
+	    }
+	    else {
+		/* If dealing with custom monsters, other_arch still needs to
+		 * point back to the original.  Otherwise what happens
+		 * is that other_arch points at the custom archetype, but
+		 * that can be freed.  Reference count doesn't work because
+		 * the loader will not be able to resolve the other_arch at
+		 * load time (server may has restarted, etc.)
+		 */
+		archetype* original = find_archetype(donor->arch->name);
+
+		if (original) item->other_arch = original;
+		else {
+		    LOG(llevError, "could not find original archetype %s for custom monster!", donor->arch->name);
+		    abort();
+		}
+	    }
+	}
+
 	/* weight is FLESH weight/100 * donor */
 	if((item->weight = (signed long) (((double)item->weight/(double)100.0) * (double)donor->weight))==0)
 		item->weight=1;
@@ -1649,8 +1680,9 @@ void fix_flesh_item(object *item, object *donor) {
 	for (i=0; i<NROFATTACKS; i++)
 	    item->resist[i] = donor->resist[i]/2;
 	
-	/* item inherits donor's level (important for quezals) */
+	/* item inherits donor's level and exp (important for dragons) */
 	item->level = donor->level;
+        item->stats.exp = donor->stats.exp;
 	
 	/* if donor has some attacktypes, the flesh is poisonous */
 	if(donor->attacktype&AT_POISON)
