@@ -9,11 +9,11 @@ void fix_auto_apply(mapstruct * m)
 {
 }
 
-void new_draw_info(int a, int b, object *ob, const char *txt)
+void new_draw_info(int a, int b, const object *ob, const char *txt)
     {
     fprintf(logfile, "%s\n", txt);
     }
-void new_info_map(int color, mapstruct *map, const char *str)
+void new_info_map(int color, const mapstruct *map, const char *str)
     {
     fprintf(logfile, "new_info_map: %s\n", str);
     }
@@ -74,6 +74,10 @@ object* find_skill_by_number( object* ob, int x )
 void esrv_del_item(player *pl, int tag)
     {
     }
+
+void esrv_update_spells(player *pl)
+    {
+    }
  
 void monster_check_apply( object* ob, object* obt )
     {
@@ -81,4 +85,14 @@ void monster_check_apply( object* ob, object* obt )
  
 void trap_adjust( object* ob, int x )
     {
-    }
\ No newline at end of file
+    }
+    
+int execute_event(object* op, int eventcode, object* activator, object* third, const char* message, int fix)
+    {
+    return 0;
+    }
+
+int execute_global_event(int eventcode, ...)
+    {
+    return 0;
+    }
