@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h,v 1.109 2006/05/21 13:44:42 akirschbaum Exp $";
+ *   "$Id: define.h 5544 2007-02-15 19:44:29Z ryo_saeba $";
  */
 
 /*
@@ -378,6 +378,8 @@ extern typedata ItemTypes[];
 
 #define PU_NOT_CURSED		0x01000000
 #define PU_JEWELS		0x02000000
+#define PU_FLESH		0x04000000
+
 
 
 /* Instead of using arbitrary constants for indexing the
@@ -440,9 +442,11 @@ extern typedata ItemTypes[];
 	(op->type == ARROW || op->type == BOW || op->type == WEAPON)
 
 #define IS_ARMOR(op) \
-	(op->type == ARMOUR || op->type == SHIELD || op->type == HELMET || \
-	 op->type == CLOAK || op->type == BOOTS || op->type == GLOVES || \
-	 op->type == BRACERS || op->type == GIRDLE)
+	(op->type == ARMOUR || op->type == HELMET || \
+	 op->type == BOOTS || op->type == GLOVES)
+
+#define IS_SHIELD(op) \
+	(op->type == SHIELD)
 
 #define IS_LIVE(op) \
 	((op->type == PLAYER || QUERY_FLAG(op, FLAG_MONSTER) || \
@@ -811,6 +815,9 @@ typedef unsigned char	MoveType;
 #define ST_GET_PASSWORD 7
 #define ST_CONFIRM_PASSWORD     8
 #define ST_GET_PARTY_PASSWORD 10
+#define ST_CHANGE_PASSWORD_OLD      11  /* Player is entering old password to change password */
+#define ST_CHANGE_PASSWORD_NEW      12  /* Player is entering new password */
+#define ST_CHANGE_PASSWORD_CONFIRM  13  /* Player is confirming new password */
 
 #define BLANK_FACE_NAME "blank.111"
 #define EMPTY_FACE_NAME "empty.111"
@@ -921,8 +928,10 @@ enum apply_flag {
   /* Optional flags, for bitwise or with a basic flag */
     AP_NO_MERGE			= 16,
     AP_IGNORE_CURSE		= 32,
-    AP_PRINT			= 64	/* Print what to do, don't actually do it */
+    AP_PRINT			= 64,	/* Print what to do, don't actually do it */
 					/* Note this is supported in all the functions */
+    AP_NOPRINT			= 128	/* Don't print messages - caller will do that */
+					/* may be some that still print */
 
 };
 
@@ -1020,103 +1029,4 @@ enum apply_flag {
  if (size__!=strlen(buf2__)) LOG(llevError, "Error, bad length for %s\n",buf2__);}*/
 
  
- 
- 
- 
- 
- 
-/*
- * Warning!
- * If you add message types here, don't forget
- * to keep the client up to date too!
- */
- 
-  
-/* message types */
-#define MSG_TYPE_BOOK            1
-#define MSG_TYPE_CARD            2
-#define MSG_TYPE_PAPER           3
-#define MSG_TYPE_SIGN            4
-#define MSG_TYPE_MONUMENT        5
-#define MSG_TYPE_SCRIPTED_DIALOG 6
-#define MSG_TYPE_MOTD            7
-#define MSG_TYPE_ADMIN           8
-#define MSG_TYPE_LAST            9
-
-#define MSG_SUBTYPE_NONE         0
-
-/* book messages subtypes */
-#define MSG_TYPE_BOOK_CLASP_1    1
-#define MSG_TYPE_BOOK_CLASP_2    2
-#define MSG_TYPE_BOOK_ELEGANT_1  3
-#define MSG_TYPE_BOOK_ELEGANT_2  4
-#define MSG_TYPE_BOOK_QUARTO_1   5
-#define MSG_TYPE_BOOK_QUARTO_2   6
-#define MSG_TYPE_BOOK_SPELL_EVOKER    7
-#define MSG_TYPE_BOOK_SPELL_PRAYER    8
-#define MSG_TYPE_BOOK_SPELL_PYRO      9
-#define MSG_TYPE_BOOK_SPELL_SORCERER  10
-#define MSG_TYPE_BOOK_SPELL_SUMMONER  11
-
-/* card messages subtypes*/
-#define MSG_TYPE_CARD_SIMPLE_1    1
-#define MSG_TYPE_CARD_SIMPLE_2    2
-#define MSG_TYPE_CARD_SIMPLE_3    3
-#define MSG_TYPE_CARD_ELEGANT_1   4
-#define MSG_TYPE_CARD_ELEGANT_2   5
-#define MSG_TYPE_CARD_ELEGANT_3   6
-#define MSG_TYPE_CARD_STRANGE_1   7
-#define MSG_TYPE_CARD_STRANGE_2   8
-#define MSG_TYPE_CARD_STRANGE_3   9
-#define MSG_TYPE_CARD_MONEY_1     10
-#define MSG_TYPE_CARD_MONEY_2     11
-#define MSG_TYPE_CARD_MONEY_3     12
-
-/* Paper messages subtypes */
-#define MSG_TYPE_PAPER_NOTE_1       1
-#define MSG_TYPE_PAPER_NOTE_2       2
-#define MSG_TYPE_PAPER_NOTE_3       3
-#define MSG_TYPE_PAPER_LETTER_OLD_1 4
-#define MSG_TYPE_PAPER_LETTER_OLD_2 5
-#define MSG_TYPE_PAPER_LETTER_NEW_1 6
-#define MSG_TYPE_PAPER_LETTER_NEW_2 7
-#define MSG_TYPE_PAPER_ENVELOPE_1   8
-#define MSG_TYPE_PAPER_ENVELOPE_2   9
-#define MSG_TYPE_PAPER_SCROLL_OLD_1 10
-#define MSG_TYPE_PAPER_SCROLL_OLD_2 11
-#define MSG_TYPE_PAPER_SCROLL_NEW_1 12
-#define MSG_TYPE_PAPER_SCROLL_NEW_2 13
-#define MSG_TYPE_PAPER_SCROLL_MAGIC 14
-
-/* road signs messages subtypes */
-#define MSG_TYPE_SIGN_BASIC         1
-#define MSG_TYPE_SIGN_DIR_LEFT      2
-#define MSG_TYPE_SIGN_DIR_RIGHT     3
-#define MSG_TYPE_SIGN_DIR_BOTH      4
-
-/* stones and monument messages */
-#define MSG_TYPE_MONUMENT_STONE_1      1
-#define MSG_TYPE_MONUMENT_STONE_2      2
-#define MSG_TYPE_MONUMENT_STONE_3      3
-#define MSG_TYPE_MONUMENT_STATUE_1     4
-#define MSG_TYPE_MONUMENT_STATUE_2     5
-#define MSG_TYPE_MONUMENT_STATUE_3     6
-#define MSG_TYPE_MONUMENT_GRAVESTONE_1 7
-#define MSG_TYPE_MONUMENT_GRAVESTONE_2 8
-#define MSG_TYPE_MONUMENT_GRAVESTONE_3 9
-#define MSG_TYPE_MONUMENT_WALL_1       10
-#define MSG_TYPE_MONUMENT_WALL_2       11
-#define MSG_TYPE_MONUMENT_WALL_3       12
-
-/*some readable flags*/
-
-/* dialog messsage */
-#define MSG_TYPE_DIALOG_NPC            1 /*A message from the npc*/
-#define MSG_TYPE_DIALOG_ANSWER         2 /*One of possible answers*/
-#define MSG_TYPE_DIALOG_ANSWER_COUNT   3 /*Number of possible answers*/
-
-/* admin messages */
-#define MSG_TYPE_ADMIN_RULES           1
-#define MSG_TYPE_ADMIN_NEWS            2
-
 #endif /* DEFINE_H */
