@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.67 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: global.h 5555 2007-02-16 00:06:50Z ryo_saeba $";
  */
 
 /*
@@ -220,6 +220,7 @@ EXTERN long nrofallowedstr;		/* Only used in malloc_info() */
 EXTERN archetype *empty_archetype;	/* Nice to have fast access to it */
 EXTERN archetype *map_archeytpe;
 EXTERN char first_map_path[MAX_BUF];	/* The start-level */
+EXTERN char first_map_ext_path[MAX_BUF]; /* Path used for per-race start maps */
 
 EXTERN char errmsg[HUGE_BUF];
 EXTERN long ob_count;
@@ -391,6 +392,8 @@ typedef struct Settings {
     int     armor_speed_improvement;    /* Speed improvement */
     uint8   armor_speed_linear;         /* If 1, speed improvement is linear, else exponantiel */
     uint8   no_player_stealing;     /* If 1, can not steal from other players */
+    uint8   create_home_portals;     /* If 1, can create portals in unique maps (apartments) */
+    int     allow_denied_spells_writing; /* If set, players can write spells they can't cast. */
 } Settings;
 
 extern Settings settings;
