@@ -1,3 +1,6 @@
+#ifndef GOD_H
+#define GOD_H
+
 typedef struct glnk { /* Used to link together the gods */
   const char *name;       		/* name of this god */
   struct archt *arch;      	/* pointer to the archetype of this god */
@@ -6,3 +9,4 @@ typedef struct glnk { /* Used to link together the gods */
   struct glnk *next;    
 } godlink; 
 
+#endif /* GOD_H */
