@@ -179,7 +179,7 @@ extern int on_same_map(const object *op1, const object *op2);
 extern int can_merge(object *ob1, object *ob2);
 extern signed long sum_weight(object *op);
 extern object *object_get_env_recursive(object *op);
-extern object *is_player_inv(object *op);
+extern object *get_player_container(object *op);
 extern void dump_object(object *op);
 extern void dump_all_objects(void);
 extern object *find_object(tag_t i);
@@ -220,6 +220,7 @@ extern object *present_arch_in_ob(const archetype *at, const object *op);
 extern void flag_inv(object *op, int flag);
 extern void unflag_inv(object *op, int flag);
 extern void set_cheat(object *op);
+extern int find_multi_free_spot_around(object *ob, object *gen, int *hx, int *hy);
 extern int find_free_spot(const object *ob, mapstruct *m, int x, int y, int start, int stop);
 extern int find_first_free_spot(const object *ob, mapstruct *m, int x, int y);
 extern void get_search_arr(int *search_arr);
@@ -238,6 +239,7 @@ extern const char *get_ob_key_value(const object *op, const char *const key);
 extern int set_ob_key_value(object *op, const char *key, const char *value, int add_key);
 extern object *find_best_weapon_used_match(object *pl, const char *params);
 extern int item_matched_string(object *pl, object *op, const char *name);
+extern void fix_multipart_object(object* tmp);
 /* path.c */
 extern char *path_combine(const char *src, const char *dst);
 extern void path_normalize(char *path);
@@ -261,6 +263,7 @@ extern void make_path_to_file(char *filename);
 extern void free_player(player *pl);
 extern int atnr_is_dragon_enabled(int attacknr);
 extern int is_dragon_pl(const object *op);
+extern client_spell* get_client_spell_state(player* pl, object* spell);
 /* quest.c */
 extern int quest_is_quest_marker(const object *marker, int task);
 extern int quest_is_in_progress(const object *marker, int task);
@@ -394,5 +397,3 @@ extern int set_variable(object *op, char *buf);
 extern void init_vars(void);
 extern char *get_ob_diff(object *op, object *op2);
 extern void save_object(FILE *fp, object *op, int flag);
-extern void insert_event(object *op, int etype, char *ehook, char *eplug, char *eoptions);
-extern event *find_event(object *op, int etype);
