@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_h =
- *   "$Id: living.h,v 1.13 2005/08/15 22:40:13 akirschbaum Exp $";
+ *   "$Id: living.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -38,7 +38,7 @@
 #define POW 6
 #define NUM_STATS 7
 
-#define MAXLEVEL      110
+#define MAXLEVEL      115
 
 /* Changed from NO_STAT to NO_STAT_VAL to fix conlfict on
  * AIX systems
@@ -78,5 +78,4 @@ typedef struct liv { /* Mostly used by "alive" objects */
   sint8		luck;		/* Affects thaco and ac from time to time */
 } living;
 
-
-#endif
+#endif /* LIVING_H */
