@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newserver_h =
- *   "$Id: newserver.h,v 1.40 2006/05/20 12:36:48 ryo_saeba Exp $";
+ *   "$Id: newserver.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -59,12 +59,6 @@ struct map_cell_struct {
 #define MAX_CLIENT_X (MAP_CLIENT_X + MAX_HEAD_OFFSET)
 #define MAX_CLIENT_Y (MAP_CLIENT_Y + MAX_HEAD_OFFSET)
 
-/* How much the x,y coordinates in the map2 are off from
- * actual upper left corner.  Necessary for light sources
- * that may be off the edge of the visible map.
- */
-#define MAP2_COORD_OFFSET   15
-
 struct Map {
   struct map_cell_struct cells[MAX_CLIENT_X][MAX_CLIENT_Y];
 };
@@ -166,31 +160,6 @@ typedef struct socket_struct {
 #define NS_FACESENT_FACE	0x1
 #define NS_FACESENT_SMOOTH	0x2
 
-/* Constants in the form EMI_ is for extended map infos.
- * Even if the client select the additionnal infos it wants
- * on the map, there may exist cases where this whole info
- * is not given in one buch but in separate bunches. This 
- * is done performance reasons (imagine some info related to
- * a visible object and another info related to a 4 square
- * width and height area). At the begin of an extended info packet
- * is a bit field. A bit is activated for each extended info
- * present in the data 
- */
-/* Meanings:
- * EMI_NOREDRAW  Take extended infos into account but don't redraw,
- *               some additionnal datas will follow in a new packet
- * EMI_SMOOTH    Datas about smoothing  
- */ 
-#define EMI_NOREDRAW        0x01  
-#define EMI_SMOOTH          0x02
-
-/* this last one says the bitfield continue un next byte
- * There may be several on contiguous bytes. So there is 7
- * actual bits used per byte, and the number of bytes
- * is not fixed in protocol
- */
-#define EMI_HASMOREBITS     0x80
-
 #define FACE_TYPES  1
 #define PNG_FACE_INDEX	0
 
@@ -207,6 +176,4 @@ extern Socket_Info socket_info;
 #define VERSION_SC 1027
 #define VERSION_INFO "Crossfire Server"
 
-
-
-#endif
+#endif /* NEWSERVER_H */
