@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: object.h,v 1.48 2006/05/30 07:16:23 mwedel Exp $";
+ *   "$Id: object.h 5159 2006-12-09 18:12:21Z ryo_saeba $";
  */
 
 /*
@@ -43,15 +43,6 @@ typedef struct body_locations_struct {
 
 extern body_locations_struct body_locations[NUM_BODY_LOCATIONS];
 
-typedef struct _event
-{
-    int type;
-    const char *hook;
-    const char *plugin;
-    const char *options;
-    struct _event *next;
-} event;
-
 /*
  * Each object (this also means archetypes!) could have a few of these
  * "dangling" from it; this could also end up containing 'parse errors'.
@@ -240,17 +231,18 @@ typedef struct obj {
 
     MoveType   move_type;	/* Type of movement this object uses */
     MoveType   move_block;	/* What movement types this blocks */
-    MoveType   move_allow;	/* What movement types explicitly allowd */
+    MoveType   move_allow;	/* What movement types explicitly allowed */
     MoveType   move_on;		/* Move types affected moving on to this space */
     MoveType   move_off;	/* Move types affected moving off this space */
     MoveType   move_slow;	/* Movement types this slows down */
     float   move_slow_penalty;	/* How much this slows down the object */
 
-    event   *events;
-
     const char  *custom_name; /* Custom name assigned by player */
     key_value *key_values; /* Fields not explictly known by the loader. */
-
+    
+    uint8       no_save;       /* This field indicates that the object should never */
+                                /* be saved even for map swapout. Not handled by the */
+                                /* loading or saving code. */
 } object;
 
 typedef struct oblnk { /* Used to link together several objects */
@@ -284,6 +276,7 @@ typedef struct archt {
     sint8   tail_x, tail_y; /* Where the lower right most portion of the object is
 			     * in comparison to the head.
 			     */
+    int reference_count; /**< How many times this temporary archetype is used. If 0, "permanent" archetype. */
 } archetype;
 
 extern object *objects;
@@ -347,4 +340,4 @@ extern int nroffreeobjects;
 #define ARCH_DEPLETION          "depletion"
 #define ARCH_SYMPTOM            "symptom"
 
-#endif
+#endif /* OBJECT_H */
