@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h,v 1.47 2006/03/18 15:05:32 ryo_saeba Exp $";
+ *   "$Id: player.h 5149 2006-12-02 19:17:49Z ryo_saeba $";
  */
 
 /*
@@ -26,6 +26,9 @@
     The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
+#ifndef PLAYER_H
+#define PLAYER_H
+
 #define NUM_OUTPUT_BUFS	5
 typedef struct {
     const char *buf;		/* Actual string pointer */
@@ -89,6 +92,17 @@ typedef enum unapplymode {
 			    /* to control which of these will be unapplied. */
 } unapplymode;
 
+/**
+ * This stores, for a spell a player knows, the last sp/gr/dam information sent to client.
+ */
+typedef struct client_spell {
+    object *spell;
+    sint16 last_sp;
+    sint16 last_grace;
+    sint16 last_dam;
+    struct client_spell *next;
+} client_spell;
+
 /* not really the player, but tied pretty closely */  
 typedef struct party_struct {
     char * partyleader;
@@ -187,6 +201,7 @@ typedef struct pl {
     char	write_buf[MAX_BUF]; /* Holds arbitrary input from client */
     char	input_buf[MAX_BUF]; /* Holds command to run */
     char	password[16];	    /* 2 (seed) + 11 (crypted) + 1 (EOS) + 2 (safety) = 16 */
+    char	new_password[16];   /* 2 (seed) + 11 (crypted) + 1 (EOS) + 2 (safety) = 16 */
 
 #ifdef SAVE_INTERVAL
     time_t	last_save_time;
@@ -207,7 +222,10 @@ typedef struct pl {
     object	*mark;		    /* marked object */
     uint32	mark_count;	    /* count of mark object */
     object	*transport;	    /* transport the player is in */
+    client_spell* spell_state; /**< Spell information sent to client */
     /* Special DM fields */
     tag_t*  stack_items;    /* Item stack for patch/dump/... commands */
     int     stack_position; /* Current stack position, 0 for no item */
 } player;
+
+#endif /* PLAYER_H */
