@@ -7,8 +7,8 @@
 /* Headers needed.                                                           */
 /*****************************************************************************/
 
-#ifndef PLUGIN_H_
-#define PLUGIN_H_
+#ifndef PLUGIN_H
+#define PLUGIN_H
 
 /*****************************************************************************/
 /* This one does not exist under Win32.                                      */
@@ -66,26 +66,29 @@
 #define EVENT_TRIGGER  10 /* Button pushed, lever pulled, etc.               */
 #define EVENT_CLOSE    11 /* Container closed.                               */
 #define EVENT_TIMER    12 /* Timer connected triggered it.                   */
+#define EVENT_DESTROY  13 /* Object destroyed (includes map reset/swapout)   */
 /*****************************************************************************/
 /* Global events. Those are never linked to a specific object.               */
 /*****************************************************************************/
-#define EVENT_BORN     13 /* A new character has been created.               */
-#define EVENT_CLOCK    14 /* Global time event.                              */
-#define EVENT_CRASH    15 /* Triggered when the server crashes. Not recursive*/
-#define EVENT_PLAYER_DEATH  16 /* Global Death event                         */
-#define EVENT_GKILL    17 /* Triggered when anything got killed by anyone.   */
-#define EVENT_LOGIN    18 /* Player login.                                   */
-#define EVENT_LOGOUT   19 /* Player logout.                                  */
-#define EVENT_MAPENTER 20 /* A player entered a map.                         */
-#define EVENT_MAPLEAVE 21 /* A player left a map.                            */
-#define EVENT_MAPRESET 22 /* A map is resetting.                             */
-#define EVENT_REMOVE   23 /* A Player character has been removed.            */
-#define EVENT_SHOUT    24 /* A player 'shout' something.                     */
-#define EVENT_TELL     25 /* A player 'tell' something.						 */
-#define EVENT_MUZZLE   26 /* A player was Muzzled (no_shout set).			 */
-#define EVENT_KICK     27 /* A player was Kicked by a DM                     */
+#define EVENT_BORN     14 /* A new character has been created.               */
+#define EVENT_CLOCK    15 /* Global time event.                              */
+#define EVENT_CRASH    16 /* Triggered when the server crashes. Not recursive*/
+#define EVENT_PLAYER_DEATH  17 /* Global Death event                         */
+#define EVENT_GKILL    18 /* Triggered when anything got killed by anyone.   */
+#define EVENT_LOGIN    19 /* Player login.                                   */
+#define EVENT_LOGOUT   20 /* Player logout.                                  */
+#define EVENT_MAPENTER 21 /* A player entered a map.                         */
+#define EVENT_MAPLEAVE 22 /* A player left a map.                            */
+#define EVENT_MAPRESET 23 /* A map is resetting.                             */
+#define EVENT_REMOVE   24 /* A Player character has been removed.            */
+#define EVENT_SHOUT    25 /* A player 'shout' something.                     */
+#define EVENT_TELL     26 /* A player 'tell' something.                      */
+#define EVENT_MUZZLE   27 /* A player was Muzzled (no_shout set).            */
+#define EVENT_KICK     28 /* A player was Kicked by a DM                     */
+#define EVENT_MAPUNLOAD     29 /* A map is freed (includes swapping out)     */
+#define EVENT_MAPLOAD       30 /* A map is loaded                            */
 
-#define NR_EVENTS 28
+#define NR_EVENTS 31
 
 #include <stdarg.h>
 
@@ -104,6 +107,8 @@
 #define CFAPI_PPARTY  12
 #define CFAPI_PREGION 13
 #define CFAPI_INT16   14
+#define CFAPI_TIME    15
+#define CFAPI_SINT64  16
 
 typedef void* (*f_plug_api) (int* type, ...);
 typedef int   (*f_plug_postinit) (void);
@@ -207,7 +212,7 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_OBJECT_PROP_CHOSEN_SKILL      57
 #define CFAPI_OBJECT_PROP_HIDDEN            58
 #define CFAPI_OBJECT_PROP_MOVE_STATUS       59
-#define CFAPI_OBJECT_PROP_MOVE_TYPE         60
+#define CFAPI_OBJECT_PROP_ATTACK_MOVEMENT   60
 #define CFAPI_OBJECT_PROP_SPELL_ITEM        61
 #define CFAPI_OBJECT_PROP_EXP_MULTIPLIER    62
 #define CFAPI_OBJECT_PROP_ARCHETYPE         63
@@ -248,9 +253,21 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_OBJECT_PROP_INVISIBLE         98
 #define CFAPI_OBJECT_PROP_FACE              99
 #define CFAPI_OBJECT_ANIMATION              100
+#define CFAPI_OBJECT_PROP_NO_SAVE           101
+#define CFAPI_OBJECT_PROP_MOVE_TYPE         102
+#define CFAPI_OBJECT_PROP_MOVE_BLOCK        103
+#define CFAPI_OBJECT_PROP_MOVE_ALLOW        104
+#define CFAPI_OBJECT_PROP_MOVE_ON           105
+#define CFAPI_OBJECT_PROP_MOVE_OFF          106
+#define CFAPI_OBJECT_PROP_MOVE_SLOW         107
+#define CFAPI_OBJECT_PROP_MOVE_SLOW_PENALTY 108
+
 #define CFAPI_PLAYER_PROP_IP                150
 #define CFAPI_PLAYER_PROP_MARKED_ITEM       151
 #define CFAPI_PLAYER_PROP_PARTY             152
+#define CFAPI_PLAYER_PROP_BED_MAP           153
+#define CFAPI_PLAYER_PROP_BED_X             154
+#define CFAPI_PLAYER_PROP_BED_Y             155
 
 #define CFAPI_MAP_PROP_FLAGS                0
 #define CFAPI_MAP_PROP_DIFFICULTY           1
@@ -317,4 +334,4 @@ typedef struct _hook_entry
     char fname[256];
 } hook_entry;
 
-#endif /*PLUGIN_H_*/
+#endif /* PLUGIN_H */
