@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_h =
- *   "$Id: skills.h,v 1.11 2005/08/12 13:46:34 ryo_saeba Exp $";
+ *   "$Id: skills.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -26,6 +26,8 @@
     The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
+#ifndef SKILLS_H
+#define SKILLS_H
 
 /* This list is just a subtype <-> skill (code wise) in the 
  * server translation.  In theory, the processing of the different
@@ -34,7 +36,6 @@
  * The order of this list has no special meaning.  0 is not used
  * to denote improperly set objects.
  */
-
 #define SK_LOCKPICKING		1
 #define SK_HIDING		2
 #define SK_SMITHERY		3
@@ -140,3 +141,5 @@ SK_PUNCHING,
 #undef WANT_UNARMED_SKILLS
 
 #endif
+
+#endif /* SKILLS_H */
