@@ -1,5 +1,5 @@
 /* alchemy.c */
-void attempt_do_alchemy(object *caster, object *cauldron);
+int use_alchemy(object* op);
 /* apply.c */
 int transport_can_hold(const object *transport, const object *op, int nrof);
 int apply_transport(object *pl, object *transport, int aflag);
@@ -55,6 +55,7 @@ int command_orcknuckle(object *op, char *params);
 int command_shout(object *op, char *params);
 int command_chat(object *op, char *params);
 int command_tell(object *op, char *params);
+int command_dmtell(object *op, char *params);
 int command_reply(object *op, char *params);
 int command_nod(object *op, char *params);
 int command_dance(object *op, char *params);
@@ -177,6 +178,7 @@ int command_brace(object *op, char *params);
 int command_style_map_info(object *op, char *params);
 int command_kill_pets(object *op, char *params);
 int command_quests(object *pl, char *params);
+int command_passwd(object *pl, char *params);
 /* c_move.c */
 int command_east(object *op, char *params);
 int command_north(object *op, char *params);
@@ -206,7 +208,7 @@ int sack_can_hold(const object *pl, const object *sack, const object *op, uint32
 void pick_up(object *op, object *alt);
 int command_take(object *op, char *params);
 void put_object_in_sack(object *op, object *sack, object *tmp, uint32 nrof);
-void drop_object(object *op, object *tmp, uint32 nrof);
+object *drop_object(object *op, object *tmp, uint32 nrof);
 void drop(object *op, object *tmp);
 int command_dropall(object *op, char *params);
 int command_drop(object *op, char *params);
@@ -220,6 +222,7 @@ void inventory(object *op, object *inv);
 int command_pickup(object *op, char *params);
 int command_search_items(object *op, char *params);
 int command_rename_item(object *op, char *params);
+int command_lock_item(object *op, char *params);
 /* c_party.c */
 partylist *get_firstparty(void);
 void remove_party(partylist *target_party);
@@ -244,7 +247,8 @@ int command_hide(object *op, char *params);
 int command_setgod(object *op, char *params);
 int command_banish(object *op, char *params);
 int command_kick(object *op, const char *params);
-int command_save_overlay(object *op, char *params);
+int command_overlay_save(object *op, char *params);
+int command_overlay_reset(object *op, char *params);
 int command_toggle_shout(object *op, char *params);
 int command_shutdown(object *op, char *params);
 int command_goto(object *op, char *params);
@@ -340,6 +344,7 @@ void npc_call_help(object *op);
 void check_earthwalls(object *op, mapstruct *m, int x, int y);
 void check_doors(object *op, mapstruct *m, int x, int y);
 void communicate(object *op, const char *txt);
+void npc_say(object *npc, char *cp);
 object *find_mon_throw_ob(object *op);
 int can_detect_enemy(object *op, object *enemy, rv_vector *rv);
 int stand_in_light(object *op);
@@ -490,6 +495,10 @@ void *cfapi_archetype_get_first(int *type, ...);
 void *cfapi_archetype_get_property(int *type, ...);
 void *cfapi_party_get_property(int *type, ...);
 void *cfapi_region_get_property(int *type, ...);
+void *cfapi_get_time(int *type, ...);
+void *cfapi_timer_create(int *type, ...);
+void *cfapi_timer_destroy(int *type, ...);
+void *cfapi_friendlylist_get_next(int *type, ...);
 command_array_struct *find_plugin_command(char *cmd, object *op);
 int initPlugins(void);
 /* resurrection.c */
@@ -516,6 +525,8 @@ void sell_item(object *op, object *pl);
 double shopkeeper_approval(const mapstruct *map, const object *player);
 int describe_shop(const object *op);
 void shop_listing(object *op);
+int is_in_shop(object *ob);
+int coords_in_shop(mapstruct *map, int x, int y);
 /* skills.c */
 int steal(object *op, int dir, object *skill);
 int pick_lock(object *pl, int dir, object *skill);
@@ -604,6 +615,7 @@ void set_spell_skill(object *op, object *caster, object *spob, object *dest);
 void init_spells(void);
 void dump_spells(void);
 void spell_effect(object *spob, int x, int y, mapstruct *map, object *originator);
+int min_casting_level(object *caster, object *spell);
 int caster_level(object *caster, object *spell);
 sint16 SP_level_spellpoint_cost(object *caster, object *spell, int flags);
 int SP_level_dam_adjust(object *caster, object *spob);
@@ -625,6 +637,8 @@ void spell_failure(object *op, int failure, int power, object *skill);
 int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stringarg);
 void move_spell_effect(object *op);
 void apply_spell_effect(object *spell, object *victim);
+void store_spell_expiry(object* spell);
+void check_spell_expiry(object* spell);
 /* swamp.c */
 void walk_on_deep_swamp(object *op, object *victim);
 void move_deep_swamp(object *op);
@@ -655,6 +669,7 @@ void cftimer_process_timers(void);
 int cftimer_create(int id, long delay, object *ob, int mode);
 int cftimer_destroy(int id);
 int cftimer_find_free_id(void);
+void cftimer_init();
 /* weather.c */
 void set_darkness_map(mapstruct *m);
 void tick_the_clock(void);
