@@ -61,6 +61,15 @@
 #include <stdarg.h>
 
 #define PYTHON_DEBUG   /* give us some general infos out */
+#define PYTHON_CACHE_SIZE 16    /* number of python scripts to store the bytecode of at a time */
+
+typedef struct {
+    char *file;
+    PyCodeObject *code;
+    time_t cached_time, used_time;
+} pycode_cache_entry;
+
+static pycode_cache_entry pycode_cache[PYTHON_CACHE_SIZE];
 
 f_plug_api gethook;
 f_plug_api registerGlobalEvent;
@@ -110,10 +119,13 @@ static PyObject* getArchetypes(PyObject* self, PyObject* args);
 static PyObject* getMaps(PyObject* self, PyObject* args);
 static PyObject* getParties(PyObject* self, PyObject* args);
 static PyObject* getRegions(PyObject* self, PyObject* args);
+static PyObject* getFriendlyList(PyObject* self, PyObject* args);
 static PyObject* registerCommand(PyObject* self, PyObject* args);
 static PyObject* registerGEvent(PyObject* self, PyObject* args);
 static PyObject* unregisterGEvent(PyObject* self, PyObject* args);
 static PyObject* CFPythonError;
+static PyObject* getTime(PyObject* self, PyObject* args);
+static PyObject* destroyTimer(PyObject* self, PyObject* args);
 
 /** Set up an Python exception object. */
 static void set_exception(const char *fmt, ...)
@@ -170,9 +182,12 @@ static PyMethodDef CFPythonMethods[] = {
     {"GetMaps",             getMaps,                METH_VARARGS},
     {"GetParties",          getParties,             METH_VARARGS},
     {"GetRegions",          getRegions,             METH_VARARGS},
+    {"GetFriendlyList",     getFriendlyList,        METH_VARARGS},
     {"RegisterCommand",     registerCommand,        METH_VARARGS},
     {"RegisterGlobalEvent", registerGEvent,         METH_VARARGS},
     {"UnregisterGlobalEvent",unregisterGEvent,      METH_VARARGS},
+    {"GetTime",             getTime,                METH_VARARGS},
+    {"DestroyTimer",        destroyTimer,           METH_VARARGS},
     {NULL, NULL, 0}
 };
 
@@ -210,10 +225,6 @@ static PyObject* createCFObject(PyObject* self, PyObject* args)
     object* op;
     op = cf_create_object();
 
-    if (op == NULL) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
     return Crossfire_Object_wrap(op);
 }
 static PyObject* createCFObjectByName(PyObject* self, PyObject* args)
@@ -226,10 +237,6 @@ static PyObject* createCFObjectByName(PyObject* self, PyObject* args)
 
     op = cf_create_object_by_name(obname);
 
-    if (op == NULL) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
     return Crossfire_Object_wrap(op);
 }
 static PyObject* getCFPythonVersion(PyObject* self, PyObject* args)
@@ -296,10 +303,6 @@ static PyObject* readyMap(PyObject* self, PyObject* args)
 
     map = cf_map_get_map(mapname);
 
-    if (map == NULL) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
     return Crossfire_Map_wrap(map);
 }
 static PyObject* getCostFlagTrue(PyObject* self, PyObject* args)
@@ -587,6 +590,20 @@ static PyObject* getRegions(PyObject* self, PyObject* args)
    return list;
 }
 
+static PyObject* getFriendlyList(PyObject* self, PyObject* args)
+{
+    PyObject *list;
+    object *ob;
+
+    list = PyList_New(0);
+    ob = cf_friendlylist_get_first();
+    while (ob) {
+        PyList_Append(list, Crossfire_Object_wrap(ob));
+        ob = cf_friendlylist_get_next(ob);
+   }
+   return list;
+}
+
 static PyObject* registerCommand(PyObject* self, PyObject* args)
 {
     char *cmdname;
@@ -623,6 +640,38 @@ static PyObject* registerCommand(PyObject* self, PyObject* args)
     return Py_None;
 }
 
+static PyObject* getTime(PyObject* self, PyObject* args)
+{
+    PyObject* list;
+    partylist* party;
+    timeofday_t tod;
+
+    if (!PyArg_ParseTuple(args, "", NULL))
+        return NULL;
+
+    cf_get_time(&tod);
+
+    list = PyList_New(0);
+    PyList_Append(list, Py_BuildValue("i",tod.year));
+    PyList_Append(list, Py_BuildValue("i",tod.month));
+    PyList_Append(list, Py_BuildValue("i",tod.day));
+    PyList_Append(list, Py_BuildValue("i",tod.hour));
+    PyList_Append(list, Py_BuildValue("i",tod.minute));
+    PyList_Append(list, Py_BuildValue("i",tod.dayofweek));
+    PyList_Append(list, Py_BuildValue("i",tod.weekofmonth));
+    PyList_Append(list, Py_BuildValue("i",tod.season));
+
+    return list;
+    }
+
+static PyObject* destroyTimer(PyObject* self, PyObject* args)
+{
+    int id;
+    if (!PyArg_ParseTuple(args, "i", &id))
+        return NULL;
+    return Py_BuildValue("i", cf_timer_destroy(id));
+}
+
 void initContextStack()
 {
     current_context = NULL;
@@ -660,9 +709,101 @@ void freeContext(CFPContext* context)
     free(context);
 }
 
+/* Outputs the compiled bytecode for a given python file, using in-memory caching of bytecode */
+static PyCodeObject *compilePython(char *filename) {
+    PyObject*   scriptfile;
+    char  *sh_path;
+    struct  stat stat_buf;
+    struct _node *n;
+    int i;
+    pycode_cache_entry *replace = NULL, *run = NULL;
+
+    if (!(scriptfile = PyFile_FromString(filename, "r"))) {
+        cf_log(llevDebug, "cfpython - The Script file %s can't be opened\n", filename);
+        return NULL;
+    }
+    if(stat(filename, &stat_buf)) {
+        cf_log(llevDebug, "cfpython - The Script file %s can't be stat:ed\n", filename);
+        if(scriptfile)
+            Py_DECREF(scriptfile);
+        return NULL;
+    }
+
+    sh_path = cf_add_string(filename);
+
+    /* Search through cache. Three cases:
+     * 1) script in cache, but older than file  -> replace cached
+     * 2) script in cache and up to date        -> use cached
+     * 3) script not in cache, cache not full   -> add to end of cache
+     * 4) script not in cache, cache full       -> replace least recently used
+     */
+    for(i=0; i < PYTHON_CACHE_SIZE; i++) {
+        if(pycode_cache[i].file == NULL) {  /* script not in cache, cache not full */
+            replace = &pycode_cache[i];     /* add to end of cache */
+            break;
+        } else if(pycode_cache[i].file == sh_path) {
+            /* script in cache */
+            if(pycode_cache[i].code == NULL || (pycode_cache[i].cached_time<stat_buf.st_mtime)) {
+                /* cache older than file, replace cached */
+                replace = &pycode_cache[i];
+            } else {
+                /* cache uptodate, use cached*/
+                replace = NULL;
+                run = &pycode_cache[i];
+            }
+            break;
+        } else if(replace == NULL || pycode_cache[i].used_time < replace->used_time)
+            /* if we haven't found it yet, set replace to the oldest cache */
+            replace = &pycode_cache[i];
+    }
+
+    /* replace a specific cache index with the file */
+    if(replace) {
+        Py_XDECREF(replace->code); /* safe to call on NULL */
+        replace->code = NULL;
+
+        /* Need to replace path string? */
+        if (replace->file != sh_path) {
+            if(replace->file) {
+                cf_free_string(replace->file);
+            }
+            replace->file = cf_add_string(sh_path);
+        }
+
+        /* Load, parse and compile */
+        if (!scriptfile && !(scriptfile = PyFile_FromString(filename, "r"))) {
+            cf_log(llevDebug, "cfpython - The Script file %s can't be opened\n", filename);
+            replace->code = NULL;
+            return NULL;
+        } else {
+            if((n = PyParser_SimpleParseFile (PyFile_AsFile(scriptfile), filename, Py_file_input))) {
+                replace->code = PyNode_Compile(n, filename);
+                PyNode_Free (n);
+            }
+
+            if(PyErr_Occurred())
+                PyErr_Print();
+            else
+                replace->cached_time = stat_buf.st_mtime;
+            run = replace;
+        }
+    }
+
+    cf_free_string(sh_path);
+
+    if(scriptfile)
+        Py_DECREF(scriptfile);
+
+    if (run)
+        return run->code;
+    else
+        return NULL;
+}
+
+
 static int do_script(CFPContext* context, int silent)
 {
-    FILE*   scriptfile;
+    PyCodeObject *pycode;
     PyObject* dict;
     PyObject* ret;
 #if 0
@@ -670,34 +811,219 @@ static int do_script(CFPContext* context, int silent)
     int item;
 #endif
 
-    scriptfile = fopen(context->script, "r");
-    if (scriptfile == NULL) {
-        if (!silent)
-            cf_log( llevError, "cfpython - The Script file %s can't be opened\n", context->script);
-        return 0;
-    }
-    pushContext(context);
-    dict = PyDict_New();
-    PyDict_SetItemString(dict, "__builtins__", PyEval_GetBuiltins());
-    ret = PyRun_File(scriptfile, context->script, Py_file_input, dict, dict);
-    if (PyErr_Occurred()) {
-        PyErr_Print();
-    }
-    Py_XDECREF(ret);
-#if 0
-    printf( "cfpython - %d items in heap\n", PyDict_Size(dict));
-    list = PyDict_Values(dict);
-    for (item = PyList_Size(list) - 1; item >= 0; item--) {
-        dict = PyList_GET_ITEM(list, item);
-        ret = PyObject_Str(dict);
-        printf(" ref %s = %d\n", PyString_AsString(ret), dict->ob_refcnt);
+    pycode = compilePython(context->script);
+    if (pycode) {
+        pushContext(context);
+        dict = PyDict_New();
+        PyDict_SetItemString(dict, "__builtins__", PyEval_GetBuiltins());
+        ret = PyEval_EvalCode(pycode, dict, NULL);
+        if (PyErr_Occurred()) {
+            PyErr_Print();
+        }
         Py_XDECREF(ret);
-    }
-    Py_DECREF(list);
+#if 0
+        printf( "cfpython - %d items in heap\n", PyDict_Size(dict));
+        list = PyDict_Values(dict);
+        for (item = PyList_Size(list) - 1; item >= 0; item--) {
+            dict = PyList_GET_ITEM(list, item);
+            ret = PyObject_Str(dict);
+            printf(" ref %s = %d\n", PyString_AsString(ret), dict->ob_refcnt);
+            Py_XDECREF(ret);
+        }
+        Py_DECREF(list);
 #endif
+        Py_DECREF(dict);
+        return 1;
+    } else
+        return 0;
+}
+
+typedef struct
+{
+    char* name;
+    int value;
+} CFConstant;
+
+static void addConstants(PyObject* module, const char* name, CFConstant* constants)
+{
+    int i = 0;
+    char tmp[1024];
+    PyObject* new;
+    PyObject* dict;
+
+    strncpy(tmp, "Crossfire_", 1024);
+    strncat(tmp, name, 1024 - strlen(tmp));
+
+    new = Py_InitModule(tmp, NULL);
+    dict = PyDict_New();
+
+    while ( constants[i].name != NULL)
+    {
+        PyModule_AddIntConstant(new, constants[i].name, constants[i].value);
+        PyDict_SetItem(dict, PyInt_FromLong(constants[i].value), PyString_FromString(constants[i].name));
+        i++;
+    }
+    PyDict_SetItemString(PyModule_GetDict(module), name, new);
+    Py_DECREF(new);
+
+    strncpy(tmp, name, 1024);
+    strncat(tmp, "Name", 1024 - strlen(tmp));
+    PyDict_SetItemString(PyModule_GetDict(module), tmp, dict);
     Py_DECREF(dict);
-    fclose(scriptfile);
-    return 1;
+}
+
+static void initConstants(PyObject* module)
+{
+    static CFConstant cstDirection[] = {
+        { "NORTH", 1 },
+        { "NORTHEAST", 2 },
+        { "EAST", 3 },
+        { "SOUTHEAST", 4 },
+        { "SOUTH", 5 },
+        { "SOUTHWEST", 6 },
+        { "WEST", 7 },
+        { "NORTHWEST", 8 },
+        { NULL, 0 } };
+
+    static CFConstant cstType[] = {
+        { "PLAYER", PLAYER },
+        { "TRANSPORT", TRANSPORT },
+        { "ROD", ROD },
+        { "TREASURE", TREASURE },
+        { "POTION", POTION },
+        { "FOOD", FOOD },
+        { "POISON", POISON },
+        { "BOOK", BOOK },
+        { "CLOCK", CLOCK },
+        { "LIGHTNING", LIGHTNING },
+        { "ARROW", ARROW },
+        { "BOW", BOW },
+        { "WEAPON", WEAPON },
+        { "ARMOUR", ARMOUR },
+        { "PEDESTAL", PEDESTAL },
+        { "ALTAR", ALTAR },
+        { "LOCKED_DOOR", LOCKED_DOOR },
+        { "SPECIAL_KEY", SPECIAL_KEY },
+        { "MAP", MAP },
+        { "DOOR", DOOR },
+        { "KEY", KEY },
+        { "TIMED_GATE", TIMED_GATE },
+        { "TRIGGER", TRIGGER },
+        { "GRIMREAPER", GRIMREAPER },
+        { "MAGIC_EAR", MAGIC_EAR },
+        { "TRIGGER_BUTTON", TRIGGER_BUTTON },
+        { "TRIGGER_ALTAR", TRIGGER_ALTAR },
+        { "TRIGGER_PEDESTAL", TRIGGER_PEDESTAL },
+        { "SHIELD", SHIELD },
+        { "HELMET", HELMET },
+        { "HORN", HORN },
+        { "MONEY", MONEY },
+        { "CLASS", CLASS },
+        { "GRAVESTONE", GRAVESTONE },
+        { "AMULET", AMULET },
+        { "PLAYERMOVER", PLAYERMOVER },
+        { "TELEPORTER", TELEPORTER },
+        { "CREATOR", CREATOR },
+        { "SKILL", SKILL },
+        { "EXPERIENCE", EXPERIENCE },
+        { "EARTHWALL", EARTHWALL },
+        { "GOLEM", GOLEM },
+        { "THROWN_OBJ", THROWN_OBJ },
+        { "BLINDNESS", BLINDNESS },
+        { "GOD", GOD },
+        { "DETECTOR", DETECTOR },
+        { "TRIGGER_MARKER", TRIGGER_MARKER },
+        { "DEAD_OBJECT", DEAD_OBJECT },
+        { "DRINK", DRINK },
+        { "MARKER", MARKER },
+        { "HOLY_ALTAR", HOLY_ALTAR },
+        { "PLAYER_CHANGER", PLAYER_CHANGER },
+        { "BATTLEGROUND", BATTLEGROUND },
+        { "PEACEMAKER", PEACEMAKER },
+        { "GEM", GEM },
+        { "FIREWALL", FIREWALL },
+        { "ANVIL", ANVIL },
+        { "CHECK_INV", CHECK_INV },
+        { "MOOD_FLOOR", MOOD_FLOOR },
+        { "EXIT", EXIT },
+        { "ENCOUNTER", ENCOUNTER },
+        { "SHOP_FLOOR", SHOP_FLOOR },
+        { "SHOP_MAT", SHOP_MAT },
+        { "RING", RING },
+        { "FLOOR", FLOOR },
+        { "FLESH", FLESH },
+        { "INORGANIC", INORGANIC },
+        { "SKILL_TOOL", SKILL_TOOL },
+        { "LIGHTER", LIGHTER },
+        { "TRAP_PART", TRAP_PART },
+        { "WALL", WALL },
+        { "LIGHT_SOURCE", LIGHT_SOURCE },
+        { "MISC_OBJECT", MISC_OBJECT },
+        { "MONSTER", MONSTER },
+        { "SPAWN_GENERATOR", SPAWN_GENERATOR },
+        { "LAMP", LAMP },
+        { "DUPLICATOR", DUPLICATOR },
+        { "TOOL", TOOL },
+        { "SPELLBOOK", SPELLBOOK },
+        { "BUILDFAC", BUILDFAC },
+        { "CLOAK", CLOAK },
+        { "SPINNER", SPINNER },
+        { "GATE", GATE },
+        { "BUTTON", BUTTON },
+        { "CF_HANDLE", CF_HANDLE },
+        { "HOLE", HOLE },
+        { "TRAPDOOR", TRAPDOOR },
+        { "SIGN", SIGN },
+        { "BOOTS", BOOTS },
+        { "GLOVES", GLOVES },
+        { "SPELL", SPELL },
+        { "SPELL_EFFECT", SPELL_EFFECT },
+        { "CONVERTER", CONVERTER },
+        { "BRACERS", BRACERS },
+        { "POISONING", POISONING },
+        { "SAVEBED", SAVEBED },
+        { "POISONCLOUD", POISONCLOUD },
+        { "FIREHOLES", FIREHOLES },
+        { "WAND", WAND },
+        { "SCROLL", SCROLL },
+        { "DIRECTOR", DIRECTOR },
+        { "GIRDLE", GIRDLE },
+        { "FORCE", FORCE },
+        { "POTION_EFFECT", POTION_EFFECT },
+        { "EVENT_CONNECTOR", EVENT_CONNECTOR },
+        { "CLOSE_CON", CLOSE_CON },
+        { "CONTAINER", CONTAINER },
+        { "ARMOUR_IMPROVER", ARMOUR_IMPROVER },
+        { "WEAPON_IMPROVER", WEAPON_IMPROVER },
+        { "SKILLSCROLL", SKILLSCROLL },
+        { "DEEP_SWAMP", DEEP_SWAMP },
+        { "IDENTIFY_ALTAR", IDENTIFY_ALTAR },
+        { "MENU", MENU },
+        { "RUNE", RUNE },
+        { "TRAP", TRAP },
+        { "POWER_CRYSTAL", POWER_CRYSTAL },
+        { "CORPSE", CORPSE },
+        { "DISEASE", DISEASE },
+        { "SYMPTOM", SYMPTOM },
+        { "BUILDER", BUILDER },
+        { "MATERIAL", MATERIAL },
+        { "ITEM_TRANSFORMER", ITEM_TRANSFORMER },
+        { "QUEST", QUEST },
+        { NULL, 0 } };
+
+    static CFConstant cstMove[] = {
+        { "WALK", MOVE_WALK },
+        { "FLY_LOW", MOVE_FLY_LOW },
+        { "FLY_HIGH", MOVE_FLY_HIGH },
+        { "FLYING", MOVE_FLYING },
+        { "SWIM", MOVE_SWIM },
+        { "BOAT", MOVE_BOAT },
+        { "ALL", MOVE_ALL },
+        { NULL, 0 } };
+
+    addConstants(module, "Direction", cstDirection);
+    addConstants(module, "Type", cstType);
+    addConstants(module, "Move", cstMove);
 }
 
 CF_PLUGIN int initPlugin(const char* iversion, f_plug_api gethooksptr)
@@ -708,6 +1034,9 @@ CF_PLUGIN int initPlugin(const char* iversion, f_plug_api gethooksptr)
     cf_init_plugin( gethook );
     cf_log(llevDebug, "CFPython 2.0a init\n");
 
+    init_object_assoc_table();
+    init_map_assoc_table();
+
     Py_Initialize();
     Crossfire_ObjectType.tp_new = PyType_GenericNew;
     Crossfire_MapType.tp_new    = PyType_GenericNew;
@@ -745,6 +1074,7 @@ CF_PLUGIN int initPlugin(const char* iversion, f_plug_api gethooksptr)
         CustomCommand[i].script = NULL;
         CustomCommand[i].speed  = 0.0;
     }
+    initConstants(m);
     private_data = PyDict_New();
     shared_data = PyDict_New();
     return 0;
@@ -830,7 +1160,7 @@ CF_PLUGIN int postInitPlugin()
 {
     int hooktype = 1;
     int rtype = 0;
-    FILE*   scriptfile;
+    PyObject*   scriptfile;
 
     cf_log(llevDebug, "CFPython 2.0a post init\n");
     registerGlobalEvent =   gethook(&rtype, hooktype, "cfapi_system_register_global_event");
@@ -853,11 +1183,13 @@ CF_PLUGIN int postInitPlugin()
     registerGlobalEvent(NULL, EVENT_TELL, PLUGIN_NAME, globalEventListener);
     registerGlobalEvent(NULL, EVENT_MUZZLE, PLUGIN_NAME, globalEventListener);
     registerGlobalEvent(NULL, EVENT_KICK, PLUGIN_NAME, globalEventListener);
+    registerGlobalEvent(NULL, EVENT_MAPUNLOAD, PLUGIN_NAME, globalEventListener);
+    registerGlobalEvent(NULL, EVENT_MAPLOAD, PLUGIN_NAME, globalEventListener);
 
-    scriptfile = fopen(cf_get_maps_directory("python/events/python_init.py"), "r");
+    scriptfile = PyFile_FromString(cf_get_maps_directory("python/events/python_init.py"), "r");
     if (scriptfile != NULL) {
-        PyRun_SimpleFile(scriptfile, cf_get_maps_directory("python/events/python_init.py"));
-        fclose(scriptfile);
+        PyRun_SimpleFile(PyFile_AsFile(scriptfile), cf_get_maps_directory("python/events/python_init.py"));
+        Py_DECREF(scriptfile);
     }
 
     return 0;
@@ -965,12 +1297,14 @@ CF_PLUGIN void* globalEventListener(int* type, ...)
         case EVENT_MAPENTER:
             op = va_arg(args, object*);
             context->activator = Crossfire_Object_wrap(op);
+            context->who = Crossfire_Map_wrap(va_arg(args, mapstruct*));
             snprintf(context->options, sizeof(context->options), "mapenter");
             cfob = (Crossfire_Object*)context->activator;
             break;
         case EVENT_MAPLEAVE:
             op = va_arg(args, object*);
             context->activator = Crossfire_Object_wrap(op);
+            context->who = Crossfire_Map_wrap(va_arg(args, mapstruct*));
             snprintf(context->options, sizeof(context->options), "mapleave");
             cfob = (Crossfire_Object*)context->activator;
             break;
@@ -978,14 +1312,20 @@ CF_PLUGIN void* globalEventListener(int* type, ...)
             snprintf(context->options, sizeof(context->options), "clock");
             break;
         case EVENT_MAPRESET:
-            buf = va_arg(args, char*);
-            if (buf != NULL)
-                snprintf(context->message, sizeof(context->message), "%s", buf);
+            context->who = Crossfire_Map_wrap(va_arg(args, mapstruct*));
             snprintf(context->options, sizeof(context->options), "mapreset");
             break;
         case EVENT_TELL:
             snprintf(context->options, sizeof(context->options), "tell");
             break;
+        case EVENT_MAPUNLOAD:
+            context->who = Crossfire_Map_wrap(va_arg(args, mapstruct*));
+            snprintf(context->options, sizeof(context->options), "mapunload");
+            break;
+        case EVENT_MAPLOAD:
+            context->who = Crossfire_Map_wrap(va_arg(args, mapstruct*));
+            snprintf(context->options, sizeof(context->options), "mapload");
+            break;
     }
     va_end(args);
     context->returnvalue = 0;
@@ -997,8 +1337,13 @@ CF_PLUGIN void* globalEventListener(int* type, ...)
 
     context = popContext();
     rv = context->returnvalue;
+    
+    /* Invalidate freed map wrapper. */
+    if (context->event_code == EVENT_MAPUNLOAD)
+        Handle_Map_Unload_Hook(context->who);
+    
     freeContext(context);
-
+    
     return &rv;
 }
 
@@ -1007,6 +1352,7 @@ CF_PLUGIN void* eventListener(int* type, ...)
     static int rv = 0;
     va_list args;
     char* buf;
+    char* script_tmp;
     CFPContext* context;
 
     rv = 0;
@@ -1025,11 +1371,18 @@ CF_PLUGIN void* eventListener(int* type, ...)
     if (buf != NULL)
         snprintf(context->message, sizeof(context->message), "%s", buf);
     context->fix         = va_arg(args, int);
-    snprintf(context->script, sizeof(context->script), "%s", cf_get_maps_directory(va_arg(args, char*)));
+    script_tmp = va_arg(args, char*);
+    snprintf(context->script, sizeof(context->script), "%s", cf_get_maps_directory(script_tmp));
     snprintf(context->options, sizeof(context->options), "%s", va_arg(args, char*));
     context->returnvalue = 0;
 
     va_end(args);
+    
+    if ((context->event_code == EVENT_DESTROY) && !strcmp(script_tmp, "cfpython_auto_hook")) {
+        Handle_Destroy_Hook(context->who);
+        freeContext(context);
+        return &rv;
+    }
 
     if (!do_script(context, 0)) {
         freeContext(context);
