@@ -67,8 +67,10 @@ static PyObject* Object_GetPickable(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetQuantity(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetInvisible(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetSpeed(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetSpeedLeft(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLastSP(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLastGrace(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetLastEat(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLevel(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetFace(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetAttackType(Crossfire_Object* whoptr, void* closure);
@@ -124,9 +126,21 @@ static PyObject* Object_GetOnlyAttack(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMakeInvisible(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMoney(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetType(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetSubtype(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetValue(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetArchName(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetArchetype(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetNoSave(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetExists(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetEnv(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveType(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveBlock(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveAllow(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveOn(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveOff(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveSlow(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetMoveSlowPenalty(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetOwner(Crossfire_Object* whoptr, void* closure);
 
 static int Object_SetMessage(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetName(Crossfire_Object* whoptr, PyObject* value, void* closure);
@@ -159,9 +173,11 @@ static int Object_SetDirection(Crossfire_Object* whoptr, PyObject* value, void*
 static int Object_SetFacing(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetGod(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetSpeed(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetSpeedLeft(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetQuantity(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetLastSP(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetLastGrace(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetLastEat(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetAttackType(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetUnaggressive(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetPickable(Crossfire_Object* whoptr, PyObject* value, void* closure);
@@ -196,6 +212,9 @@ static int Object_SetOnlyAttack(Crossfire_Object* whoptr, PyObject* value, void*
 static int Object_SetMakeInvisible(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetValue(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetFace(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetNoSave(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetOwner(Crossfire_Object* whoptr, PyObject* value, void* closure);
+
 
 static PyObject* Crossfire_Object_Remove( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Apply( Crossfire_Object* who, PyObject* args );
@@ -227,6 +246,8 @@ static PyObject* Crossfire_Object_CreateInside(Crossfire_Object* who, PyObject*
 static PyObject* Crossfire_Object_InsertInto(Crossfire_Object* who, PyObject* args);
 static PyObject* Crossfire_Object_ReadKey(Crossfire_Object* who, PyObject* args);
 static PyObject* Crossfire_Object_WriteKey(Crossfire_Object* who, PyObject* args);
+static PyObject* Crossfire_Object_CreateTimer(Crossfire_Object* who, PyObject* args);
+static PyObject* Crossfire_Object_AddExp(Crossfire_Object* who, PyObject* args);
 
 static int Crossfire_Object_InternalCompare(Crossfire_Object* left, Crossfire_Object* right);
 
@@ -278,8 +299,10 @@ static PyGetSetDef Object_getseters[] = {
     { "Quantity",   (getter)Object_GetQuantity, (setter)Object_SetQuantity, NULL, NULL },
     { "Invisible",  (getter)Object_GetInvisible,(setter)Object_SetInvisible, NULL, NULL },
     { "Speed",      (getter)Object_GetSpeed,    (setter)Object_SetSpeed, NULL, NULL },
+    { "SpeedLeft",  (getter)Object_GetSpeedLeft,(setter)Object_SetSpeedLeft, NULL, NULL },
     { "LastSP",     (getter)Object_GetLastSP,   (setter)Object_SetLastSP, NULL, NULL },
     { "LastGrace",  (getter)Object_GetLastGrace,(setter)Object_SetLastGrace, NULL, NULL },
+    { "LastEat",    (getter)Object_GetLastEat,  (setter)Object_SetLastEat, NULL, NULL },
     { "Level",      (getter)Object_GetLevel,    NULL, NULL, NULL },
     { "Face",       (getter)Object_GetFace,     (setter)Object_SetFace, NULL, NULL },
     { "AttackType", (getter)Object_GetAttackType, (setter)Object_SetAttackType, NULL, NULL },
@@ -335,9 +358,21 @@ static PyGetSetDef Object_getseters[] = {
     { "MakeInvisible",  (getter)Object_GetMakeInvisible,(setter)Object_SetMakeInvisible,NULL, NULL },
     { "Money",          (getter)Object_GetMoney,        NULL ,NULL, NULL },
     { "Type",           (getter)Object_GetType,         NULL ,NULL, NULL },
+    { "Subtype",        (getter)Object_GetSubtype,         NULL ,NULL, NULL },
     { "Value",          (getter)Object_GetValue,        (setter)Object_SetValue ,NULL, NULL },
     { "ArchName",       (getter)Object_GetArchName,     NULL ,NULL, NULL },
     { "Archetype",      (getter)Object_GetArchetype,    NULL ,NULL, NULL },
+    { "Exists",         (getter)Object_GetExists,       NULL ,NULL, NULL },
+    { "NoSave",         (getter)Object_GetNoSave,       (setter)Object_SetNoSave, NULL, NULL },
+    { "Env",            (getter)Object_GetEnv,          NULL, NULL, NULL },
+    { "MoveType",       (getter)Object_GetMoveType,     NULL, NULL, NULL },
+    { "MoveBlock",      (getter)Object_GetMoveBlock,    NULL, NULL, NULL },
+    { "MoveAllow",      (getter)Object_GetMoveAllow,    NULL, NULL, NULL },
+    { "MoveOn",         (getter)Object_GetMoveOn,       NULL, NULL, NULL },
+    { "MoveOff",        (getter)Object_GetMoveOff,      NULL, NULL, NULL },
+    { "MoveSlow",       (getter)Object_GetMoveSlow,     NULL, NULL, NULL },
+    { "MoveSlowPenalty",(getter)Object_GetMoveSlowPenalty,  NULL, NULL, NULL },
+    { "Owner",          (getter)Object_GetOwner,        (setter)Object_SetOwner, NULL, NULL },
     { NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -370,6 +405,8 @@ static PyMethodDef ObjectMethods[] = {
     { "InsertInto",     (PyCFunction)Crossfire_Object_InsertInto,   METH_VARARGS},
     { "ReadKey",        (PyCFunction)Crossfire_Object_ReadKey,      METH_VARARGS},
     { "WriteKey",       (PyCFunction)Crossfire_Object_WriteKey,     METH_VARARGS},
+    { "CreateTimer",    (PyCFunction)Crossfire_Object_CreateTimer,  METH_VARARGS},
+    { "AddExp",         (PyCFunction)Crossfire_Object_AddExp,       METH_VARARGS},
     {NULL, NULL, 0}
 };
 
@@ -397,6 +434,7 @@ static PyNumberMethods ObjectConvert[ ] = {
         0
 };
 
+static void Crossfire_Object_dealloc(PyObject *obj);
 static PyObject *Crossfire_Object_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
 /* Our actual Python ObjectType */
@@ -406,7 +444,7 @@ PyTypeObject Crossfire_ObjectType = {
             "Crossfire.Object",        /* tp_name*/
             sizeof(Crossfire_Object),  /* tp_basicsize*/
             0,                         /* tp_itemsize*/
-            0,                         /* tp_dealloc*/
+            Crossfire_Object_dealloc,  /* tp_dealloc*/
             0,                         /* tp_print*/
             0,                         /* tp_getattr*/
             0,                         /* tp_setattr*/
@@ -453,11 +491,20 @@ static PyObject* Crossfire_Player_Message( Crossfire_Player* who, PyObject* args
 static PyObject* Player_GetParty(Crossfire_Player* whoptr, void* closure);
 static int Player_SetParty(Crossfire_Player* whoptr, PyObject* value, void* closure);
 static PyObject* Crossfire_Player_CanPay( Crossfire_Player* who, PyObject* args );
+static PyObject* Player_GetBedMap(Crossfire_Player* whoptr, void* closure);
+static PyObject* Player_GetBedX(Crossfire_Player* whoptr, void* closure);
+static PyObject* Player_GetBedY(Crossfire_Player* whoptr, void* closure);
+static int Player_SetBedMap(Crossfire_Player* whoptr, PyObject* value, void* closure);
+static int Player_SetBedX(Crossfire_Player* whoptr, PyObject* value, void* closure);
+static int Player_SetBedY(Crossfire_Player* whoptr, PyObject* value, void* closure);
 
 static PyGetSetDef Player_getseters[] = {
     { "IP",            (getter)Player_GetIP,            NULL, NULL, NULL },
 	{ "MarkedItem",    (getter)Player_GetMarkedItem,    (setter)Player_SetMarkedItem, NULL, NULL },
 	{ "Party",         (getter)Player_GetParty,         (setter)Player_SetParty,      NULL, NULL },
+    { "BedMap",        (getter)Player_GetBedMap,        (setter)Player_SetBedMap, NULL, NULL },
+    { "BedX",          (getter)Player_GetBedX,          (setter)Player_SetBedX, NULL, NULL },
+    { "BedY",          (getter)Player_GetBedY,          (setter)Player_SetBedY, NULL, NULL },
     { NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -468,6 +515,7 @@ static PyMethodDef PlayerMethods[] = {
     {NULL, NULL }
 };
 
+static void Crossfire_Player_dealloc(PyObject *obj);
 static PyObject *Crossfire_Player_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
 /* Our actual Python ObjectPlayerType */
@@ -477,7 +525,7 @@ PyTypeObject Crossfire_PlayerType = {
             "Crossfire.Player",        /* tp_name*/
             sizeof(Crossfire_Player),  /* tp_basicsize*/
             0,                         /* tp_itemsize*/
-            0,                         /* tp_dealloc*/
+            Crossfire_Player_dealloc,  /* tp_dealloc*/
             0,                         /* tp_print*/
             0,                         /* tp_getattr*/
             0,                         /* tp_setattr*/
