@@ -42,6 +42,9 @@ static f_plug_api cfapiSystem_unregister_global_event = NULL;
 static f_plug_api cfapiSystem_strdup_local = NULL;
 static f_plug_api cfapiSystem_find_animation = NULL;
 static f_plug_api cfapiSystem_log = NULL;
+static f_plug_api cfapiSystem_get_time = NULL;
+static f_plug_api cfapiSystem_timer_create = NULL;
+static f_plug_api cfapiSystem_timer_destroy = NULL;
 
 static f_plug_api cfapiMap_create_path = NULL;
 
@@ -105,6 +108,7 @@ static f_plug_api cfapiArchetype_get_property = NULL;
 static f_plug_api cfapiParty_get_property = NULL;
 static f_plug_api cfapiRegion_get_property = NULL;
 static f_plug_api cfapiPlayer_can_pay = NULL;
+static f_plug_api cfapiFriendlylist_get_next = NULL;
 
 #define GET_HOOK( x, y, z ) \
     { \
@@ -186,6 +190,10 @@ int cf_init_plugin( f_plug_api getHooks )
     GET_HOOK( cfapiRegion_get_property, "cfapi_region_get_property", z );
     GET_HOOK( cfapiPlayer_can_pay, "cfapi_player_can_pay", z );
     GET_HOOK( cfapiSystem_log, "cfapi_log", z );
+    GET_HOOK( cfapiSystem_get_time, "cfapi_system_get_time", z );
+    GET_HOOK( cfapiSystem_timer_create, "cfapi_system_timer_create", z );
+    GET_HOOK( cfapiSystem_timer_destroy, "cfapi_system_timer_destroy", z );
+    GET_HOOK( cfapiFriendlylist_get_next, "cfapi_friendlylist_get_next", z );
     return 1;
 }
 
@@ -224,12 +232,27 @@ void* cf_object_set_long_property(object* op, int propcode, long value)
     int val;
     return cfapiObject_set_property(&val, op, propcode,value);
 }
+void* cf_object_set_float_property(object* op, int propcode, float value)
+{
+    int val;
+    return cfapiObject_set_property(&val, op, propcode, value);
+}
 /* Should get replaced by tons of more explicit wrappers */
 void* cf_object_set_string_property(object* op, int propcode, char* value)
 {
     int val;
     return cfapiObject_set_property(&val, op, propcode,value);
 }
+void* cf_object_set_object_property(object* op, int propcode, object* value)
+{
+    int val;
+    return cfapiObject_set_property(&val, op, propcode,value);
+}
+void* cf_object_set_experience(object* op, sint64 exp, const char* skill, int arg)
+{
+    int val;
+    return cfapiObject_set_property(&val, op, CFAPI_OBJECT_PROP_EXP, exp, strlen(skill) > 0 ? skill : NULL, arg);
+}
 void cf_player_move(player* pl, int dir)
 {
     int val;
@@ -752,6 +775,25 @@ void cf_log( LogLevel logLevel, const char* format, ... )
 
     va_end(ap);
 }
+
+void cf_get_time( timeofday_t* tod )
+{
+    int val;
+    cfapiSystem_get_time(&val, tod);
+}
+
+int cf_timer_create(object* ob, long delay, int mode)
+{
+    int val;
+    return *( int* )cfapiSystem_timer_create(&val, ob, delay, mode);
+}
+
+int cf_timer_destroy(int id)
+{
+    int val;
+    return *( int* )cfapiSystem_timer_destroy(&val, id);
+}
+
 char* cf_object_get_key(object* op, char* keyname)
 {
     int val;
@@ -873,6 +915,19 @@ const char* cf_region_get_message(region* reg)
 	return cfapiRegion_get_property(&val, reg, CFAPI_REGION_PROP_MESSAGE);
 }
 
+/* Friendlylist functions. */
+object* cf_friendlylist_get_first(void)
+{
+    int val;
+    return cfapiFriendlylist_get_next(&val, NULL);
+}
+object* cf_friendlylist_get_next(object* ob)
+{
+    int val;
+    return cfapiFriendlylist_get_next(&val, ob);
+}
+
+
 #ifdef WIN32
 int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Info)
 {
