@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_standalone_c =
- *   "$Id: standalone.c,v 1.18 2006/05/05 09:41:15 ryo_saeba Exp $";
+ *   "$Id: standalone.c 4961 2006-09-21 05:14:18Z mwedel $";
  */
 
 /*
@@ -248,3 +248,13 @@ void monster_check_apply( object* ob, object* obt )
 void trap_adjust( object* ob, int x )
     {
     }
+
+int execute_event(object* op, int eventcode, object* activator, object* third, const char* message, int fix)
+    {
+    return 0;
+    }
+
+int execute_global_event(int eventcode, ...)
+    {
+    return 0;
+    }
