@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_wall_c =
- *   "$Id: wall.c,v 1.12 2005/11/16 08:16:08 mwedel Exp $";
+ *   "$Id: wall.c 4961 2006-09-21 05:14:18Z mwedel $";
  */
 
 /*
@@ -72,7 +72,7 @@ int surround_flag2(char **layout,int i,int j,RMParms *RP){
  * is that this is not a tiled map.
  * What is considered blocking and not is somewhat hard coded.
  */
-int surround_flag3(mapstruct *map,sint16 i,sint16 j,RMParms *RP){
+int surround_flag3(mapstruct *map,int i,int j,RMParms *RP){
     /*
      * 1 =  blocked to left,
      * 2 = blocked to right,
