@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c,v 1.28 2006/05/05 09:26:34 ryo_saeba Exp $";
+ *   "$Id: alchemy.c 4961 2006-09-21 05:14:18Z mwedel $";
  */
 
 /*
@@ -70,12 +70,13 @@ static recipe *find_recipe(recipelist *fl, int formula, object *ingredients);
 static int content_recipe_value (object *op);
 static int numb_ob_inside (object *op);
 static void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int danger);
-static object * attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches);
+static object * attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches, int ignore_cauldron);
 static int calc_alch_danger(object *caster,object *cauldron, recipe *rp);
 static object * make_item_from_recipe(object *cauldron, recipe *rp);
 static void remove_contents (object *first_ob, object *save_item);
 static void adjust_product(object *item, int lvl, int yield);
 static object * find_transmution_ob ( object *first_ingred, recipe *rp, size_t *rp_arch_index, int create_item);
+static void attempt_do_alchemy(object *caster, object *cauldron);
 
 /** Returns a random selection from cauldron_effect[] */
 static const char *cauldron_sound(void) {
@@ -108,7 +109,7 @@ static const char *cauldron_sound(void) {
  * -b.t.
  */
  
-void attempt_do_alchemy(object *caster, object *cauldron) {
+static void attempt_do_alchemy(object *caster, object *cauldron) {
     recipelist *fl;
     recipe *rp=NULL;
     float success_chance;
@@ -136,7 +137,8 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 		    LOG(llevDebug, "WIZ got formula: %s (nbatches:%d)\n",
 			rp->arch_name[0], formula/rp->index);
 #endif
-		attempt_recipe(caster, cauldron, ability, rp, formula/rp->index);
+		attempt_recipe(caster, cauldron, ability, rp, formula/rp->index, 
+                    !is_defined_recipe(rp, cauldron, caster));
 	    } else LOG(llevDebug, "WIZ couldn't find formula for ingredients.\n"); 
 	    return;
 	} /* End of WIZ alchemy */
@@ -176,7 +178,7 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 	    attempt_shadow_alchemy = !is_defined_recipe(rp, cauldron, caster);
 
 	    /* create the object **FIRST**, then decide whether to keep it.	*/
-	    if ((item=attempt_recipe(caster, cauldron, ability, rp, formula/rp->index)) != NULL) {
+	    if ((item=attempt_recipe(caster, cauldron, ability, rp, formula/rp->index, attempt_shadow_alchemy)) != NULL) {
 		/*  compute base chance of recipe success */
 		success_chance = ((float)ability /
 				  (float)(rp->diff * (item->level+2)));
@@ -267,16 +269,18 @@ static int numb_ob_inside (object *op) {
  * gain some exp from (successfull) fabrication of the product. 
  * If nbatches==-1, don't give exp for this creation (random generation/
  * failed recipe)
+ * If ignore_cauldron, don't check if we are using the matching cauldron
+ * type (shadow alchemy)
  */ 
  
-static object * attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches) { 
+static object * attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches, int ignore_cauldron) { 
 
     object *item=NULL, *skop;
     /* this should be passed to this fctn, not effiecent cpu use this way */
     int batches=abs(nbatches);
 
     /* is the cauldron the right type? */
-    if (strcmp(rp->cauldron, cauldron->arch->name) != 0) {
+    if (!ignore_cauldron && (strcmp(rp->cauldron, cauldron->arch->name) != 0)) {
 	new_draw_info(NDI_UNIQUE, 0, caster, "You are not using the proper"
 	    " facilities for this formula.");
 	return 0;
@@ -496,7 +500,7 @@ static void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int da
 	   if((rp=get_random_recipe((recipelist *) NULL))==NULL) 
 	      	return;
 
-	if((tmp=attempt_recipe(op,cauldron,1,rp,-1))) { 
+	if((tmp=attempt_recipe(op,cauldron,1,rp,-1,0))) { 
 	    if(!QUERY_FLAG(tmp,FLAG_CURSED)) /* curse it */
 	   	SET_FLAG(tmp,FLAG_CURSED);
 
@@ -521,7 +525,7 @@ static void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int da
 	fl=get_formulalist(numb-1); /* take a lower recipe list */ 
 	if(fl &&(rp=get_random_recipe(fl)))
 	    /* even though random, don't grant user any EXP for it */
-	    (void) attempt_recipe(op,cauldron,1,rp,-1);
+	    (void) attempt_recipe(op,cauldron,1,rp,-1,0);
 	else 
 	    alchemy_failure_effect(op,cauldron,rp,level-1);
 	return;
@@ -862,3 +866,46 @@ static recipe *find_recipe(recipelist *fl, int formula, object *ingredients)
 #endif
     return result;
 }
+
+/**
+ * Handle use_skill for alchemy-like items.
+ * Will return 1 if any recipe was attempted, 0 else.
+ * Will inform player if attempting to use unpaid cauldron or ingredient.
+ **/
+int use_alchemy(object* op)
+{
+    object* tmp, *item, *next;
+    object* unpaid_cauldron = NULL;
+    object* unpaid_item = NULL;
+    int did_alchemy = 0;
+
+    for (tmp=get_map_ob(op->map, op->x, op->y); tmp != NULL;tmp=next) {
+        next=tmp->above;
+        if(QUERY_FLAG(tmp, FLAG_IS_CAULDRON)) {
+            if (QUERY_FLAG(tmp, FLAG_UNPAID)) {
+                unpaid_cauldron = tmp;
+                continue;
+            }
+            unpaid_item = NULL;
+            for (item = tmp->inv; item; item = item->below) {
+                if (QUERY_FLAG(item, FLAG_UNPAID)) {
+                    unpaid_item = item;
+                    break;
+                }
+            }
+            if ( unpaid_item != NULL )
+                continue;
+
+            attempt_do_alchemy(op, tmp);
+            if (QUERY_FLAG(tmp, FLAG_APPLIED))
+                esrv_send_inventory(op, tmp);
+            did_alchemy = 1;
+        }
+    }
+    if ( unpaid_cauldron )
+        new_draw_info_format(NDI_UNIQUE,0,op,"You must pay for your %s first!", query_base_name(unpaid_cauldron,0));
+    else if ( unpaid_item )
+        new_draw_info_format(NDI_UNIQUE,0,op,"You must pay for your %s first!", query_base_name(unpaid_item,0));
+
+    return did_alchemy;
+}
