@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_attack_c =
- *   "$Id: attack.c,v 1.125 2006/05/05 09:26:34 ryo_saeba Exp $";
+ *   "$Id: attack.c 5707 2007-03-06 23:03:58Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -156,6 +156,7 @@ void save_throw_object (object *op, int type, object *originator)
     if ( ! did_make_save_item (op, type,originator))
     {
 	object *env=op->env;
+        object *inv;
 	int x=op->x,y=op->y;
 	mapstruct *m=op->map;
 
@@ -163,6 +164,14 @@ void save_throw_object (object *op, int type, object *originator)
         if (op == NULL)
             return;
 
+        /* Set off runes in the inventory of the object being destroyed. */
+        inv=op->inv;
+        while(inv!=NULL) {
+            if (inv->type == RUNE)
+                spring_trap(inv, originator);
+            inv=inv->below;
+        }
+
 	/* Hacked the following so that type LIGHTER will work. 
 	 * Also, objects which are potenital "lights" that are hit by 
 	 * flame/elect attacks will be set to glow. "lights" are any 
@@ -202,7 +211,7 @@ void save_throw_object (object *op, int type, object *originator)
                   fix_stopped_item (op, m, originator);
 	} else {
 	    if (op->env) {
-		object *tmp= is_player_inv(op->env);
+		object *tmp= get_player_container(op->env);
 
 		if (tmp) {
 		    esrv_del_item(tmp->contr, op->count);
@@ -699,7 +708,7 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
         goto error;
 
     /* Lauwenmark: Handle for plugin attack event */
-    execute_event(op, EVENT_ATTACK,hitter,hitter,NULL,SCRIPT_FIX_ALL);
+    execute_event(op, EVENT_ATTACK,hitter,hitter->current_weapon ? hitter->current_weapon : hitter,NULL,SCRIPT_FIX_ALL);
 
     /* Lauwenmark: This is used to handle script_weapons with weapons.
      * Only used for players.
@@ -709,8 +718,8 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
         if (hitter->current_weapon != NULL)
         {
             /* Lauwenmark: Handle for plugin attack event */
-            execute_event(hitter, EVENT_ATTACK,hitter->current_weapon,
-                          op,NULL,SCRIPT_FIX_ALL);
+            execute_event(hitter->current_weapon, EVENT_ATTACK,
+                          hitter,op,NULL,SCRIPT_FIX_ALL);
         }
             }
     op_tag = op->count;
@@ -891,18 +900,23 @@ object *hit_with_arrow (object *op, object *victim)
     int hit_something=0;
     tag_t victim_tag, hitter_tag;
     sint16 victim_x, victim_y;
+    mapstruct	*victim_map;
 
     /* Disassemble missile */
-    if (op->inv) {
+    for (hitter = op->inv; hitter; hitter = hitter->below) {
+        if (hitter->type == EVENT_CONNECTOR)
+            continue;
         container = op;
         hitter = op->inv;
         remove_ob (hitter);
         insert_ob_in_map(hitter, container->map,hitter,INS_NO_MERGE | INS_NO_WALK_ON);
+        break;
         /* Note that we now have an empty THROWN_OBJ on the map.  Code that
          * might be called until this THROWN_OBJ is either reassembled or
          * removed at the end of this function must be able to deal with empty
          * THROWN_OBJs. */
-    } else {
+    }
+    if (!hitter) {
         container = NULL;
         hitter = op;
     }
@@ -910,6 +924,7 @@ object *hit_with_arrow (object *op, object *victim)
     /* Try to hit victim */
     victim_x = victim->x;
     victim_y = victim->y;
+    victim_map = victim->map;
     victim_tag = victim->count;
     hitter_tag = hitter->count;
     /* Lauwenmark: Handling plugin attack event for thrown items */
@@ -963,7 +978,7 @@ object *hit_with_arrow (object *op, object *victim)
             remove_ob (hitter);
             hitter->x = victim_x;
             hitter->y = victim_y;
-            insert_ob_in_map (hitter, victim->map, hitter,0);
+            insert_ob_in_map (hitter, victim_map, hitter,0);
         } else {
             /* Else leave arrow where it is */
             merge_ob (hitter, NULL);
@@ -1606,6 +1621,9 @@ static int kill_object(object *op,int dam, object *hitter, int type)
 	    strncpy(op->contr->killer,hitter->name,BIG_NAME);
 	    op->contr->killer[BIG_NAME-1]='\0';
 	}
+        /* Need to run kill_player (just in case, make sure is not wiz) */
+        if (!QUERY_FLAG(op,FLAG_WIZ))
+	    kill_player(op);
     }
     /* This was return -1 - that doesn't seem correct - if we return -1, process
      * continues in the calling function.
@@ -1775,6 +1793,10 @@ int hit_player(object *op,int dam, object *hitter, int type, int full_hit) {
 	      maxdam = ndam;
 	      maxattacktype = 1<<attacknum;
 	    }
+        /* Special case: death attack always deals all damage, as it should kill the monster
+         * right away. */
+        if ( attacktype == AT_DEATH && ndam > 0 )
+            full_hit = 1;
 	}
     }
 	
@@ -2078,24 +2100,32 @@ void paralyze_player(object *op, object *hitter, int dam)
 
 
 /**
- * Attempts to kill 'op'.  hitter is the attack object, dam is
+ * Attempts to kill 'op'. hitter is the attack object, dam is
  * the computed damaged.
+ *
+ * The intention of a death attack is to kill outright things
+ * that are a lot weaker than the attacker, have a chance of killing
+ * things somewhat weaker than the caster, and no chance of
+ * killing something equal or stronger than the attacker.
+ *
+ * If a deathstrike attack has a slaying, only a monster
+ * whose name or race matches a comma-delimited list in the slaying
+ * field of the deathstriking object is affected (this includes undead).
+ * If no slaying set, only undead are unaffected.
  */
 static void deathstrike_player(object *op, object *hitter, int *dam) 
 {
-    /*  The intention of a death attack is to kill outright things
-    **  that are a lot weaker than the attacker, have a chance of killing
-    **  things somewhat weaker than the caster, and no chance of
-    **  killing something equal or stronger than the attacker.
-    **  Also, if a deathstrike attack has a slaying, any monster
-    **  whose name or race matches a comma-delimited list in the slaying
-    **  field of the deathstriking object  */
-
     int atk_lev, def_lev, kill_lev;
 
-    if(hitter->slaying) 
-	if(!( (QUERY_FLAG(op,FLAG_UNDEAD)&&strstr(hitter->slaying,undead_name)) ||
-		(op->race&&strstr(hitter->slaying,op->race))))	return;
+    if(hitter->slaying)
+    {
+        if(!( (QUERY_FLAG(op,FLAG_UNDEAD)&&strstr(hitter->slaying,undead_name)) ||
+            (op->race&&strstr(hitter->slaying,op->race))))
+            return;
+    }
+    else
+        if (QUERY_FLAG(op, FLAG_UNDEAD))
+            return;
 
     def_lev = op->level;
     if (def_lev < 1) {
