@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_build_map =
- *   "$Id: build_map.c,v 1.11 2006/04/06 21:18:35 tchize Exp $";
+ *   "$Id: build_map.c 5794 2007-03-17 21:13:25Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -38,12 +38,16 @@
 int can_build_over( struct mapdef* map, object* tmp, short x, short y)
     {
     object* ob;
-    
+
     ob = GET_MAP_OB( map, x, y );
     while ( ob )
         {
+            if ((ob->head && !QUERY_FLAG(ob->head, FLAG_IS_BUILDABLE)) || (!ob->head && !QUERY_FLAG(ob, FLAG_IS_BUILDABLE)))
+                // Check for the flag is required, as this function can be called recursively
+                // on different spots.
+                return 0;
 	/* if ob is not a marking rune or floor, then check special cases */
-	if ( strcmp( ob->arch->name, "rune_mark" ) && ob->type != FLOOR )
+	if ( strcmp( ob->arch->name, "rune_mark" ) && ob->type != FLOOR && !QUERY_FLAG(ob, FLAG_IS_FLOOR ))
 	    {
             switch ( tmp->type )
                 {
@@ -67,6 +71,11 @@ int can_build_over( struct mapdef* map, object* tmp, short x, short y)
             }
         ob = ob->above;
 	}
+
+    /* If item is multi-tile, need to check other parts too. */
+    if (tmp->more)
+        return can_build_over(map, tmp->more, x + tmp->more->arch->clone.x - tmp->arch->clone.x, y + tmp->more->arch->clone.y - tmp->arch->clone.y);
+
     return 1;
     }
 
@@ -351,9 +360,10 @@ void apply_builder_floor(object* pl, object* material, short x, short y )
     {
     object* tmp, *above;
     object* above_floor; /* Item above floor, if any */
+    object* floor;       /* Floor which would be removed if required */
     struct archt* new_floor;
     struct archt* new_wall;
-    int i, xt, yt, floor_removed;
+    int i, xt, yt, wall_removed;
     char message[ MAX_BUF ];
 
     sprintf( message, "You change the floor to better suit your tastes." );
@@ -363,38 +373,46 @@ void apply_builder_floor(object* pl, object* material, short x, short y )
      * First, remove wall(s) and floor(s) at position x, y
      */
     above_floor = NULL;
+    floor = NULL;
     new_wall = NULL;
-    floor_removed = 0;
+    wall_removed = 0;
     tmp = GET_MAP_OB( pl->map, x, y );
-    if ( tmp )
+    while ( tmp )
         {
-        while ( tmp )
+        above = tmp->above;
+        if ( WALL == tmp->type )
             {
-            above = tmp->above;
-            if ( WALL == tmp->type )
-                {
-                /* There was a wall, remove it & keep its archetype to make new walls */
-                new_wall = tmp->arch;
-                remove_ob( tmp );
-                free_object( tmp );
-                sprintf( message, "You destroy the wall and redo the floor." );
-                }
-            else if ( ( FLOOR == tmp->type ) || ( QUERY_FLAG(tmp, FLAG_IS_FLOOR ) ) )
-                {
-                remove_ob( tmp );
-                free_object( tmp );
-                floor_removed = 1;
-                }
-            else
-                {
-                if ( floor_removed )
-                    above_floor = tmp;
-                }
-
-            tmp = above;
+            /* There was a wall, remove it & keep its archetype to make new walls */
+            new_wall = tmp->arch;
+            remove_ob( tmp );
+            free_object( tmp );
+            sprintf( message, "You destroy the wall and redo the floor." );
+            wall_removed = 1;
+            if ( floor != NULL ) {
+                remove_ob(floor);
+                free_object(floor);
             }
+            }
+        else if ( ( FLOOR == tmp->type ) || ( QUERY_FLAG(tmp, FLAG_IS_FLOOR ) ) ) 
+            {
+            floor = tmp;
+            }
+        else
+            {
+            if ( floor != NULL )
+                above_floor = tmp;
+            }
+
+        tmp = above;
         }
 
+    if ( wall_removed == 0 && floor != NULL ) {
+        if ( floor->arch->name == material->slaying ) {
+            draw_ext_info( NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_BUILD, "You feel too lazy to redo the exact same floor.", NULL);
+            return;
+        }
+    }
+        
     /* Now insert our floor */
     new_floor = find_archetype( material->slaying );
     if ( !new_floor )
@@ -568,15 +586,14 @@ void apply_builder_item( object* pl, object* item, short x, short y )
     if ( !arch )
         return;
 
-    tmp = arch_to_object( arch );
-    
-    if ( ( floor->above ) && ( !can_build_over(pl->map, tmp, x, y) ) )
-        /* Floor has something on top that interferes with building */
+    tmp = object_create_arch(arch);
+
+    if ( !can_build_over(pl->map, tmp, x, y) )
         {
         new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
         return;
         }
-    
+
     SET_FLAG( tmp, FLAG_IS_BUILDABLE );
     SET_FLAG( tmp, FLAG_NO_PICK );
 
@@ -704,7 +721,6 @@ void apply_map_builder( object* pl, int dir )
     {
     object* builder;
     object* tmp;
-    object* tmp2;
     short x, y;
 
     if ( !pl->type == PLAYER )
@@ -746,26 +762,21 @@ void apply_map_builder( object* pl, int dir )
         new_draw_info( NDI_UNIQUE, 0, pl, "You'd better not build here, it looks weird." );
         return;
         }
-    tmp2 = find_marked_object( pl );
-    while ( tmp )
-        {
-        if ( !QUERY_FLAG( tmp, FLAG_IS_BUILDABLE ) && ( ( tmp->type != SIGN )
-	|| ( strcmp( tmp->arch->name, "rune_mark" ) ) ) )
-            {
-	        /* The item building function already has it's own special
-		 * checks for this
-		 */
-	        if ((!tmp2) || (tmp2->subtype != ST_MAT_ITEM ))
-		    {
-	            new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
-	            return;
-		    }
+
+    builder = pl->contr->ranges[ range_builder ];
+
+    if (builder->subtype != ST_BD_BUILD) {
+        while ( tmp ) {
+            if ( !QUERY_FLAG( tmp, FLAG_IS_BUILDABLE ) && ( ( tmp->type != SIGN )
+                || ( strcmp( tmp->arch->name, "rune_mark" ) ) ) ) {
+		new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
+                return;
             }
-        tmp = tmp->above;
+            tmp = tmp->above;
         }
+    }
 
     /* Now we know the square is ok */
-    builder = pl->contr->ranges[ range_builder ];
 
     if ( builder->subtype == ST_BD_REMOVE )
         /* Remover -> call specific function and bail out */
@@ -780,7 +791,7 @@ void apply_map_builder( object* pl, int dir )
          * Find marked item to build, call specific function
          */
         {
-        tmp = tmp2;
+        tmp = find_marked_object(pl);
         if ( !tmp )
             {
 	        new_draw_info( NDI_UNIQUE, 0, pl, "You need to mark raw materials to use." );
