@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_chat_c =
- *   "$Id: c_chat.c,v 1.23 2006/05/05 09:47:40 ryo_saeba Exp $";
+ *   "$Id: c_chat.c 5803 2007-03-17 22:22:29Z ryo_saeba $";
  */
 
 /*
@@ -126,20 +126,30 @@ int command_chat (object *op, char *params)
     return command_tell_all(op, params, 9, NDI_BLUE, "chats");
 }
 
-
-
-int command_tell (object *op, char *params)
-{
+/**
+ * Actual function sending a private message.
+ *
+ * @param op
+ * player trying to tell something to someone.
+ * @param params
+ * who to tell, and message
+ * @param adjust_listen
+ * if non-zero, recipient can't ignore the message through 'listen' levels.
+ * @return
+ * 1.
+ */
+static int do_tell(object* op, char* params, int adjust_listen) {
     char buf[MAX_BUF],*name = NULL ,*msg = NULL;
     player *pl;
+    uint8 original_listen;
 
     if ( params != NULL){
         name = params;
         msg = strchr(name, ' ');
-        if(msg){
-	     *(msg++)=0;
-	     if(*msg == 0)
-		msg = NULL;
+        if(msg) {
+            *(msg++)=0;
+            if (*msg == 0)
+                msg = NULL;
         }
     }
 
@@ -156,21 +166,73 @@ int command_tell (object *op, char *params)
     pl = find_player_partial_name( name );
 
     if ( pl )
-        {
-	    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, buf);
-	    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op, 
-			     "You tell %s: %s", pl->ob->name, msg);
-
-	    /* Update last_tell value [mids 01/14/2002] */
-	    strcpy(pl->last_tell, op->name);
-	    return 1;
+    {
+        if (adjust_listen) {
+            original_listen = pl->listening;
+            pl->listening = 10;
         }
 
+	    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, buf);
+
+        if (adjust_listen)
+            pl->listening = original_listen;
+
+        /* Update last_tell value [mids 01/14/2002] */
+        snprintf(pl->last_tell, sizeof(pl->last_tell), op->name);
+
+        /* Hidden DMs get the message, but player should think DM isn't online. */
+        if (!pl->hidden || QUERY_FLAG(op, FLAG_WIZ)) {
+            new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op,
+                "You tell %s: %s", pl->ob->name, msg);
+            return 1;
+        }
+    }
+
     new_draw_info(NDI_UNIQUE, 0,op,"No such player or ambiguous name.");
     return 1;
 }
 
-/* Reply to last person who told you something [mids 01/14/2002] */
+/**
+ * Private communication.
+ *
+ * @param op
+ * player trying to tell something to someone.
+ * @param params
+ * who to tell, and message.
+ * @return
+ * 1.
+ */
+int command_tell (object *op, char *params)
+{
+    do_tell(op, params, 0);
+}
+
+/**
+ * Private communication, by a DM (can't be ignored by player).
+ *
+ * @param op
+ * player trying to tell something to someone.
+ * @param params
+ * who to tell, and message.
+ * @return
+ * 1.
+ */
+int command_dmtell (object *op, char *params) {
+    do_tell(op, params, 1);
+}
+
+/**
+ * Reply to last person who told you something [mids 01/14/2002]
+ *
+ * Must have been told something by someone first.
+ *
+ * @param op
+ * who is telling.
+ * @param params
+ * message to say.
+ * @return
+ * 1.
+ */
 int command_reply (object *op, char *params) {
     player *pl;
 
@@ -197,8 +259,14 @@ int command_reply (object *op, char *params) {
 
     new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, 
 	"%s tells you: %s", op->name, params);
-    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op, 
-		  "You tell to %s: %s", pl->ob->name, params);
+
+    if (pl->hidden && !QUERY_FLAG(op, FLAG_WIZ)) {
+        new_draw_info(NDI_UNIQUE, 0, op, "You can't reply, this player left.");
+        return 1;
+    }
+
+    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op,
+        "You tell to %s: %s", pl->ob->name, params);
     return 1;
 }
 
