@@ -1,11 +1,11 @@
 /*
  * static char *rcsid_c_object_c =
- *   "$Id: c_object.c,v 1.82 2006/06/02 08:19:35 akirschbaum Exp $";
+ *   "$Id: c_object.c 5669 2007-03-04 10:47:14Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2002,2007 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -542,7 +542,7 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
      * containers not allowed as of now)
      */
     if((pl->move_type & MOVE_FLYING) && !QUERY_FLAG(pl, FLAG_WIZ) && 
-       is_player_inv(tmp)!=pl) {
+       get_player_container(tmp)!=pl) {
 	new_draw_info(NDI_UNIQUE, 0,pl, "You are levitating, you can't reach the ground!");
 	return;
     }
@@ -604,10 +604,16 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 		query_cost_string(tmp,pl,F_BUY | F_SHOP));
     else
 	(void) sprintf(buf,"You pick up the %s.", query_name(tmp));
+
+    /* Now item is about to be picked. */
+    if (execute_event(tmp, EVENT_PICKUP, pl, op, NULL, SCRIPT_FIX_ALL) != 0)
+        return;
+
     new_draw_info(NDI_UNIQUE, 0,pl,buf);
 
     tmp = insert_ob_in_ob(tmp, op);
 
+
     /* All the stuff below deals with client/server code, and is only
      * usable by players
      */
@@ -733,61 +739,74 @@ void pick_up(object *op,object *alt)
 int command_take (object *op, char *params)
 {
     object *tmp, *next;
+    int ival;
+    int missed = 0;
 
     if (op->container) 
-	tmp=op->container->inv;
+        tmp=op->container->inv;
     else {
-	tmp=op->above;
-	if (tmp) while (tmp->above) {
-	    tmp=tmp->above;
-	}
-	if (!tmp)
-	    tmp=op->below;
+        tmp=op->above;
+        if (tmp) while (tmp->above) {
+            tmp=tmp->above;
+        }
+        if (!tmp)
+            tmp=op->below;
     }
 
     if (tmp==NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Nothing to take!");
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Nothing to take!");
+        return 0;
     }
 
     /* Makes processing easier */
     if (params && *params=='\0') params=NULL;
 
     while (tmp) {
-	next=tmp->below;
+        next=tmp->below;
 
-	if (tmp->invisible) {
-	    tmp=next;
-	    continue;
-	}
-	/* This following two if and else if could be merged into line
-	 * but that probably will make it more difficult to read, and
-	 * not make it any more efficient
-	 */
-	if (params && item_matched_string(op, tmp, params)) {
-	    pick_up(op, tmp);
-	}
-	else if (can_pick(op, tmp) && !params) {
-	    pick_up(op,tmp);
-	    break;
-	}
-	tmp=next;
-	/* Might as well just skip over the player immediately -
-	 * we know it can't be picked up
-	 */
-	if (tmp == op) tmp=tmp->below;
+        if (tmp->invisible) {
+            tmp=next;
+            continue;
+        }
+        /* This following two if and else if could be merged into line
+         * but that probably will make it more difficult to read, and
+         * not make it any more efficient
+         */
+        if (params && (ival=item_matched_string(op, tmp, params))>0) {
+            if ((ival<=2)&&(!can_pick(op,tmp)))
+            {
+                if(!QUERY_FLAG(tmp, FLAG_IS_FLOOR))/* don't count floor tiles */
+                    missed++;
+            }
+            else
+                pick_up(op, tmp);
+        }
+        else if (can_pick(op, tmp) && !params) {
+            pick_up(op,tmp);
+            break;
+        }
+        tmp=next;
+        /* Might as well just skip over the player immediately -
+         * we know it can't be picked up
+         */
+        if (tmp == op) tmp=tmp->below;
     }
     if (!params && !tmp) {
-	for (tmp=op->below; tmp!=NULL; tmp=tmp->next)
-	    if (!tmp->invisible) {
-		char buf[MAX_BUF];
-		sprintf(buf,"You can't pick up a %s.",
-		    tmp->name? tmp->name:"null");
-		new_draw_info(NDI_UNIQUE, 0,op, buf);
-		break;
-	    }
-	if (!tmp) new_draw_info(NDI_UNIQUE, 0,op, "There is nothing to pick up.");
+        for (tmp=op->below; tmp!=NULL; tmp=tmp->next)
+            if (!tmp->invisible) {
+                char buf[MAX_BUF];
+                sprintf(buf,"You can't pick up a %s.",
+                    tmp->name? tmp->name:"null");
+                new_draw_info(NDI_UNIQUE, 0,op, buf);
+                break;
+            }
+        if (!tmp) new_draw_info(NDI_UNIQUE, 0,op, "There is nothing to pick up.");
     }
+    if (missed==1)
+        new_draw_info(NDI_UNIQUE, 0, op, "You were unable to take one of the items.");
+    else if (missed>1)
+        new_draw_info_format(NDI_UNIQUE, 0, op,
+            "You were unable to take %d of the items.",missed);
     return 0;
 }
 
@@ -904,22 +923,22 @@ void put_object_in_sack (object *op, object *sack, object *tmp, uint32 nrof)
  *  nrof objects is tried to dropped.
  * This is used when dropping objects onto the floor.
  */
-void drop_object (object *op, object *tmp, uint32 nrof) 
+object *drop_object (object *op, object *tmp, uint32 nrof) 
 {
     char buf[MAX_BUF];
-    object *floor;
+    tag_t tmp_tag;
 
     if (QUERY_FLAG(tmp, FLAG_NO_DROP)) {
 #if 0
       /* Eneq(@csd.uu.se): Objects with NO_DROP defined can't be dropped. */
       new_draw_info(NDI_UNIQUE, 0,op, "This item can't be dropped.");
 #endif
-      return;
+      return NULL;
     }
 
     if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
       if (apply_special (op, tmp, AP_UNAPPLY | AP_NO_MERGE))
-          return;		/* can't unapply it */
+          return NULL;		/* can't unapply it */
     }
 
     /* We are only dropping some of the items.  We split the current objec
@@ -931,7 +950,7 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 	tmp = get_split_ob (tmp, nrof);
 	if(!tmp) {
 	    new_draw_info(NDI_UNIQUE, 0,op, errmsg);
-	    return;
+	    return NULL;
 	}
 	/* Tell a client what happened rest of objects.  tmp2 is now the
 	 * original object
@@ -948,7 +967,7 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 
     /* Lauwenmark: Handle for plugin drop event */
     if (execute_event(tmp, EVENT_DROP,op,NULL,NULL,SCRIPT_FIX_ALL)!= 0)
-        return;
+        return NULL;
 
     if (QUERY_FLAG (tmp, FLAG_STARTEQUIP)) {
       sprintf(buf,"You drop the %s.", query_name(tmp));
@@ -958,7 +977,7 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 	esrv_del_item (op->contr, tmp->count);
       free_object(tmp);
       fix_player(op);
-      return;
+      return NULL;
     }
 
 /*  If SAVE_INTERVAL is commented out, we never want to save
@@ -970,25 +989,24 @@ void drop_object (object *op, object *tmp, uint32 nrof)
      * of what he is dropping?
      */
     if (op->type == PLAYER && !QUERY_FLAG(tmp, FLAG_UNPAID) &&
-      (tmp->nrof ? tmp->value * tmp->nrof : tmp->value > 2000) &&
-      (op->contr->last_save_time + SAVE_INTERVAL) <= time(NULL)) {
-	  save_player(op, 1);
-	  op->contr->last_save_time = time(NULL);
+        (tmp->nrof ? tmp->value * tmp->nrof : tmp->value > 2000) &&
+        (op->contr->last_save_time + SAVE_INTERVAL) <= time(NULL)) {
+        save_player(op, 1);
+        op->contr->last_save_time = time(NULL);
     }
 #endif /* SAVE_INTERVAL */
 
 
-    floor = get_map_ob (op->map, op->x, op->y);
-    if( floor && floor->type == SHOP_FLOOR &&
-       !QUERY_FLAG(tmp, FLAG_UNPAID) && tmp->type != MONEY)
-      sell_item(tmp,op);
-
     tmp->x = op->x;
     tmp->y = op->y;
 
     if (op->type == PLAYER)
         esrv_del_item (op->contr, tmp->count);
+    tmp_tag = tmp->count;
     insert_ob_in_map(tmp, op->map, op,0);
+    if (!was_destroyed(tmp, tmp_tag) && !QUERY_FLAG(tmp, FLAG_UNPAID) && tmp->type != MONEY && is_in_shop(op)) {
+        sell_item(tmp, op);
+    }
 
 
     SET_FLAG (op, FLAG_NO_APPLY);
@@ -1008,6 +1026,7 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 	/* Need to update the weight for the player */
 	esrv_send_item (op, op);
     }
+    return tmp;
 }
 
 void drop(object *op, object *tmp)
@@ -1199,27 +1218,37 @@ int command_drop (object *op, char *params)
 {
     object  *tmp, *next;
     int did_one=0;
+    int ival=0;
+    int missed = 0;
 
     if (!params) {
-	new_draw_info(NDI_UNIQUE,0, op, "Drop what?");
-	return 0;
+        new_draw_info(NDI_UNIQUE,0, op, "Drop what?");
+        return 0;
     } else {
-	for (tmp=op->inv; tmp; tmp=next) {
-	    next=tmp->below;
-	    if (QUERY_FLAG(tmp,FLAG_NO_DROP) ||
-		tmp->invisible) continue;
-	    if (item_matched_string(op,tmp,params)) {
-		drop(op, tmp);
-		did_one=1;
-	    }
-	}
-	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop.");
+        for (tmp=op->inv; tmp; tmp=next) {
+            next=tmp->below;
+            if (QUERY_FLAG(tmp,FLAG_NO_DROP) || tmp->invisible) continue;
+            if ((ival = item_matched_string(op,tmp,params))>0) {
+                if ((QUERY_FLAG(tmp, FLAG_INV_LOCKED))&&((ival==1)||(ival==2)))
+                    missed++;
+                else
+                    drop(op, tmp);
+                did_one=1;
+            }
+        }
+        if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop.");
+        if (missed==1)
+            new_draw_info(NDI_UNIQUE, 0,op,
+                "One item couldn't be dropped because it was locked.");
+        else if (missed>1)
+            new_draw_info_format(NDI_UNIQUE, 0, op,
+                "%d items couldn't be dropped because they were locked.",missed);
     }
     if (op->type==PLAYER)
     {
         op->contr->count=0;
         op->contr->socket.update_look=1;
-    };
+    }
 /*    draw_look(op);*/
     return 0;
 }
@@ -1400,6 +1429,7 @@ const char *long_desc(const object *tmp, const object *pl) {
 
 void examine(object *op, object *tmp) {
     char buf[VERY_BIG_BUF];
+    int in_shop;
     int i;
 
     if (tmp == NULL || tmp->type == CLOSE_CON)
@@ -1485,13 +1515,13 @@ void examine(object *op, object *tmp) {
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
 
+    in_shop = is_in_shop(op);
+
     if (tmp->value && !QUERY_FLAG(tmp, FLAG_STARTEQUIP) && !QUERY_FLAG(tmp, FLAG_NO_PICK)) {
-    	object *floor;
     	sprintf(buf,"You reckon %s worth %s.",
 		    tmp->nrof>1?"they are":"it is",query_cost_string(tmp,op,F_SELL | F_APPROX));
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
-	floor = get_map_ob (op->map, op->x, op->y);
-	if (floor && floor->type == SHOP_FLOOR) {
+	if (in_shop) {
 	    if(QUERY_FLAG(tmp, FLAG_UNPAID))
 	    	sprintf(buf,"%s would cost you %s.",
 		    tmp->nrof>1?"They":"It",query_cost_string(tmp,op,F_BUY | F_SHOP));
@@ -1513,8 +1543,8 @@ void examine(object *op, object *tmp) {
      * types - especially if the first entry is a match
      */
     if(tmp->msg && tmp->type != EXIT && tmp->type != BOOK && 
-       tmp->type != CORPSE && !tmp->move_on && 
-       strncasecmp(tmp->msg, "@match",7)) {
+       tmp->type != CORPSE && !tmp->move_on &&
+       strncasecmp(tmp->msg, "@match",6)) {
 
 	/* This is just a hack so when identifying hte items, we print
 	 * out the extra message
@@ -1629,6 +1659,11 @@ static void display_new_pickup(const object* op)
     new_draw_info_format(NDI_UNIQUE, 0,op,"%d NOT CURSED", i & PU_NOT_CURSED?1:0);
 
     new_draw_info_format(NDI_UNIQUE, 0,op,"%d JEWELS", i & PU_JEWELS?1:0);
+    
+    draw_ext_info_format(NDI_UNIQUE, 0,op,MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+             "%d FLESH",
+             "%d FLESH",
+             i & PU_FLESH?1:0);
 
     new_draw_info_format(NDI_UNIQUE, 0,op,"");
     }
@@ -1639,11 +1674,13 @@ int command_pickup (object *op, char *params)
   static const char* names[ ] = {
       "debug", "inhibit", "stop", "food", "drink", "valuables", "bow", "arrow", "helmet",
       "shield", "armour", "boots", "gloves", "cloak", "key", "missile", "allweapon",
-      "magical", "potion", "spellbook", "skillscroll", "readables", "magicdevice", "notcursed", "jewels", NULL };
-  static uint32 modes[ ] = {
+      "magical", "potion", "spellbook", "skillscroll", "readables", "magicdevice",
+      "notcursed", "jewels", "flesh", NULL };
+      static uint32 modes[ ] = {
       PU_DEBUG, PU_INHIBIT, PU_STOP, PU_FOOD, PU_DRINK, PU_VALUABLES, PU_BOW, PU_ARROW, PU_HELMET,
       PU_SHIELD, PU_ARMOUR, PU_BOOTS, PU_GLOVES, PU_CLOAK, PU_KEY, PU_MISSILEWEAPON, PU_ALLWEAPON,
-      PU_MAGICAL, PU_POTION, PU_SPELLBOOK, PU_SKILLSCROLL, PU_READABLES, PU_MAGIC_DEVICE, PU_NOT_CURSED, PU_JEWELS, 0 };
+      PU_MAGICAL, PU_POTION, PU_SPELLBOOK, PU_SKILLSCROLL, PU_READABLES, PU_MAGIC_DEVICE,
+      PU_NOT_CURSED, PU_JEWELS, PU_FLESH, 0 };
 
   if(!params) {
     /* if the new mode is used, just print the settings */
@@ -1916,3 +1953,52 @@ int command_rename_item(object *op, char *params)
 
   return 1;
 }
+
+/**
+ * Alternate way to lock/unlock items (command line).
+ *
+ * @param op
+ * player
+ * @param params
+ * sent command line.
+ */
+int command_lock_item(object *op, char *params) {
+    object* item;
+    object* tmp;
+    tag_t tag;
+
+    if (!params || strlen(params) == 0) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+            "Lock what item?", "Lock what item?");
+        return 1;
+    }
+
+    item = find_best_object_match(op, params);
+    if (!item) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+            "Can't find any matching item.", "Can't find any matching item.");
+        return 1;
+    }
+
+    if (QUERY_FLAG(item, FLAG_INV_LOCKED)) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+            "Unlocked %s.", "Unlocked %s.", query_short_name(item));
+        CLEAR_FLAG(item,FLAG_INV_LOCKED);
+    } else {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+            "Locked %s.", "Locked %s.", query_short_name(item));
+        SET_FLAG(item,FLAG_INV_LOCKED);
+    }
+
+    tag = item->count;
+    tmp = merge_ob(item, NULL);
+    if (tmp == NULL) {
+        /* object was not merged */
+        esrv_update_item(UPD_FLAGS, op, item);
+    } else {
+        /* object was merged into tmp */
+        esrv_del_item(op->contr, tag);
+        esrv_send_item(op, tmp);
+    }
+    return 1;
+}
