@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_party_c =
- *   "$Id: c_party.c,v 1.24 2005/12/17 19:45:00 ryo_saeba Exp $";
+ *   "$Id: c_party.c 5127 2006-11-19 20:15:44Z akirschbaum $";
  */
 
 /*
@@ -69,7 +69,7 @@ void remove_party(partylist *target_party) {
     player *pl;
     
     if (firstparty==NULL) {
-	LOG(llevError, "remove_party(): I was asked to remove party %s, but no parties are defined",
+	LOG(llevError, "remove_party(): I was asked to remove party %s, but no parties are defined\n",
 	    target_party->partyname);
 	return;
     }
