@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c,v 1.84 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: c_wiz.c 5813 2007-03-17 23:38:55Z ryo_saeba $";
  */
 
 /*
@@ -285,7 +285,7 @@ int command_kick(object *op, const char *params) {
     return 1;
 }
 
-int command_save_overlay(object *op, char *params) {
+int command_overlay_save(object *op, char *params) {
     if (!op)
         return 0;
 
@@ -296,15 +296,33 @@ int command_save_overlay(object *op, char *params) {
     }
 
     new_save_map(op->map, 2);
-    new_save_map(op->map, 0);
+    // No need to save again the map and reset it. OVerlay saving is non destructive.
+    // new_save_map(op->map, 0);
+    // This fixes bug #1553636 (Crashbug: reset/swaped map after use of "overlay_save")
+    // Ryo 2006-10-22
     new_draw_info(NDI_UNIQUE, 0, op, "Current map has been saved as an"
         " overlay.");
 
-    ready_map_name(op->map->path, 0);
+    //ready_map_name(op->map->path, 0);
 
     return 1;
 }
 
+int command_overlay_reset(object *op, char* params) {
+    char filename[MAX_BUF];
+    struct stat stats;
+    strcpy(filename, create_overlay_pathname(op->map->path));
+    if (!stat(filename, &stats))
+        if (!unlink(filename))
+            new_draw_info(NDI_UNIQUE, 0, op, "Overlay successfully removed.");
+        else
+            new_draw_info(NDI_UNIQUE, 0, op, "Overlay couldn't be removed.");
+    else
+        new_draw_info(NDI_UNIQUE, 0, op, "No overlay for current map.");
+    
+    return 1;
+}
+
 /*
  * A simple toggle for the no_shout field.
  * AKA the MUZZLE command
@@ -506,6 +524,13 @@ int command_summon(object *op, char *params) {
 
 /**
  * Teleport next to target player.
+ *
+ * @param op
+ * DM teleporting.
+ * @param params
+ * options sent by player.
+ * @return
+ * 0 if couldn't teleport, 1 if teleport successful.
  */
 /* mids 01/16/2002 */
 int command_teleport(object *op, char *params) {
@@ -518,17 +543,20 @@ int command_teleport(object *op, char *params) {
 
     if (params == NULL) {
         new_draw_info(NDI_UNIQUE, 0, op, "Usage: teleport <player>.");
-        return 1;
+        return 0;
     }
 
-    pl = get_other_player_from_name(op, params);
-    if (!pl)
-        return 1;
+    pl = find_player_partial_name(params);
+    if (!pl) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+            "No such player or ambiguous name.", NULL);
+        return 0;
+    }
 
    i = find_free_spot(pl->ob, pl->ob->map, pl->ob->x, pl->ob->y, 1, 9);
    if (i == -1) {
       new_draw_info(NDI_UNIQUE, 0, op, "Can not find a free spot to teleport to.");
-      return 1;
+      return 0;
    }
 
    dummy = get_object();
@@ -643,7 +671,8 @@ int command_create(object *op, char *params) {
                     bp2 = cp+strlen(spell_name)+1;
                 else
                     at_spell = NULL;
-            }
+            } else
+                at_spell = NULL;
         }
 
         /* OK - we didn't find a spell - presume the 'of'
@@ -1058,6 +1087,12 @@ int command_remove (object *op, char *params) {
     /* Always work on the head - otherwise object will get in odd state */
     if (tmp->head)
         tmp = tmp->head;
+    if (tmp->speed != 0) {
+        tmp->speed = 0;
+        update_ob_speed(tmp);
+    }
+    if (!tmp->invisible && tmp->env && tmp->env->type == PLAYER)
+        esrv_del_item(tmp->env->contr, tmp->count);
     remove_ob(tmp);
     return 1;
 }
@@ -1077,13 +1112,14 @@ int command_free(object *op, char *params) {
         /* Item is either stack top, or is a number thus is now stack top, let's remove it  */
         dm_stack_pop(op->contr);
 
+    if (tmp->head)
+        tmp = tmp->head;
+
     if (!QUERY_FLAG(tmp, FLAG_REMOVED)) {
         new_draw_info(NDI_UNIQUE, 0, op, "Warning, item wasn't removed.");
         remove_ob(tmp);
     }
 
-    if (tmp->head)
-        tmp = tmp->head;
     free_object(tmp);
     return 1;
 }
@@ -1097,8 +1133,9 @@ int command_addexp(object *op, char *params) {
     object *skillob = NULL;
     player *pl;
 
-    if (params == NULL || ((q = sscanf(params, "%s %d", buf, &i)) < 2)) {
-        new_draw_info(NDI_UNIQUE, 0, op, "Usage: addexp [who] [how much].");
+    skill[0] = '\0';
+    if ((params == NULL) || (strlen(params) > MAX_BUF) || ((q = sscanf(params, "%s %d %s", buf, &i, skill)) < 2)) {
+        new_draw_info(NDI_UNIQUE, 0, op, "Usage: addexp player quantity [skill].");
         return 1;
     }
 
@@ -1264,6 +1301,15 @@ int command_reset (object *op, char *params) {
         return 1;
     }
 
+    /* Forbid using reset on our own map when we're in a transport, as
+     * it has the displeasant effect of crashing the server.
+     * - gros, July 25th 2006 */
+    if ((op->contr && op->contr->transport)&&(op->map == m))
+    {
+        new_draw_info(NDI_UNIQUE, 0, op, "You need to disembark first.");
+        return 1;
+    }
+
     if (m->in_memory != MAP_SWAPPED) {
         if (m->in_memory != MAP_IN_MEMORY) {
             LOG(llevError, "Tried to swap out map which was not in memory.\n");
@@ -1275,6 +1321,20 @@ int command_reset (object *op, char *params) {
          * players or wiz's.
          */
         if (op->map == m) {
+            if (strncmp(m->path, "/random/", 8)==0) {
+            /* This is not a very satisfying solution - it would be much better
+             * to recreate a random map with the same seed value as the old one.
+             * Unfortunately, I think recreating the map would require some
+             * knowledge about its 'parent', which appears very non-trivial to
+             * me.
+             * On the other hand, this should prevent the freeze that this
+             * situation caused. - gros, 26th July 2006.
+             */
+                new_draw_info(NDI_UNIQUE, 0, op,
+                    "You cannot reset a random map when inside it.");
+                return 1;
+            }
+
             dummy = get_object();
             dummy->map = NULL;
             EXIT_X(dummy) = op->x;
@@ -1340,6 +1400,9 @@ int command_nowiz(object *op, char *params) { /* 'noadm' is alias */
         op->invisible = 1;
     } else
         new_draw_info(NDI_UNIQUE|NDI_ALL|NDI_LT_GREEN, 1, NULL, "The Dungeon Master is gone..");
+
+    update_los(op);
+
     return 1;
 }
 
@@ -2004,3 +2067,43 @@ int command_insert_into(object* op, char *params)
     return 0;
 
 }
+
+int command_style_map_info(object *op, char *params)
+{
+    extern mapstruct *styles;
+    mapstruct	*mp;
+    int	    maps_used=0, mapmem=0, objects_used=0, x,y;
+    object  *tmp;
+
+    for (mp = styles; mp!=NULL; mp=mp->next) {
+	maps_used++;
+	mapmem += MAP_WIDTH(mp)*MAP_HEIGHT(mp)*(sizeof(object *)+sizeof(MapSpace)) + sizeof(mapstruct);
+	for (x=0; x<MAP_WIDTH(mp); x++) {
+	    for (y=0; y<MAP_HEIGHT(mp); y++) {
+		for (tmp=get_map_ob(mp, x, y); tmp!=NULL; tmp=tmp->above) 
+		    objects_used++;
+	    }
+	}
+    }
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+		 "[fixed]Style maps loaded:    %d", 
+		 "Style maps loaded:    %d", 
+		 maps_used);
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+		  "[fixed]Memory used, not", 
+		  "Memory used, not");
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+		 "[fixed]including objects:    %d", 
+		 "including objects:    %d", 
+		 mapmem);
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+		 "Style objects:        %d", 
+		 "Style objects:        %d", 
+		 objects_used);
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+		 "Mem for objects:      %d",
+		 "Mem for objects:      %d",
+		 objects_used * sizeof(object));
+    return 0;
+}
+
