@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.63 2006/06/06 21:38:41 akirschbaum Exp $";
+ *   "$Id: commands.c 5803 2007-03-17 22:22:29Z ryo_saeba $";
  */
 
 /*
@@ -80,6 +80,7 @@ command_array_struct Commands[] = {
   {"invoke", command_invoke,	1.0},
   {"killpets", command_kill_pets,0.0},
   {"listen", command_listen,	0.0},
+  {"lock", command_lock_item,	0.0},
   {"logs", command_logs,	0.0},
   {"malloc", command_malloc,	0.0},
   {"maps", command_maps,	0.0},
@@ -90,6 +91,7 @@ command_array_struct Commands[] = {
   {"news", command_news,	0.0},
   {"output-sync", command_output_sync,	0.0},
   {"output-count", command_output_count,0.0},
+  {"passwd", command_passwd, 0.0},
   {"peaceful", command_peaceful,0.0},
   {"pickup", command_pickup,	1.0},
   {"players", command_players,	0.0},
@@ -226,6 +228,7 @@ command_array_struct WizCommands [] = {
   {"create", command_create,0.0},
   {"debug", command_debug,0.0},
   {"diff", command_diff, 0.0 },
+  {"dmtell", command_dmtell,0.0},
   {"dump", command_dump,0.0},
   {"dumpbelow", command_dumpbelow,0.0},
   {"dumpfriendlyobjects", command_dumpfriendlyobjects,0.0},
@@ -265,8 +268,9 @@ command_array_struct WizCommands [] = {
   {"toggle_shout", command_toggle_shout,0.0},
   {"wizpass", command_wizpass,0.0},
   {"wizcast", command_wizcast,0.0},
-  {"overlay_save", command_save_overlay, 0.0},
-/*  {"possess", command_possess, 0.0}, */
+  {"overlay_save", command_overlay_save, 0.0},
+  {"overlay_reset", command_overlay_reset, 0.0},
+  /*  {"possess", command_possess, 0.0}, */
   {"mon_aggr", command_mon_aggr, 0.0},
   {"loadtest", command_loadtest, 0.0},
 };
