@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_daemon_c =
- *   "$Id: daemon.c,v 1.8 2006/05/05 09:47:40 ryo_saeba Exp $";
+ *   "$Id: daemon.c 4961 2006-09-21 05:14:18Z mwedel $";
  */
 
 /*
@@ -103,7 +103,7 @@ FILE *become_daemon (const char *filename)
     /*
      * Set up the standard file descriptors.
      */
-    (void) open ("/", O_RDONLY);	/* root inode already in core */
+    (void) open ("/dev/null", O_RDONLY);	/* root inode already in core */
     (void) dup2 (0, 1);
     (void) dup2 (0, 2);
 
