@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_gods_c =
- *   "$Id: gods.c,v 1.61 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: gods.c 5564 2007-02-18 18:34:13Z ryo_saeba $";
  */
 
 /*
@@ -202,6 +202,54 @@ static void follower_remove_similar_item (object *op, object *item)
     }
 }
 
+/**
+ * Removes from a player's inventory all items bestowed by
+ * a particular god.  Intended mainly for use in punishing
+ * characters for switching gods.  Should make the buggy
+ * follower_remove_similar_item obsolete.
+ *
+ * @param pl
+ *     the player object. May not be null.
+ *
+ * @param op
+ *     the object to be searched for items. May not be null.
+ *
+ * @param god
+ *     the god whose objects to remove. May not be null.
+ *
+ */
+
+static void follower_remove_given_items (object *pl, object *op, object *god)
+{
+    object *tmp, *next;
+    const char* given_by;
+
+    /* search the inventory */
+    for (tmp = op->inv; tmp != NULL; tmp = next) {
+        next = tmp->below;   /* backup in case we remove tmp */
+
+        given_by = get_ob_key_value(tmp, "divine_giver_name");
+        if(given_by == god->name){
+            /* Send the client a message. */
+            if (tmp->nrof > 1)
+                draw_ext_info_format(NDI_UNIQUE,0,pl, MSG_TYPE_ITEM, MSG_TYPE_ITEM_REMOVE,
+                                     "The %s crumble to dust!",
+                                     "The %s crumble to dust!",
+                                     query_short_name(tmp));
+            else
+                draw_ext_info_format(NDI_UNIQUE,0,pl, MSG_TYPE_ITEM, MSG_TYPE_ITEM_REMOVE,
+                                     "The %s crumbles to dust!",
+                                     "The %s crumbles to dust!",
+                                     query_short_name(tmp));
+                
+            remove_ob(tmp);    /* remove obj from players inv. */
+            esrv_del_item(pl->contr, tmp->count); /* notify client */
+            free_object(tmp);
+        } else if (tmp->inv)
+            follower_remove_given_items(pl, tmp, god);
+    }
+}
+
 /**
  * Checks for any occurrence of the given 'item' in the inventory of 'op' (recursively).
  * Returns 1 if found, else 0.
@@ -234,8 +282,14 @@ static int god_gives_present (object *op, object *god, treasure *tr)
         return 0;
 
     tmp = arch_to_object (tr->item);
-    new_draw_info_format (NDI_UNIQUE, 0, op,
-        "%s lets %s appear in your hands.", god->name, query_short_name (tmp));
+    draw_ext_info_format (NDI_UNIQUE, 0, op, MSG_TYPE_ITEM, MSG_TYPE_ITEM_ADD,
+			  "%s lets %s appear in your hands.",
+			  god->name, query_short_name (tmp),
+			  god->name, query_short_name (tmp));
+    /**
+     * Mark what god gave it, so it can be taken vengefully later!
+     */
+    set_ob_key_value(tmp, "divine_giver_name", god->name, TRUE);
     tmp = insert_ob_in_ob (tmp, op);
     if (op->type == PLAYER)
         esrv_send_item (op, tmp);
@@ -409,7 +463,8 @@ void become_follower (object *op, object *new_god) {
     treasure *tr;
     object *item, *skop, *next;
     int i,sk_applied;
-    
+    int undeadified = 0; /* Turns to true if changing god can changes the undead
+                          * status of the player.*/
     old_god = find_god(determine_god(op));
     
     /* take away any special god-characteristic items. */
@@ -431,12 +486,14 @@ void become_follower (object *op, object *new_god) {
 	}
     }
     
-    /* remove any godgiven items from the old god */
+    /* remove any items given by the old god */
     if (old_god) {
-        for(tr=old_god->randomitems->items; tr!=NULL; tr = tr->next) {
-	    if (tr->item && QUERY_FLAG(&tr->item->clone, FLAG_STARTEQUIP))
-	        follower_remove_similar_item(op, &tr->item->clone);
-	}
+        /* Changed to use the new "divine_giver_name" key_value
+         *   so it can reliably delete enchanted items.  Now it loops
+         *   through the player's inventory, instead of the god's
+         *   treasure list.
+         */
+        follower_remove_given_items(op, op, old_god);
     }
     
     if(!op||!new_god) return;
@@ -476,11 +533,21 @@ void become_follower (object *op, object *new_god) {
 
     sk_applied=QUERY_FLAG(skop,FLAG_APPLIED); /* save skill status */
 
+    /* Clear the "undead" status. We also need to force a call to change_abil,
+     * so I set undeadified for that.
+     * - gros, 21th July 2006.
+     */
+    if ((old_god)&&(QUERY_FLAG(old_god,FLAG_UNDEAD)))
+    {
+        CLEAR_FLAG(skop,FLAG_UNDEAD);
+        undeadified=1;
+    }
+
     if(skop->title) { /* get rid of old god */ 
 	new_draw_info_format(NDI_UNIQUE,0,op,
 	       "%s's blessing is withdrawn from you.",skop->title);
 	/* The point of this is to really show what abilities the player just lost */
-	if (sk_applied) {
+	if (sk_applied||undeadified) {
 	    CLEAR_FLAG(skop,FLAG_APPLIED); 
 	    (void) change_abil(op,skop);
 	}
@@ -517,6 +584,9 @@ void become_follower (object *op, object *new_god) {
     update_priest_flag(new_god,skop,FLAG_UNDEAD);
     update_priest_flag(new_god,skop,FLAG_BLIND);
     update_priest_flag(new_god,skop,FLAG_XRAYS); /* better have this if blind! */
+    update_priest_flag(new_god,skop,FLAG_USE_WEAPON);
+    update_priest_flag(new_god,skop,FLAG_USE_ARMOUR);
+    update_priest_flag(new_god,skop,FLAG_USE_SHIELD);
 
     new_draw_info_format(NDI_UNIQUE,0,op,
 	"You are bathed in %s's aura.",new_god->name);
@@ -540,9 +610,11 @@ void become_follower (object *op, object *new_god) {
 	stop_using_item(op,HELMET,1);
 	stop_using_item(op,BOOTS,1);
 	stop_using_item(op,GLOVES,1);
-	stop_using_item(op,SHIELD,1);
     }
 
+    if(worship_forbids_use(op,skop,FLAG_USE_SHIELD,"shield"))
+    	stop_using_item(op,SHIELD,1);
+
     SET_FLAG(skop,FLAG_APPLIED);
     (void) change_abil(op,skop);
 
@@ -1069,7 +1141,7 @@ static const char *get_god_for_race(const char *race) {
 
     if (race == NULL) return NULL;
     while(gl) {
-	if (!strcasecmp(gl->arch->clone.race, race)) {
+	if (gl->arch->clone.race && !strcasecmp(gl->arch->clone.race, race)) {
 	    godname=gl->name;
 	    break;
 	}
