@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.78 2006/06/01 18:59:14 tchize Exp $";
+ *   "$Id: init.c 5555 2007-02-16 00:06:50Z ryo_saeba $";
  */
 
 /*
@@ -701,6 +701,23 @@ static void load_settings(void)
 		LOG(llevError, "load_settings: unknown value for no_player_stealing: %s\n", cp);
         }
 
+    } else if ( !strcasecmp( buf, "create_home_portals" ) ) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.create_home_portals = TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.create_home_portals = FALSE;
+	    } else {
+		LOG(llevError, "load_settings: unknown value for create_home_portals: %s\n", cp);
+        }
+
+    } else if ( !strcasecmp( buf, "allow_denied_spells_writing" ) ) {
+            if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+                settings.allow_denied_spells_writing = TRUE;
+            } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+                settings.allow_denied_spells_writing = FALSE;
+            } else {
+                LOG(llevError, "load_settings: unknown value for allow_denied_spells_writing: %s\n", cp);
+        }
     } else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
@@ -741,6 +758,7 @@ void init(int argc, char **argv) {
     init_commands();	/* Sort command tables */
     read_map_log();	/* Load up the old temp map files */
     init_skills();
+    cftimer_init();
 
     parse_args(argc, argv, 3);
 
