@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c,v 1.101 2006/06/01 18:59:13 tchize Exp $";
+ *    "$Id: monster.c 5544 2007-02-15 19:44:29Z ryo_saeba $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2002,2007 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -1139,7 +1139,14 @@ static int monster_can_pick(object *monster, object *item) {
     if (monster->pick_up&64)           /* All */
 	flag=1;
 
-    else switch(item->type) {
+    else {
+        if (IS_WEAPON(item))
+            flag=(monster->pick_up&8)||QUERY_FLAG(monster,FLAG_USE_WEAPON);
+        else if (IS_ARMOR(item))
+            flag=(monster->pick_up&16)||QUERY_FLAG(monster,FLAG_USE_ARMOUR);
+        else if (IS_SHIELD(item))
+            flag=(monster->pick_up&16)||QUERY_FLAG(monster,FLAG_USE_SHIELD);
+        else switch(item->type) {
 	case MONEY:
 	case GEM:
 	    flag=monster->pick_up&2;
@@ -1149,19 +1156,6 @@ static int monster_can_pick(object *monster, object *item) {
 	    flag=monster->pick_up&4;
 	    break;
 
-	case WEAPON:
-	    flag=(monster->pick_up&8)||QUERY_FLAG(monster,FLAG_USE_WEAPON);
-	    break;
-
-	case ARMOUR:
-	case SHIELD:
-	case HELMET:
-	case BOOTS:
-	case GLOVES:
-	case GIRDLE:
-	    flag=(monster->pick_up&16)||QUERY_FLAG(monster,FLAG_USE_ARMOUR);
-	    break;
-
 	case SKILL:
 	    flag=QUERY_FLAG(monster,FLAG_CAN_USE_SKILL);
 	    break;
@@ -1199,6 +1193,7 @@ static int monster_can_pick(object *monster, object *item) {
 	    break;
 	}
     }
+    }
 
     if (((!(monster->pick_up&32))&&flag) || ((monster->pick_up&32)&&(!flag)))
 	return 1;
@@ -1280,14 +1275,14 @@ void monster_check_apply(object *mon, object *item) {
     else if (item->type == FOOD && mon->will_apply & WILL_APPLY_FOOD) flag=1;
     else if (item->type == SCROLL && QUERY_FLAG(mon, FLAG_USE_SCROLL)) {
         if (!item->inv)
-            LOG(llevDebug,"Monster %d having scroll %d with empty inventory!", mon->count, item->count);
+            LOG(llevDebug,"Monster %d having scroll %d with empty inventory!\n", mon->count, item->count);
         else if (monster_should_cast_spell(mon, item->inv))
             SET_FLAG(mon, FLAG_READY_SCROLL);
         /* Don't use it right now */
         return;
     }
     else if (item->type == WEAPON) flag = check_good_weapon(mon,item);
-    else if (IS_ARMOR(item)) flag = check_good_armour(mon,item);
+    else if (IS_ARMOR(item) || IS_SHIELD(item)) flag = check_good_armour(mon,item);
     /* Should do something more, like make sure this is a better item */
     else if (item->type == RING)
         flag=1;
@@ -1543,7 +1538,7 @@ static char *find_matching_message(object* pl, const char *msg, const char *matc
 
     while (1) {
 	if (strncmp(cp, "@match ", 7)) {
-	    LOG(llevDebug,"find_matching_message: Invalid message %s", msg);
+	    LOG(llevDebug,"find_matching_message: Invalid message %s\n", msg);
 	    return NULL;
 	}
 	else {
@@ -1656,23 +1651,27 @@ void communicate(object *op, const char *txt) {
 static int do_talk_npc(object* op, object* npc, object* override, const char* txt)
 {
     char* cp;
-    char buf[MAX_BUF];
 
     if(override->msg == NULL || *override->msg != '@')
 	return 0;
 
     cp = find_matching_message(op, override->msg, txt);
     if (cp) {
-        sprintf(buf,"%s says:",query_name(npc));
-	    new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map,buf);
-	    new_info_map(NDI_NAVY | NDI_UNIQUE, npc->map, cp);
+        npc_say(npc, cp);
         quest_apply_items(override,op->contr);
-	    free(cp);
+        free(cp);
         return 1;
     }
     return 0;
 }
 
+void npc_say(object *npc, char *cp) {
+    char buf[MAX_BUF];
+    snprintf(buf, sizeof(buf), "%s says:", query_name(npc));
+    new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, buf);
+    new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, cp);
+}
+
 static int talk_to_npc(object *op, object *npc, const char *txt) {
     object *cobj;
 
