@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_pets_c =
- *    "$Id: pets.c,v 1.44 2006/06/08 04:30:17 qal21 Exp $";
+ *    "$Id: pets.c 5206 2006-12-26 19:22:49Z ryo_saeba $";
  */
 
 /*
@@ -255,9 +255,11 @@ void remove_all_pets(mapstruct *map) {
     if(obl->ob->type != PLAYER && QUERY_FLAG(obl->ob,FLAG_FRIENDLY) &&
        (owner = get_owner(obl->ob)) != NULL && !on_same_map(owner, obl->ob))
     {
-	/* follow owner checks map status for us */
+	/* follow owner checks map status for us.  Note that pet can
+	 * die in follow_owner, so check for obl->ob existence
+	 */
 	follow_owner(obl->ob,owner);
-	if(QUERY_FLAG(obl->ob, FLAG_REMOVED) && FABS(obl->ob->speed) > MIN_ACTIVE_SPEED) {
+	if(obl->ob && QUERY_FLAG(obl->ob, FLAG_REMOVED) && FABS(obl->ob->speed) > MIN_ACTIVE_SPEED) {
 	    object *ob = obl->ob;
 	    LOG(llevMonster,"(pet failed to follow)\n");
 	    remove_friendly_object(ob);
@@ -411,50 +413,61 @@ void pet_move(object * ob)
 static object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem) {
     archetype *atmp;
     object *tmp=NULL, *prev=NULL, *head=NULL; 
- 
+
     for(atmp = at; atmp!=NULL; atmp = atmp->more) {
-	tmp = arch_to_object(atmp);
-	if (atmp == at) {
-	    if(!is_golem) 
-		SET_FLAG(tmp, FLAG_MONSTER);
-	    set_owner(tmp, op);
-	    if (op->type == PLAYER) {
-		tmp->stats.exp = 0;
-		add_friendly_object(tmp);
-		SET_FLAG(tmp, FLAG_FRIENDLY);
-		if(is_golem) CLEAR_FLAG(tmp, FLAG_MONSTER);
-	    } else { 
-		if(QUERY_FLAG(op, FLAG_FRIENDLY)) {
-		    object *owner = get_owner(op);
-		    if(owner != NULL) {/* For now, we transfer ownership */
-			set_owner(tmp,owner);
-			tmp->attack_movement = PETMOVE;
-			add_friendly_object(tmp);
-			SET_FLAG(tmp, FLAG_FRIENDLY);
-		    }
-		}
-	    }
-	    if(op->type!=PLAYER || !is_golem) { 
-		tmp->attack_movement = PETMOVE;
-		tmp->speed_left = -1;
-		tmp->type = 0;
-		tmp->enemy = op->enemy; 
-	    } else
-		tmp->type = GOLEM;
-       
-	}
-	if(head == NULL)
-	    head = tmp;
-	tmp->x = op->x + freearr_x[dir] + tmp->arch->clone.x;
-	tmp->y = op->y + freearr_y[dir] + tmp->arch->clone.y;
-	tmp->map = op->map;
-	if(tmp->invisible) tmp->invisible=0;
-	if(head != tmp)
-	    tmp->head = head, prev->more = tmp;
-	prev = tmp;
+        tmp = arch_to_object(atmp);
+        if (atmp == at) {
+            if(!is_golem)
+                SET_FLAG(tmp, FLAG_MONSTER);
+            set_owner(tmp, op);
+            if (op->type == PLAYER) {
+                tmp->stats.exp = 0;
+                add_friendly_object(tmp);
+                SET_FLAG(tmp, FLAG_FRIENDLY);
+                if(is_golem) CLEAR_FLAG(tmp, FLAG_MONSTER);
+            } else if(QUERY_FLAG(op, FLAG_FRIENDLY)) {
+                object *owner = get_owner(op);
+                if(owner != NULL) {/* For now, we transfer ownership */
+                    set_owner(tmp,owner);
+                    tmp->attack_movement = PETMOVE;
+                    add_friendly_object(tmp);
+                    SET_FLAG(tmp, FLAG_FRIENDLY);
+                }
+            }
+            if(op->type!=PLAYER || !is_golem) {
+                tmp->attack_movement = PETMOVE;
+                tmp->speed_left = -1;
+                tmp->type = 0;
+                tmp->enemy = op->enemy;
+            } else
+                tmp->type = GOLEM;
+        }
+        if(head == NULL)
+            head = tmp;
+        tmp->x = op->x + freearr_x[dir] + tmp->arch->clone.x;
+        tmp->y = op->y + freearr_y[dir] + tmp->arch->clone.y;
+        tmp->map = op->map;
+        if(tmp->invisible) tmp->invisible=0;
+        if(head != tmp)
+            tmp->head = head, prev->more = tmp;
+        prev = tmp;
     }
     head->direction = dir;
 
+    if ((head->type != GOLEM)&&(op->type!= PLAYER)) {
+        /* It isn't a golem, and the caster wasn't a player; that's probably
+         * a monster generated from a trap, thus we should give it treasures
+         * and equipment - gros, 12th August 2006
+         */
+        if (head && head->randomitems) {
+            object *htmp;
+            create_treasure(head->randomitems, head, GT_APPLY | GT_STARTEQUIP,
+                6, 0);
+            for (htmp=head->inv; htmp; htmp=htmp->below)
+                if (!htmp->nrof) SET_FLAG(htmp, FLAG_NO_DROP);
+        }
+    }
+
     /* need to change some monster attr to prevent problems/crashing */
     head->last_heal=0;
     head->last_eat=0;
@@ -785,7 +798,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 {
     sint16 x,y, nrof=1, i;
     archetype *summon_arch;
-    int ndir;
+    int ndir, mult;
 
     if (spell_ob->other_arch) {
 	summon_arch = spell_ob->other_arch;
@@ -866,13 +879,27 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 	return 0;
     }
 
+    if (dir) {
+        // Only fail if caster specified a blocked direction.
+        x = freearr_x[dir];
+        y = freearr_y[dir];
+        if (ob_blocked(&summon_arch->clone, op->map, op->x + x, op->y + y)) {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE,
+                        "There is something in the way.", NULL);
+
+            return 0;
+        }
+    }
+
+    mult = (RANDOM()%2 ? -1 : 1);
+
     for (i=1; i <= nrof; i++) {
 	archetype *atmp;
 	object *prev=NULL, *head=NULL, *tmp;
 
         if (dir) {
-            ndir = dir;
-            dir = absdir (dir+1);
+            ndir = absdir (dir + (i / 2) * mult);
+            mult = - mult;
         } else
             ndir = find_free_spot(&summon_arch->clone, op->map, op->x, op->y, 1, SIZEOFFREE);
 
@@ -881,13 +908,8 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
             y = freearr_y[ndir];
         }
 
-        if (ndir == -1 || ob_blocked(&summon_arch->clone, op->map, op->x + x, op->y + y)){
-            new_draw_info(NDI_UNIQUE, 0, op, "There is something in the way.");
-            if (nrof > 1)
-		new_draw_info(NDI_UNIQUE, 0,op, "No more pets for this casting.");
-
-            return nrof > 1;
-        }
+        if (ndir == -1 || ob_blocked(&summon_arch->clone, op->map, op->x + x, op->y + y))
+            continue;
 
 	for (atmp = summon_arch; atmp!=NULL; atmp=atmp->more) {
 	    tmp = arch_to_object(atmp);
