@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_plugins_c =
- *   "$Id: plugins.c,v 1.72 2006/05/20 15:47:48 ryo_saeba Exp $";
+ *   "$Id: plugins.c 5796 2007-03-17 21:29:46Z ryo_saeba $";
  */
 
 /*****************************************************************************/
@@ -41,9 +41,10 @@
 
 #ifndef __CEXTRACT__
 #include <sproto.h>
+#include <timers.h>
 #endif
 
-#define NR_OF_HOOKS 76
+#define NR_OF_HOOKS 80
 
 static const hook_entry plug_hooks[NR_OF_HOOKS] =
 {
@@ -123,6 +124,10 @@ static const hook_entry plug_hooks[NR_OF_HOOKS] =
     {cfapi_region_get_property,     73, "cfapi_region_get_property"},
     {cfapi_player_can_pay,          74, "cfapi_player_can_pay"},
     {cfapi_log,                     75, "cfapi_log"},
+    {cfapi_get_time,                76, "cfapi_system_get_time"},
+    {cfapi_timer_create,            77, "cfapi_system_timer_create"},
+    {cfapi_timer_destroy,           78, "cfapi_system_timer_destroy"},
+    {cfapi_friendlylist_get_next,   79, "cfapi_friendlylist_get_next"},
 };
 int plugin_number = 0;
 crossfire_plugin* plugins_list = NULL;
@@ -159,7 +164,7 @@ static void send_changed_object(object *op)
     player *pl;
 
     if (op->env != NULL) {
-        tmp = is_player_inv(op->env);
+        tmp = get_player_container(op->env);
         if (!tmp) {
             for (pl = first_player; pl; pl = pl->next)
                 if (pl->ob->container == op->env)
@@ -194,7 +199,7 @@ static void send_removed_object(object *op)
         return;
     }
 
-    tmp = is_player_inv(op->env);
+    tmp = get_player_container(op->env);
     if (!tmp) {
         for (pl = first_player; pl; pl = pl->next)
             if (pl->ob->container == op->env)
@@ -266,6 +271,7 @@ int execute_event(object* op, int eventcode, object* activator, object* third, c
 int execute_global_event(int eventcode, ...)
 {
     va_list args;
+    mapstruct* map;
     object* op;
     object* op2;
     player* pl;
@@ -342,29 +348,31 @@ int execute_global_event(int eventcode, ...)
         break;
 
     case EVENT_MAPENTER:
-        /*MAPENTER: op*/
+        /*MAPENTER: op, map*/
         op = va_arg(args, object*);
+        map = va_arg(args, mapstruct*);
         for (cp = plugins_list; cp != NULL; cp = cp->next) {
             if (cp->gevent[eventcode] != NULL)
-                cp->gevent[eventcode](&rt, eventcode, op);
+                cp->gevent[eventcode](&rt, eventcode, op, map);
         }
         break;
 
     case EVENT_MAPLEAVE:
-        /*MAPLEAVE: op*/
+        /*MAPLEAVE: op, map*/
         op = va_arg(args, object*);
+        map = va_arg(args, mapstruct*);
         for (cp = plugins_list; cp != NULL; cp = cp->next) {
             if (cp->gevent[eventcode] != NULL)
-                cp->gevent[eventcode](&rt, eventcode, op);
+                cp->gevent[eventcode](&rt, eventcode, op, map);
         }
         break;
 
     case EVENT_MAPRESET:
-        /*MAPRESET: map->path*/
-        buf = va_arg(args, char*);
+        /*MAPRESET: map*/
+        map = va_arg(args, mapstruct*);
         for (cp = plugins_list; cp != NULL; cp = cp->next) {
             if (cp->gevent[eventcode] != NULL)
-                cp->gevent[eventcode](&rt, eventcode, buf);
+                cp->gevent[eventcode](&rt, eventcode, map);
         }
         break;
 
@@ -414,6 +422,22 @@ int execute_global_event(int eventcode, ...)
                 cp->gevent[eventcode](&rt, eventcode, op, buf);
         }
         break;
+    case EVENT_MAPUNLOAD:
+        /*MAPUNLOAD: map*/
+        map = va_arg(args, mapstruct*);
+        for (cp = plugins_list; cp != NULL; cp = cp->next) {
+            if (cp->gevent[eventcode] != NULL)
+                cp->gevent[eventcode](&rt, eventcode, map);
+        }
+        break;
+    case EVENT_MAPLOAD:
+        /*MAPLOAD: map*/
+        map = va_arg(args, mapstruct*);
+        for (cp = plugins_list; cp != NULL; cp = cp->next) {
+            if (cp->gevent[eventcode] != NULL)
+                cp->gevent[eventcode](&rt, eventcode, map);
+        }
+        break;
     }
     va_end(args);
     return 0;
@@ -788,6 +812,62 @@ void* cfapi_system_directory(int* type, ...)
     return NULL;
 }
 
+void *cfapi_get_time(int *type, ...)
+{
+    va_list args;
+    timeofday_t *tod;
+
+    va_start(args,type);
+    tod = va_arg(args, timeofday_t*);
+    va_end(args);
+
+    get_tod(tod);
+    *type = CFAPI_NONE;
+    return NULL;
+}
+
+void *cfapi_timer_create(int *type, ...)
+{
+    va_list args;
+    int res;
+    static int rv;
+    object* ob;
+    long delay;
+    int mode;
+
+    va_start(args, type);
+    ob = va_arg(args, object*);
+    delay = va_arg(args, long);
+    mode = va_arg(args, int);
+    va_end(args);
+    *type = CFAPI_INT;
+
+    rv = cftimer_find_free_id();
+    if ( rv != TIMER_ERR_ID )
+    {
+        res = cftimer_create(rv, delay, ob, mode);
+        if ( res != TIMER_ERR_NONE )
+            rv = res;
+    }
+    return &rv;
+}
+
+void *cfapi_timer_destroy(int *type, ...)
+{
+    va_list args;
+    int id;
+    static int rv;
+
+    va_start(args, type);
+    id = va_arg(args, int);
+    va_end(args);
+    *type = CFAPI_INT;
+
+    rv = cftimer_destroy(id);
+
+    return &rv;
+}
+
 /* Logging hook */
 void* cfapi_log(int* type, ...)
 {
@@ -798,7 +878,7 @@ void* cfapi_log(int* type, ...)
     va_start(args, type);
     logLevel = va_arg(args, LogLevel);
     message = va_arg(args, const char*);
-    LOG(logLevel, message);
+    LOG(logLevel, "%s", message);
     va_end(args);
 
     return NULL;
@@ -1363,6 +1443,7 @@ void* cfapi_object_get_property(int* type, ...)
     object* op;
     void* rv;
     static int ri;
+    static float rf;
 
     va_start(args, type);
 
@@ -1483,12 +1564,12 @@ void* cfapi_object_get_property(int* type, ...)
 
         case CFAPI_OBJECT_PROP_SPEED:
             rv = &op->speed;
-            *type = CFAPI_DOUBLE;
+            *type = CFAPI_FLOAT;
             break;
 
         case CFAPI_OBJECT_PROP_SPEED_LEFT:
             rv = &op->speed_left;
-            *type = CFAPI_DOUBLE;
+            *type = CFAPI_FLOAT;
             break;
 
         case CFAPI_OBJECT_PROP_NROF:
@@ -1507,17 +1588,20 @@ void* cfapi_object_get_property(int* type, ...)
             break;
 
         case CFAPI_OBJECT_PROP_TYPE:
-            rv = &op->type;
+            ri = op->type;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
         case CFAPI_OBJECT_PROP_SUBTYPE:
-            rv = &op->subtype;
+            ri = op->subtype;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
         case CFAPI_OBJECT_PROP_CLIENT_TYPE:
-            rv = &op->client_type;
+            ri = op->client_type;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
@@ -1576,22 +1660,26 @@ void* cfapi_object_get_property(int* type, ...)
             break;
 
         case CFAPI_OBJECT_PROP_LAST_HEAL:
-            rv = &op->last_heal;
+            ri = op->last_heal;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
         case CFAPI_OBJECT_PROP_LAST_SP:
-            rv = &op->last_sp;
+            ri = op->last_sp;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
         case CFAPI_OBJECT_PROP_LAST_GRACE:
-            rv = &op->last_grace;
+            ri = op->last_grace;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
         case CFAPI_OBJECT_PROP_LAST_EAT:
-            rv = &op->last_eat;
+            ri = op->last_eat;
+            rv = &ri;
             *type = CFAPI_INT;
             break;
 
@@ -1675,7 +1763,7 @@ void* cfapi_object_get_property(int* type, ...)
             *type = CFAPI_INT;
             break;
 
-        case CFAPI_OBJECT_PROP_MOVE_TYPE:
+        case CFAPI_OBJECT_PROP_ATTACK_MOVEMENT:
             rv = &op->attack_movement;
             *type = CFAPI_INT;
             break;
@@ -1743,7 +1831,7 @@ void* cfapi_object_get_property(int* type, ...)
 
         case CFAPI_OBJECT_PROP_EXP:
             rv = &op->stats.exp;
-            *type = CFAPI_LONG;
+            *type = CFAPI_SINT64;
             break;
 
         case CFAPI_OBJECT_PROP_OWNER:
@@ -1957,6 +2045,61 @@ void* cfapi_object_get_property(int* type, ...)
             rv = (op->contr ? op->contr->party : NULL);
             *type = CFAPI_PPARTY;
             break;
+        case CFAPI_OBJECT_PROP_NO_SAVE:
+            ri = op->no_save;
+            rv = &ri;
+            *type = CFAPI_INT;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_TYPE:
+            ri = op->move_type;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_BLOCK:
+            ri = op->move_block;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_ALLOW:
+            ri = op->move_allow;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_ON:
+            ri = op->move_on;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_OFF:
+            ri = op->move_off;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_SLOW:
+            ri = op->move_type;
+            rv = &ri;
+            *type = CFAPI_CHAR;
+            break;
+        case CFAPI_OBJECT_PROP_MOVE_SLOW_PENALTY:
+            rf = op->move_slow_penalty;
+            rv = &rf;
+            *type = CFAPI_FLOAT;
+            break;
+        case CFAPI_PLAYER_PROP_BED_MAP:
+            ri = op->no_save;
+            rv = op->contr->savebed_map;
+            *type = CFAPI_STRING;
+            break;
+        case CFAPI_PLAYER_PROP_BED_X:
+            ri = op->contr->bed_x;
+            rv = &ri;
+            *type = CFAPI_INT;
+            break;
+        case CFAPI_PLAYER_PROP_BED_Y:
+            ri = op->contr->bed_y;
+            rv = &ri;
+            *type = CFAPI_INT;
+            break;
         default:
             *type = CFAPI_NONE;
             break;
@@ -1977,6 +2120,7 @@ void* cfapi_object_set_property(int* type, ...)
     object* op;
     int property;
     void* rv;
+    sint64 s64arg;
     partylist* partyarg;
     va_start(args, type);
 
@@ -2031,7 +2175,10 @@ void* cfapi_object_set_property(int* type, ...)
 
         case CFAPI_OBJECT_PROP_SPEED:
             darg = va_arg(args, double);
-            op->speed = darg;
+            if (darg != op->speed) {
+                op->speed = darg;
+                update_ob_speed(op);
+            }
             break;
 
         case CFAPI_OBJECT_PROP_SPEED_LEFT:
@@ -2050,7 +2197,7 @@ void* cfapi_object_set_property(int* type, ...)
                 player *pl;
                 op->nrof = iarg;
                 if (op->env != NULL) {
-                    tmp = is_player_inv(op->env);
+                    tmp = get_player_container(op->env);
                     if (!tmp) {
                         for (pl = first_player; pl; pl = pl->next)
                             if (pl->ob->container == op->env)
@@ -2186,7 +2333,7 @@ void* cfapi_object_set_property(int* type, ...)
                 player *pl;
                 op->weight = iarg;
                 if (op->env != NULL) {
-                    tmp = is_player_inv(op->env);
+                    tmp = get_player_container(op->env);
                     if (!tmp) {
                         for (pl = first_player; pl; pl = pl->next)
                             if (pl->ob->container == op->env)
@@ -2253,7 +2400,7 @@ void* cfapi_object_set_property(int* type, ...)
             op->move_status = iarg;
             break;
 
-        case CFAPI_OBJECT_PROP_MOVE_TYPE:
+        case CFAPI_OBJECT_PROP_ATTACK_MOVEMENT:
             iarg = va_arg(args, int);
             op->attack_movement = iarg;
             break;
@@ -2296,10 +2443,10 @@ void* cfapi_object_set_property(int* type, ...)
             {
                 char* skillname;
 
-                larg = va_arg(args, long);
+                s64arg = va_arg(args, sint64);
                 skillname = va_arg(args, char*);
                 iarg = va_arg(args, int);
-                change_exp(op, larg, skillname, iarg);
+                change_exp(op, s64arg, skillname, iarg);
             }
             break;
 
@@ -2438,7 +2585,22 @@ void* cfapi_object_set_property(int* type, ...)
                 op->contr->party = partyarg;
             }
             break;
-
+        case CFAPI_OBJECT_PROP_NO_SAVE:
+            iarg = va_arg(args, int);
+            op->no_save = iarg;
+            break;
+        case CFAPI_PLAYER_PROP_BED_MAP:
+            sarg = va_arg(args, char*);
+            strncpy(op->contr->savebed_map, sarg, MAX_BUF);
+            break;
+        case CFAPI_PLAYER_PROP_BED_X:
+            iarg = va_arg(args, int);
+            op->contr->bed_x = iarg;
+            break;
+        case CFAPI_PLAYER_PROP_BED_Y:
+            iarg = va_arg(args, int);
+            op->contr->bed_y = iarg;
+            break;
         default:
             *type = CFAPI_NONE;
             break;
@@ -2595,6 +2757,13 @@ void* cfapi_object_remove(int* type, ...)
 
     op = va_arg(args, object*);
 
+    if (QUERY_FLAG(op, FLAG_REMOVED))
+    {
+        LOG(llevError, "Plugin trying to remove removed object %s\n", op->name);
+        *type = CFAPI_NONE;
+        return NULL;
+    }
+
     va_end(args);
 
     send_removed_object(op);
@@ -2611,6 +2780,13 @@ void* cfapi_object_delete(int* type, ...)
 
     op = va_arg(args, object*);
 
+    if (QUERY_FLAG(op, FLAG_FREED) || !QUERY_FLAG(op, FLAG_REMOVED))
+    {
+        LOG(llevError, "Plugin trying to free freed/non removed object %s\n", op->name);
+        *type = CFAPI_NONE;
+        return NULL;
+    }
+
     va_end(args);
 
     free_object(op);
@@ -2676,7 +2852,7 @@ void* cfapi_object_find(int* type, ...)
 
     case 3:
         op = va_arg(args, object*);
-        rv = is_player_inv(op);
+        rv = get_player_container(op);
         break;
 
     default:
@@ -2710,12 +2886,13 @@ void* cfapi_object_create(int* type, ...)
             object* op;
             sval = va_arg(args, char*);
 
-            op = create_archetype_by_object_name(sval);
+            op = create_archetype(sval);
 
             if (strncmp(query_name(op), ARCH_SINGULARITY, ARCH_SINGULARITY_LEN) == 0) {
                 free_object(op);
                 /* Try with archetype names... */
-                op = create_archetype(sval);
+                op = create_archetype_by_object_name(sval);
+
                 if (strncmp(query_name(op), ARCH_SINGULARITY, ARCH_SINGULARITY_LEN) == 0) {
                     free_object(op);
                     *type = CFAPI_NONE;
@@ -3257,8 +3434,13 @@ void* cfapi_object_say(int* type, ...)
     op = va_arg(args, object*);
     msg = va_arg(args, char*);
     va_end(args);
-
-    rv = command_say(op, msg);
+    
+    if (op->type == PLAYER) {
+        rv = command_say(op, msg);
+    } else {
+        npc_say(op, msg);
+        rv = 0;
+    }
     *type = CFAPI_INT;
     return &rv;
 }
@@ -3363,14 +3545,18 @@ void* cfapi_object_teleport(int *type, ...)
             return &result;
         }
 
-        send_removed_object(who);
-        remove_ob(who);
+        if (!QUERY_FLAG(who, FLAG_REMOVED)) {
+            send_removed_object(who);
+            remove_ob(who);
+        }
 
         for (tmp = who; tmp != NULL; tmp = tmp->more)
             tmp->x = x+freearr_x[k]+(tmp->arch == NULL ? 0 : tmp->arch->clone.x),
             tmp->y = y+freearr_y[k]+(tmp->arch == NULL ? 0 : tmp->arch->clone.y);
 
         insert_ob_in_map(who, map, NULL, 0);
+        if (who->type == PLAYER)
+            map_newmap_cmd(who->contr);
         result = 0;
     }
 
@@ -3544,6 +3730,47 @@ void* cfapi_region_get_property(int* type, ...)
     return rv;
 }
 
+/**
+ * Friend list access, to get objects on it.
+ *
+ * Expects one parameter, ob.
+ *
+ * @param type
+ * unused
+ * @return
+ * - if ob is NULL, gets the first object on the friendlylist.
+ * - if not NULL, get next object on the friendlylist after ob. NULL if none or ob not on list.
+ */
+void *cfapi_friendlylist_get_next(int *type, ...)
+{
+    object* ob;
+    object* next;
+    va_list args;
+    objectlink* link;
+
+    va_start(args, type);
+    ob = va_arg(args, object*);
+    va_end(args);
+
+    if (ob) {
+        for (link = first_friendly_object; link; link = link->next) {
+            if (ob == link->ob) {
+                if (link->next)
+                    return link->next->ob;
+                else
+                    return NULL;
+            }
+        }
+        return NULL;
+    }
+
+    if (first_friendly_object)
+        return first_friendly_object->ob;
+
+    return NULL;
+
+}
+
 /*****************************************************************************/
 /* NEW PLUGIN STUFF ENDS HERE                                                */
 /*****************************************************************************/
