@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c,v 1.54 2006/05/05 09:47:40 ryo_saeba Exp $";
+ *   "$Id: shop.c 5550 2007-02-15 20:40:39Z ryo_saeba $";
  */
 
 /*
@@ -56,8 +56,9 @@ static uint64 value_limit(uint64 val, int quantity, const object *who, int issho
 static double shop_specialisation_ratio(const object *item, const mapstruct *map);
 static double shop_greed(const mapstruct *map);
 
-#define NUM_COINS 3	/* number of coin types */
-static const char* const coins[] = {"platinacoin", "goldcoin", "silvercoin", NULL};
+#define NUM_COINS 5	/* number of coin types */
+#define LARGEST_COIN_GIVEN 2 /* never give amber or jade, but accept them */
+static const char* const coins[] = {"ambercoin", "jadecoin","platinacoin", "goldcoin", "silvercoin", NULL};
 
 /* Added F_TRUE flag to define.h to mean that the price should not
  * be adjusted by players charisma. With F_TRUE, it returns the amount
@@ -338,7 +339,8 @@ static const char *cost_string_from_value(uint64 cost)
     static char buf[MAX_BUF];
     archetype *coin, *next_coin;
     char *endbuf;
-    int num, cointype = 0;
+    int num;
+    int cointype = LARGEST_COIN_GIVEN;
 
     coin = find_next_coin(cost, &cointype);
     if (coin == NULL)
@@ -415,7 +417,8 @@ const char *query_cost_string(const object *tmp,object *who,int flag) {
 	    if (!idskill2 || !find_skill_by_number(who, idskill2)) {
 		if (!find_skill_by_number(who,SK_BARGAINING)) {
 		    static char buf[MAX_BUF];
-		    int num, cointype = 0;
+		    int num;
+		    int cointype = LARGEST_COIN_GIVEN;
 		    archetype *coin = find_next_coin(real_value, &cointype);
 	
 		    if (coin == NULL) return "nothing";
@@ -479,12 +482,16 @@ int pay_for_amount(uint64 to_pay,object *pl) {
     to_pay = pay_from_container(pl, pl, to_pay);
 
     for (pouch=pl->inv; (pouch!=NULL) && (to_pay>0); pouch=pouch->below) {
-	if (pouch->type == CONTAINER
-	    && QUERY_FLAG(pouch, FLAG_APPLIED)
-	    && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
-	    to_pay = pay_from_container(pl, pouch, to_pay);
-	}
+        if (pouch->type == CONTAINER
+          && QUERY_FLAG(pouch, FLAG_APPLIED)
+          && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
+            to_pay = pay_from_container(pl, pouch, to_pay);
+        }
     }
+    if (to_pay > 0) {
+        LOG(llevError, "pay_for_amount: Cannot remove enough money -- %"FMT64U" remains\n", to_pay);
+    }
+
     fix_player(pl);
     return 1;
 }
@@ -514,18 +521,117 @@ int pay_for_item(object *op,object *pl) {
     to_pay = pay_from_container(pl, pl, to_pay);
 
     for (pouch=pl->inv; (pouch!=NULL) && (to_pay>0); pouch=pouch->below) {
-	if (pouch->type == CONTAINER
-	    && QUERY_FLAG(pouch, FLAG_APPLIED)
-	    && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
-	    to_pay = pay_from_container(pl, pouch, to_pay);
-	}
+        if (pouch->type == CONTAINER
+                && QUERY_FLAG(pouch, FLAG_APPLIED)
+                && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
+            to_pay = pay_from_container(pl, pouch, to_pay);
+        }
+    }
+    if (to_pay > 0) {
+        LOG(llevError, "pay_for_amount: Cannot remove enough money -- %"FMT64U" remains\n", to_pay);
     }
     if (settings.real_wiz == FALSE && QUERY_FLAG(pl, FLAG_WAS_WIZ))
-	SET_FLAG(op, FLAG_WAS_WIZ);
+        SET_FLAG(op, FLAG_WAS_WIZ);
     fix_player(pl);
     return 1;
 }
 
+/**
+ * This function removes a given amount from a list of coins.
+ *
+ * @param coin_objs the list coins to remove from; the list must be ordered
+ * from least to most valuable coin
+ *
+ * @param remain the value (in silver coins) to remove
+ *
+ * @return the value remaining
+ */
+static sint64 remove_value(object *coin_objs[], sint64 remain) {
+    int i;
+
+    for (i = 0; i < NUM_COINS; i++) {
+        int count;
+        int num_coins;
+
+        if (coin_objs[i]->nrof*coin_objs[i]->value > remain) {
+            num_coins = remain/coin_objs[i]->value;
+            if ((uint64)num_coins*(uint64)coin_objs[i]->value < remain) {
+                num_coins++;
+            }
+        } else {
+            num_coins = coin_objs[i]->nrof;
+        }
+        remain -= (sint64)num_coins*(sint64)coin_objs[i]->value;
+        coin_objs[i]->nrof -= num_coins;
+        /* Now start making change.  Start at the coin value
+         * below the one we just did, and work down to
+         * the lowest value.
+         */
+        count = i-1;
+        while (remain < 0 && count >= 0) {
+            num_coins = -remain/coin_objs[count]->value;
+            coin_objs[count]->nrof += num_coins;
+            remain += num_coins*coin_objs[count]->value;
+            count--;
+        }
+    }
+
+    return remain;
+}
+
+/**
+ * This function adds a given amount to a list of coins.
+ *
+ * @param coin_objs the list coins to add to; the list must be ordered
+ * from least to most valuable coin
+ *
+ * @param amount the value (in silver coins) to add
+ */
+static void add_value(object *coin_objs[], sint64 value) {
+    int i;
+
+    for (i = NUM_COINS-LARGEST_COIN_GIVEN-1; i >= 0; i--) {
+        uint32 nrof;
+
+        nrof = (uint32)(value/coin_objs[i]->value);
+        value -= nrof*coin_objs[i]->value;
+        coin_objs[i]->nrof += nrof;
+    }
+}
+
+/**
+ * Insert a list of objects into a player object.
+ *
+ * @param pl the player to add to
+ *
+ * @param container the container (inside the player object) to add to
+ *
+ * @param objects the list of objects to add; the objects will be either
+ * inserted into the player object or freed
+ *
+ * @param objects_len the length of objects
+ */
+static void insert_objects(object *pl, object *container, object *objects[], int objects_len) {
+    int i;
+
+    for (i = 0; i < objects_len; i++) {
+        if (objects[i]->nrof > 0) {
+            object *tmp = insert_ob_in_ob(objects[i], container);
+
+            esrv_send_item(pl, tmp);
+            esrv_send_item(pl, container);
+            if (pl != container) {
+                esrv_update_item(UPD_WEIGHT, pl, container);
+            }
+            if (pl->type != PLAYER) {
+                esrv_send_item(pl, pl);
+            }
+        } else {
+            free_object(objects[i]);
+        }
+    }
+}
+
 /* This pays for the item, and takes the proper amount of money off
  * the player.
  * CF 0.91.4 - this function is mostly redone in order to fix a bug
@@ -541,9 +647,11 @@ int pay_for_item(object *op,object *pl) {
  * returns the amount still missing after using "pouch".
  */
 static uint64 pay_from_container(object *pl, object *pouch, uint64 to_pay) {
-    int count, i;
+    int i;
     sint64 remain;
     object *tmp, *coin_objs[NUM_COINS], *next;
+    object *other_money[16]; /* collects MONEY objects not matching coins[] */
+    size_t other_money_len; /* number of allocated entries in other_money[] */
     archetype *at;
 
     if (pouch->type != PLAYER && pouch->type != CONTAINER) return to_pay;
@@ -552,89 +660,114 @@ static uint64 pay_from_container(object *pl, object *pouch, uint64 to_pay) {
     for (i=0; i<NUM_COINS; i++) coin_objs[i] = NULL;
 
     /* This hunk should remove all the money objects from the player/container */
+    other_money_len = 0;
     for (tmp=pouch->inv; tmp; tmp=next) {
-	next = tmp->below;
+        next = tmp->below;
+        if (tmp->type == MONEY) {
+            for (i=0; i<NUM_COINS; i++) {
+                if (!strcmp(coins[NUM_COINS-1-i], tmp->arch->name) &&
+                        (tmp->value == tmp->arch->clone.value) ) {
 
-	if (tmp->type == MONEY) {
-	    for (i=0; i<NUM_COINS; i++) {
-		if (!strcmp(coins[NUM_COINS-1-i], tmp->arch->name) &&
-		    (tmp->value == tmp->arch->clone.value) ) {
-
-		    /* This should not happen, but if it does, just		*
-		     * merge the two.						*/
-		    if (coin_objs[i]!=NULL) {
-			LOG(llevError,"%s has two money entries of (%s)\n",
-			    pouch->name, coins[NUM_COINS-1-i]);
-			remove_ob(tmp);
-			coin_objs[i]->nrof += tmp->nrof;
-			esrv_del_item(pl->contr, tmp->count);
-			free_object(tmp);
-		    }
-		    else {
-			remove_ob(tmp);
-			if(pouch->type==PLAYER) esrv_del_item(pl->contr, tmp->count);
-			coin_objs[i] = tmp;
-		    }
-		    break;
-		}
-	    }
-	    if (i==NUM_COINS)
-		LOG(llevError,"in pay_for_item: Did not find string match for %s\n", tmp->arch->name);
-	}
+                    /* This should not happen, but if it does, just		*
+                     * merge the two.						
+                     */
+                    if (coin_objs[i]!=NULL) {
+                        LOG(llevError,"%s has two money entries of (%s)\n",
+                            pouch->name, coins[NUM_COINS-1-i]);
+                        remove_ob(tmp);
+                        coin_objs[i]->nrof += tmp->nrof;
+                        esrv_del_item(pl->contr, tmp->count);
+                        free_object(tmp);
+                    } else {
+                        remove_ob(tmp);
+                        if(pouch->type==PLAYER) esrv_del_item(pl->contr, tmp->count);
+                        coin_objs[i] = tmp;
+                    }
+                    break;
+                }
+            }
+            if (i==NUM_COINS) {
+                if (other_money_len >= sizeof(other_money)/sizeof(*other_money)) {
+                    LOG(llevError, "pay_for_item: Cannot store non-standard money object %s\n", tmp->arch->name);
+                } else {
+                    remove_ob(tmp);
+                    if(pouch->type == PLAYER) {
+                        esrv_del_item(pl->contr, tmp->count);
+                    }
+                    other_money[other_money_len++] = tmp;
+                }
+            }
+        }
     }
 
-    /* Fill in any gaps in the coin_objs array - needed to make change. */
+    /* Fill in any gaps in the coin_objs array - needed to make change.      */
     /* Note that the coin_objs array goes from least value to greatest value */
     for (i=0; i<NUM_COINS; i++)
-	if (coin_objs[i]==NULL) {
-	    at = find_archetype(coins[NUM_COINS-1-i]);
-	    if (at==NULL) LOG(llevError, "Could not find %s archetype\n", coins[NUM_COINS-1-i]);
-	    coin_objs[i] = get_object();
-	    copy_object(&at->clone, coin_objs[i]);
-	    coin_objs[i]->nrof = 0;
-	}
+        if (coin_objs[i]==NULL) {
+            at = find_archetype(coins[NUM_COINS-1-i]);
+            if (at==NULL) LOG(llevError, "Could not find %s archetype\n", coins[NUM_COINS-1-i]);
+            coin_objs[i] = get_object();
+            copy_object(&at->clone, coin_objs[i]);
+            coin_objs[i]->nrof = 0;
+        }
 
-    for (i=0; i<NUM_COINS; i++) {
-	int num_coins;
+    /* Try to pay from standard coins first. */
+    remain = remove_value(coin_objs, remain);
 
-	if (coin_objs[i]->nrof*coin_objs[i]->value> remain) {
-	    num_coins = remain / coin_objs[i]->value;
-	    if ((uint64)num_coins*(uint64)coin_objs[i]->value < remain) num_coins++;
-	} else {
-	    num_coins = coin_objs[i]->nrof;
-	}
-	remain -= (sint64) num_coins * (sint64)coin_objs[i]->value;
-	coin_objs[i]->nrof -= num_coins;
-	/* Now start making change.  Start at the coin value
-	 * below the one we just did, and work down to
-	 * the lowest value.
+    /* Now pay from non-standard coins until all is paid. */
+    for (i = 0; i < other_money_len && remain > 0; i++) {
+        uint32 nrof;
+        object *coin;
+
+        coin = other_money[i];
+
+        /* Find the minimal number of coins to use. This prevents converting
+	 * excess non-standard coins to standard money.
 	 */
-	count=i-1;
-	while (remain<0 && count>=0) {
-		num_coins = -remain/ coin_objs[count]->value;
-		coin_objs[count]->nrof += num_coins;
-		remain += num_coins * coin_objs[count]->value;
-		count--;
-	}
-    }
-    for (i=0; i<NUM_COINS; i++) {
-	if (coin_objs[i]->nrof) {
-	    object *tmp = insert_ob_in_ob(coin_objs[i], pouch);
+        nrof = (remain+coin->value-1)/coin->value;
+        if (nrof > coin->nrof) {
+            nrof = coin->nrof;
+        }
+        coin->nrof -= nrof;
+        add_value(coin_objs, nrof*coin->value);
 
-	    esrv_send_item(pl, tmp);
-	    esrv_send_item (pl, pouch);
-	    if (pl != pouch) esrv_update_item (UPD_WEIGHT, pl, pouch);
-	    if (pl->type != PLAYER) {
-		esrv_send_item (pl, pl);
-	    }
-	} else {
-	    free_object(coin_objs[i]);
-	}
+        remain = remove_value(coin_objs, remain);
     }
+
+    /* re-insert remaining coins into player */
+    insert_objects(pl, pouch, coin_objs, NUM_COINS);
+    insert_objects(pl, pouch, other_money, other_money_len);
+
     return(remain);
 }
 
-/* Checks all unpaid items in op's inventory, adds up all the money they
+/**
+ * Helper function for can_pay. Checks all items from item and item->below, and recurse if inventory found.
+ * coincount is supposed to be of size NUM_COINS. Parameters can't be NULL.
+ */
+static void count_unpaid(object* pl, object* item, int* unpaid_count, uint64* unpaid_price, uint32* coincount)
+{
+    int i;
+    for (;item;item = item->below) {
+        if QUERY_FLAG(item, FLAG_UNPAID) {
+            (*unpaid_count)++;
+            (*unpaid_price) += query_cost(item, pl, F_BUY | F_SHOP);
+        }
+        /* merely converting the player's monetary wealth won't do, if we did that, 
+        * we could print the wrong numbers for the coins, so we count the money instead 
+        */
+        for (i=0; i< NUM_COINS; i++)
+            if (!strcmp(coins[i], item->arch->name)) {
+                coincount[i] += item->nrof;
+                break;
+            }
+        if (item->inv) 
+            count_unpaid(pl, item->inv, unpaid_count, unpaid_price, coincount);
+    }
+}
+
+/**
+ * Checks all unpaid items in op's inventory, adds up all the money they
  * have, and checks that they can actually afford what they want to buy.
  * Returns 1 if they can, and 0 if they can't. also prints an appropriate message 
  * to the player
@@ -644,44 +777,39 @@ int can_pay(object *pl) {
     int unpaid_count = 0, i;
     uint64 unpaid_price = 0;
     uint64 player_wealth = query_money(pl);
-    object *item;
     uint32 coincount[NUM_COINS];
+
     if (!pl || pl->type != PLAYER) {
-	LOG(llevError, "can_pay(): called against something that isn't a player\n");
-	return 0;
-    }
-    for (i=0; i< NUM_COINS; i++) coincount[i] = 0; 
-    for (item = pl->inv;item;) {
-	if QUERY_FLAG(item, FLAG_UNPAID) {
-	    unpaid_count++;
-	    unpaid_price += query_cost(item, pl, F_BUY | F_SHOP);
-	}
-	/* merely converting the player's monetary wealth won't do, if we did that, 
-	 * we could print the wrong numbers for the coins, so we count the money instead 
-	 */
-	for (i=0; i< NUM_COINS; i++) 
-	    if (!strcmp(coins[i], item->arch->name)) 
-		coincount[i] += item->nrof;
-	if (item->inv) item = item->inv; 
-	else if (item->below) item = item->below;
-	else if (item->env && item->env != pl && item->env->below) item = item->env->below;
-	else item = NULL; 
+        LOG(llevError, "can_pay(): called against something that isn't a player\n");
+        return 0;
     }
+
+    for (i=0; i< NUM_COINS; i++)
+        coincount[i] = 0;
+
+    count_unpaid(pl, pl->inv, &unpaid_count, &unpaid_price, coincount);
+
     if (unpaid_price > player_wealth) {
-	char buf[MAX_BUF], coinbuf[MAX_BUF];
-	int denominations = 0;
-	sprintf(buf, "You have %d unpaid items that would cost you %s, but you only have", 
-	    unpaid_count, cost_string_from_value(unpaid_price));
-	for (i=0; i< NUM_COINS; i++) {
-	    if (coincount[i] > 0 && coins[i]) {
-		denominations++;
-		sprintf(coinbuf, " %d %s,", coincount[i], find_archetype(coins[i])->clone.name_pl);
-		strcat (buf, coinbuf);
-	    }
-	}
-	if (denominations > 1) make_list_like(buf);
-	new_draw_info(NDI_UNIQUE, 0, pl, buf);
-	return 0;
+        char buf[MAX_BUF], coinbuf[MAX_BUF];
+        int denominations = 0;
+        snprintf(buf, sizeof(buf), "You have %d unpaid items that would cost you %s, ", 
+                unpaid_count, cost_string_from_value(unpaid_price));
+        for (i=0; i< NUM_COINS; i++) {
+            if (coincount[i] > 0 && coins[i]) {
+                if (denominations == 0)
+                    snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "but you only have");
+                denominations++;
+                snprintf(coinbuf, sizeof(coinbuf), " %d %s,", coincount[i],
+                        find_archetype(coins[i])->clone.name_pl);
+                snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), coinbuf);
+            }
+        }
+        if (denominations == 0)
+            snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "but you don't have any money.");
+        else if (denominations > 1)
+            make_list_like(buf);
+        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_PAYMENT, buf, NULL);
+        return 0;
     }
     else return 1;
 }
@@ -711,35 +839,38 @@ int get_payment(object *pl, object *op) {
 
     if (!ret) 
         return 0;
-   
+
     if(op!=NULL&&QUERY_FLAG(op,FLAG_UNPAID)) {
         strncpy(buf,query_cost_string(op,pl,F_BUY | F_SHOP),MAX_BUF);
         buf[MAX_BUF-1] = '\0';
         if(!pay_for_item(op,pl)) {
             uint64 i=query_cost(op,pl,F_BUY | F_SHOP) - query_money(pl);
-	    CLEAR_FLAG(op, FLAG_UNPAID);
-	    new_draw_info_format(NDI_UNIQUE, 0, pl,
-		"You lack %s to buy %s.", cost_string_from_value(i),
-		query_name(op));
-	    SET_FLAG(op, FLAG_UNPAID);
+            CLEAR_FLAG(op, FLAG_UNPAID);
+	    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_PAYMENT, 
+                    "You lack %s to buy %s.", 
+                    "You lack %s to buy %s.", 
+		    cost_string_from_value(i), query_name(op));
+            SET_FLAG(op, FLAG_UNPAID);
             return 0;
         } else {
-	    object *tmp;
-	    tag_t c = op->count;
+            object *tmp;
+            tag_t c = op->count;
 
-	    CLEAR_FLAG(op, FLAG_UNPAID);
-	    CLEAR_FLAG(op, FLAG_PLAYER_SOLD);
-	    new_draw_info_format(NDI_UNIQUE, 0, op,
-		"You paid %s for %s.",buf,query_name(op));
-	    tmp=merge_ob(op,NULL);
-	    if (pl->type == PLAYER) {
-		if (tmp) {      /* it was merged */
-		    esrv_del_item (pl->contr, c);
-		    op = tmp;
-		}
-	        esrv_send_item(pl, op);
-	    }
-	}
+            CLEAR_FLAG(op, FLAG_UNPAID);
+            CLEAR_FLAG(op, FLAG_PLAYER_SOLD);
+	    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_PAYMENT, 
+                    "You paid %s for %s.",
+                    "You paid %s for %s.",
+		    buf,query_name(op));
+            tmp=merge_ob(op,NULL);
+            if (pl->type == PLAYER) {
+                if (tmp) {      /* it was merged */
+                    esrv_del_item (pl->contr, c);
+                    op = tmp;
+                }
+                esrv_send_item(pl, op);
+            }
+        }
     }
     return 1;
 }
@@ -766,8 +897,10 @@ void sell_item(object *op, object *pl) {
     if(op->custom_name) FREE_AND_CLEAR_STR(op->custom_name);
 
     if(!i) {
-	new_draw_info_format(NDI_UNIQUE, 0, pl,
-	     "We're not interested in %s.",query_name(op));
+	draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_SELL, 
+	     "We're not interested in %s.",
+	     "We're not interested in %s.",
+	     query_name(op));
 
 	/* Even if the character doesn't get anything for it, it may still be
 	 * worth something.  If so, make it unpaid
@@ -790,7 +923,7 @@ void sell_item(object *op, object *pl) {
     if (extra_gain > 0)
 	change_exp(pl,extra_gain/10,"bargaining",SK_EXP_NONE);
   
-    for (count=0; coins[count]!=NULL; count++) {
+    for (count=LARGEST_COIN_GIVEN; coins[count]!=NULL; count++) {
 	at = find_archetype(coins[count]);
 	if (at==NULL) LOG(llevError, "Could not find %s archetype\n", coins[count]);
 	else if ((i/at->clone.value) > 0) {
@@ -835,9 +968,11 @@ void sell_item(object *op, object *pl) {
 	LOG(llevError,"Warning - payment not zero: %I64u\n", i);
 #endif
 
-    new_draw_info_format(NDI_UNIQUE, 0, pl,
-	"You receive %s for %s.",query_cost_string(op,pl,F_SELL | F_SHOP),
-          query_name(op));
+    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_SELL, 
+	"You receive %s for %s.",
+	"You receive %s for %s.",
+	query_cost_string(op,pl,F_SELL | F_SHOP), query_name(op));
+
     SET_FLAG(op, FLAG_UNPAID);
     identify(op);
 }
@@ -946,11 +1081,15 @@ int describe_shop(const object *op) {
     int pos=0, i;
     double opinion=0;
     char tmp[MAX_BUF]="\0";
+
     if (op->type != PLAYER) return 0;
 
     /*check if there is a shop specified for this map */
     if (map->shopitems || map->shopgreed || map->shoprace || map->shopmin || map->shopmax) {
-	new_draw_info(NDI_UNIQUE,0,op,"From looking at the nearby shop you determine that it trades in:");
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING,
+		      "From looking at the nearby shop you determine that it trades in:", 
+		      NULL);
+
 	if (map->shopitems) {
 	    for (i=0; i < map->shopitems[0].index; i++) {
 		if (map->shopitems[i].name && map->shopitems[i].strength > 10) {
@@ -959,39 +1098,53 @@ int describe_shop(const object *op) {
 		}
 	    }
 	}
-	if (!pos) strcat(tmp, "a little of everything.");
+	if (!pos) strcpy(tmp, "a little of everything.");
 
 	/* format the string into a list */
 	make_list_like(tmp);
-	new_draw_info_format(NDI_UNIQUE, 0, op, "%s", tmp);
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, tmp, NULL);
 	
 	if (map->shopmax)
-	    new_draw_info_format(NDI_UNIQUE,0,op,"It won't trade for items above %s.",
+	    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+		"It won't trade for items above %s.",
+		"It won't trade for items above %s.",
 		cost_string_from_value(map->shopmax));
+
 	if (map->shopmin)
-	    new_draw_info_format(NDI_UNIQUE,0,op,"It won't trade in items worth less than %s.",
+	    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+		"It won't trade in items worth less than %s.",
+		"It won't trade in items worth less than %s.",
 		cost_string_from_value(map->shopmin));
+
 	if (map->shopgreed) {
 	    if (map->shopgreed >2.0)
-		new_draw_info(NDI_UNIQUE,0,op,"It tends to overcharge massively.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "It tends to overcharge massively.", NULL);
 	    else if (map->shopgreed >1.5)
-		new_draw_info(NDI_UNIQUE,0,op,"It tends to overcharge substantially.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "It tends to overcharge substantially.", NULL);
 	    else if (map->shopgreed >1.1)
-		new_draw_info(NDI_UNIQUE,0,op,"It tends to overcharge slightly.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "It tends to overcharge slightly.", NULL);
 	    else if (map->shopgreed <0.9)
-		new_draw_info(NDI_UNIQUE,0,op,"It tends to undercharge.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "It tends to undercharge.", NULL);
 	}
 	if (map->shoprace) {
 	    opinion=shopkeeper_approval(map, op);
 	    if (opinion > 0.8) 
-		new_draw_info(NDI_UNIQUE,0,op,"You think the shopkeeper likes you.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "You think the shopkeeper likes you.", NULL);
 	    else if (opinion > 0.5)
-		new_draw_info(NDI_UNIQUE,0,op,"The shopkeeper seems unconcerned by you.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "The shopkeeper seems unconcerned by you.", NULL);
 	    else
-		new_draw_info(NDI_UNIQUE,0,op,"The shopkeeper seems to have taken a dislike to you.");
+		draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+			      "The shopkeeper seems to have taken a dislike to you.", NULL);
 	}
     }
-    else new_draw_info(NDI_UNIQUE,0,op,"There is no shop nearby.");
+    else draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC, 
+		       "There is no shop nearby.", NULL);
 
     return 1;
 }
@@ -1072,7 +1225,8 @@ void shop_listing(object *op)
     /* Should never happen, but just in case a monster does apply a sign */
     if (op->type!=PLAYER) return;
 
-    new_draw_info(NDI_UNIQUE, 0, op, "\nThe shop contains:");
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, 
+		  "\nThe shop contains:", NULL);
 
     magic_mapping_mark(op, map_mark, 3);
     items=malloc(40*sizeof(shopinv));
@@ -1107,7 +1261,8 @@ void shop_listing(object *op)
     }
     free(map_mark);
     if (numitems == 0) {
-	new_draw_info(NDI_UNIQUE, 0, op, "The shop is currently empty.\n");
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, 
+		      "The shop is currently empty.\n", NULL);
 	free(items);
 	return;
     }
@@ -1120,12 +1275,29 @@ void shop_listing(object *op)
 	    free(items[i].item_sort);
 	    free(items[i].item_real);
 	} else {
-	    new_draw_info_format(NDI_UNIQUE, 0, op, "%d %s", 
-				 items[i].nrof? items[i].nrof:1,
-			 items[i].nrof==1?items[i].item_sort: items[i].item_real);
+	    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, 
+		"%d %s", "%d %s", 
+		 items[i].nrof? items[i].nrof:1,
+		 items[i].nrof==1?items[i].item_sort: items[i].item_real);
 	    free(items[i].item_sort);
 	    free(items[i].item_real);
 	}
     }
     free(items);
 }
+
+/* Check if an object is in a shop. Returns 1 if so, 0 otherwise. */
+int is_in_shop(object *ob) {
+    if (!ob->map)
+        return 0;
+    return coords_in_shop(ob->map, ob->x, ob->y);
+}
+
+/* Check if given map coords are in a shop. Returns 1 if so, 0 otherwise. */
+int coords_in_shop(mapstruct *map, int x, int y) {
+    object *floor;
+    for (floor = get_map_ob (map, x, y); floor; floor = floor->above)
+        if (floor->type == SHOP_FLOOR)
+            return 1;
+    return 0;
+}
