@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c,v 1.67 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: skill_util.c 5817 2007-03-18 00:06:07Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -248,12 +248,12 @@ int change_skill (object *who, object *new_skill, int flag)
     }
 
     if (!new_skill || who->chosen_skill) 
-	if (who->chosen_skill) apply_special(who, who->chosen_skill, AP_UNAPPLY);
+	if (who->chosen_skill) apply_special(who, who->chosen_skill, AP_UNAPPLY | (flag & AP_NOPRINT));
 
     /* Only goal in this case was to unapply a skill */
     if (!new_skill) return 0;
 
-    if (apply_special (who, new_skill, AP_APPLY)) {
+    if (apply_special (who, new_skill, AP_APPLY | (flag & AP_NOPRINT))) {
 	return 0;
     }
     if (flag & 0x1)
@@ -276,21 +276,27 @@ void clear_skill(object *who)
     }
 }
 
-/* do_skill() - Main skills use function-similar in scope to cast_spell(). 
+/**
+ * Main skills use function-similar in scope to cast_spell().
  * We handle all requests for skill use outside of some combat here. 
  * We require a separate routine outside of fire() so as to allow monsters 
- * to utilize skills.  Returns 1 on use of skill, otherwise 0.
+ * to utilize skills.
  * This is changed (2002-11-30) from the old method that returned
  * exp - no caller needed that info, but it also prevented the callers
  * from know if a skill was actually used, as many skills don't
  * give any exp for their direct use (eg, throwing).
  * It returns 0 if no skill was used.
+ * @param op The object actually using the skill
+ * @param part An object taking part of the skill usage, used by throwing
+ * @param skill The skill used by op
+ * @param dir The direction in which the skill is used
+ * @param string A parameter string, necessary to use some skills
+ * @return 1 if the use of the skill was successful, 0 otherwise
  */
 
 int do_skill (object *op, object *part, object *skill, int dir, const char *string) {
     int success=0, exp=0;
-    int did_alc = 0;
-    object *tmp, *next;
+    object *tmp;
 
     if (!skill) return 0;
 
@@ -392,20 +398,8 @@ int do_skill (object *op, object *part, object *skill, int dir, const char *stri
 	case SK_THAUMATURGY:
 	case SK_LITERACY:
 	case SK_WOODSMAN:
-	    /* first, we try to find a cauldron, and do the alchemy thing.
-	     * failing that, we go and identify stuff. 
-	     */
-	    for (tmp=get_map_ob(op->map, op->x, op->y); tmp != NULL;tmp=next) {
-		next=tmp->above;
-		if(QUERY_FLAG(tmp, FLAG_IS_CAULDRON)) {
-		    attempt_do_alchemy(op, tmp);
-		    if (QUERY_FLAG(tmp, FLAG_APPLIED))
-			esrv_send_inventory(op, tmp);
-		    did_alc=1;
-		}
-	    }
-	    if (did_alc == 0)
-		exp = success = skill_ident(op,skill);
+        if (use_alchemy(op) == 0)
+			exp = success = skill_ident(op,skill);
 	    break;
 
 	case SK_DET_MAGIC:
@@ -466,7 +460,7 @@ int do_skill (object *op, object *part, object *skill, int dir, const char *stri
      * gain problems.
      */
  
-    if(success && exp) change_exp(op,exp, skill->skill, 0);
+    if(success && exp) change_exp(op,exp, skill->skill, SK_SUBTRACT_SKILL_EXP);
             
     return success;
 }
@@ -784,9 +778,16 @@ static object *find_best_player_hth_skill(object *op)
     object *tmp, *best_skill=NULL;
     int	dragon = is_dragon_pl(op), last_skill=sizeof(unarmed_skills), i;
 
+    /* Dragons are a special case - gros 25th July 2006 */
+    if (dragon)
+    {
+        tmp = find_skill_by_number(op, SK_CLAWING);
+        if (tmp) /* I suppose it should always be true - but maybe there's
+                  * draconic toothache ? :) */
+            return tmp;
+    }
     for (tmp=op->inv; tmp; tmp=tmp->below) {
 	if (tmp->type == SKILL) {
-	    if (dragon && tmp->subtype == SK_CLAWING) return tmp;
 
 	    /* The order in the array is preferred order.  So basically,
 	     * we just cut down the number to search - eg, if we find a skill
@@ -854,7 +855,7 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
 	    }
 	    if (skill != op->chosen_skill) {
 		/* now try to ready the new skill */
-		if(!change_skill(op,skill,0)) {  /* oh oh, trouble! */
+		if(!change_skill(op,skill,1)) {  /* oh oh, trouble! */
 		    new_draw_info_format(NDI_UNIQUE, 0, tmp, "Couldn't change to skill %s", skill->name);
 		    return 0;
 		}
@@ -989,14 +990,15 @@ static int attack_hth(object *pl, int dir, const char *string, object *skill) {
     if(QUERY_FLAG(pl, FLAG_READY_WEAPON))
 	for(weapon=pl->inv;weapon;weapon=weapon->below) {
 	    if (weapon->type==WEAPON && QUERY_FLAG(weapon, FLAG_APPLIED)) {
-		CLEAR_FLAG(weapon,FLAG_APPLIED);
-		CLEAR_FLAG(pl,FLAG_READY_WEAPON);
-		fix_player(pl);
-		if(pl->type==PLAYER) {
+		if (apply_special(pl, weapon, AP_UNAPPLY | AP_NOPRINT)) {
+		    new_draw_info_format(NDI_UNIQUE, 0,pl,
+				  "You are unable to unwield %s in order to attack with %s.",
+				  query_name(weapon), skill->name);
+		    return 0;
+		} else {
 		    new_draw_info(NDI_UNIQUE, 0,pl,"You unwield your weapon in order to attack.");
-		    esrv_update_item(UPD_FLAGS, pl, weapon);
+		    break;
 		}
-		break;
 	    }
 	}
     return skill_attack(enemy,pl,dir,string, skill);
