@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_effect_c =
- *   "$Id: spell_effect.c,v 1.155 2006/05/13 05:18:44 qal21 Exp $";
+ *   "$Id: spell_effect.c 5704 2007-03-06 19:41:03Z ryo_saeba $";
  */
 
 
@@ -421,7 +421,8 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
  * create nonnmagic arrows, or even -1, etc...
  */
 
-int cast_create_missile(object *op, object *caster,object *spell, int dir, const char *stringarg)
+int cast_create_missile(object *op, object *caster,object *spell, int dir, 
+    const char *stringarg)
 {
     int missile_plus=0, bonus_plus=0;
     const char *missile_name;
@@ -431,59 +432,65 @@ int cast_create_missile(object *op, object *caster,object *spell, int dir, const
     missile_name = "arrow";
 
     for (tmp=op->inv; tmp != NULL; tmp=tmp->below)
-	if (tmp->type == BOW && QUERY_FLAG(tmp, FLAG_APPLIED)) {
-	    missile_name=tmp->race;
-	}
+        if (tmp->type == BOW && QUERY_FLAG(tmp, FLAG_APPLIED))
+            missile_name=tmp->race;
 
     missile_plus = spell->stats.dam  + SP_level_dam_adjust(caster, spell);
 
+    if (!strcmp(missile_name, "arrows"))
+        missile_name = "arrow";
+    else if (!strcmp(missile_name, "crossbow bolts"))
+        missile_name = "bolt";
+
     if (find_archetype(missile_name)==NULL) {
-	LOG(llevDebug, "Cast create_missile: could not find archetype %s\n", 
-	    missile_name);
-	return 0;
+        LOG(llevDebug, "Cast create_missile: could not find archetype %s\n",
+            missile_name);
+        return 0;
     }
     missile = create_archetype(missile_name);
 
     if (stringarg) {
-	/* If it starts with a letter, presume it is a description */
-	if (isalpha(*stringarg)) {
-	    artifact *al = find_artifactlist(missile->type)->items;
+        /* If it starts with a letter, presume it is a description */
+        if (isalpha(*stringarg)) {
+            artifact *al = find_artifactlist(missile->type)->items;
 
-	    for ( ; al != NULL; al=al->next) 
-		if (!strcasecmp(al->item->name, stringarg)) break;
+            for ( ; al != NULL; al=al->next)
+                if (!strcasecmp(al->item->name, stringarg)) break;
 
-	    if (!al) {
-		free_object(missile);
-		new_draw_info_format(NDI_UNIQUE, 0, op ,"No such object %ss of %s", missile_name,
-				     stringarg);
-		return 0;
-	    }
-	    if (al->item->slaying) {
-		free_object(missile);
-		new_draw_info_format(NDI_UNIQUE, 0, op ,"You are not allowed to create %ss of %s", 
-				     missile_name, stringarg);
-		return 0;
-	    }
-	    give_artifact_abilities(missile, al->item);
-	    /* These special arrows cost something extra.  Don't have them also be magical -
-	     * otherwise, in most cases, not enough will be created.  I don't want to get into
-	     * the parsing of having to do both plus and type.
-	     */
-	    bonus_plus = 1 + (al->item->value / 5);
-	    missile_plus=0;
-	} else
-	    if (atoi(stringarg) < missile_plus)
-		missile_plus = atoi(stringarg);
+            if (!al) {
+                free_object(missile);
+                new_draw_info_format(NDI_UNIQUE, 0, op ,"No such object %ss of %s",
+                    missile_name, stringarg);
+                return 0;
+            }
+            if (al->item->slaying) {
+                free_object(missile);
+                new_draw_info_format(NDI_UNIQUE, 0, op ,
+                    "You are not allowed to create %ss of %s", missile_name,
+                    stringarg);
+                return 0;
+            }
+            give_artifact_abilities(missile, al->item);
+            /* These special arrows cost something extra.  Don't have them also be
+            * magical - otherwise, in most cases, not enough will be created.
+            * I don't want to get into the parsing of having to do both plus and
+            * type.
+            */
+            bonus_plus = 1 + (al->item->value / 5);
+            missile_plus=0;
+        }
+        else if (atoi(stringarg) < missile_plus)
+            missile_plus = atoi(stringarg);
     }
     if (missile_plus > 4)
-	missile_plus = 4;
+        missile_plus = 4;
     else if (missile_plus < -4)
-	missile_plus = -4;
+        missile_plus = -4;
 
     missile->nrof = spell->duration + SP_level_duration_adjust(caster, spell);
     missile->nrof -= 3 * (missile_plus + bonus_plus);
     if (missile->nrof < 1)
-	missile->nrof=1;
+        missile->nrof=1;
 
     missile->magic = missile_plus;
     /* Can't get any money for these objects */
@@ -493,9 +500,9 @@ int cast_create_missile(object *op, object *caster,object *spell, int dir, const
     tag = missile->count;
 
     if ( ! cast_create_obj (op, caster, missile, dir) && op->type == PLAYER
-      && ! was_destroyed (missile, tag)) 
+      && ! was_destroyed (missile, tag))
     {
-	pick_up(op, missile);
+        pick_up(op, missile);
     }
     return 1;
 }
@@ -855,6 +862,9 @@ int perceive_self(object *op) {
 	    }
 	}
     }
+    if (op->glow_radius > 0)
+        draw_ext_info(NDI_UNIQUE, 0,op,MSG_TYPE_SPELL, MSG_TYPE_SPELL_PERCEIVE_SELF,
+            "You glow in the dark.", NULL);
 
     if (is_dragon_pl(op)) {
        /* now grab the 'dragon_ability'-force from the player's inventory */
@@ -901,16 +911,19 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 
     /* Check to see if the map the player is currently on is a per player unique
      * map.  This can be determined in that per player unique maps have the
-     * full pathname listed.
+     * full pathname listed. Ignore if settings.create_home_portals is true.
      */
-    if (!strncmp(op->map->path, settings.localdir, strlen(settings.localdir))) {
-	new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"You can't cast that here.\n");
-	return 0;
+    if (!settings.create_home_portals) {
+        if (!strncmp(op->map->path, settings.localdir, strlen(settings.localdir))) {
+            new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"You can't cast that here.\n");
+            return 0;
+        }
     }
 
     /* Check to see if the player is on a transport */
     if (op->contr && op->contr->transport) {
-	new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"You need to exit the transport to cast that.\n");
+        new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,
+            "You need to exit the transport to cast that.\n");
         return 0;
     }
 
@@ -919,25 +932,29 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      */
     dummy=arch_to_object(spell->other_arch);
     if(dummy == NULL){
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (force in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (force in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     force=check_inv_recursive (op,dummy);
 
     if (force==NULL) {
-	/* Here we know there is no destination marked up.
-	 * We have 2 things to do:
-	 * 1. Mark the destination in the player inventory.
-	 * 2. Let the player know it worked.
-	 */
-	free_string (dummy->name);
-	dummy->name = add_string (op->map->path);
-	EXIT_X(dummy)= op->x;
-	EXIT_Y(dummy)= op->y;
-	insert_ob_in_ob (dummy,op);
-	new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"You fix this place in your mind.\nYou feel you are able to come here from anywhere.");
-	return 1;
+        /* Here we know there is no destination marked up.
+         * We have 2 things to do:
+         * 1. Mark the destination in the player inventory.
+         * 2. Let the player know it worked.
+         */
+        free_string (dummy->name);
+        dummy->name = add_string (op->map->path);
+        EXIT_X(dummy)= op->x;
+        EXIT_Y(dummy)= op->y;
+        dummy->weapontype = op->map->last_reset_time.tv_sec;
+        insert_ob_in_ob (dummy,op);
+        new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,
+            "You fix this place in your mind.\nYou feel you are able to come here from anywhere.");
+        return 1;
     }
     free_object (dummy);
 
@@ -960,9 +977,11 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
     /* First step: killing existing town portals */
     dummy=create_archetype(spell->race);
     if(dummy == NULL){
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (force) in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     perm_portal = find_archetype (spell->slaying);
 
@@ -974,29 +993,30 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      *   -We destruct the force indicating that portal.
      */
     while ( (old_force=check_inv_recursive (op,dummy))) {
-	exitx=EXIT_X(old_force);
-	exity=EXIT_Y(old_force);
-	LOG (llevDebug,"Trying to kill a portal in %s (%d,%d)\n",old_force->race,exitx,exity);
+        exitx=EXIT_X(old_force);
+        exity=EXIT_Y(old_force);
+        LOG (llevDebug,"Trying to kill a portal in %s (%d,%d)\n",
+            old_force->race,exitx,exity);
 
-	if (!strncmp(old_force->race, settings.localdir, strlen(settings.localdir))) 
-	    exitmap = ready_map_name(old_force->race, MAP_PLAYER_UNIQUE);
-	else exitmap = ready_map_name(old_force->race, 0);
+        if (!strncmp(old_force->race, settings.localdir, strlen(settings.localdir)))
+            exitmap = ready_map_name(old_force->race, MAP_PLAYER_UNIQUE);
+        else exitmap = ready_map_name(old_force->race, 0);
 
-	if (exitmap) {
-	    tmp=present_arch (perm_portal,exitmap,exitx,exity);
-	    while (tmp) {
-		if (tmp->name == old_force->name) {
-		    remove_ob (tmp);
-		    free_object (tmp);
-		    break;
-		} else {
-		    tmp = tmp->above;
-		}
-	    }
-	}
-	remove_ob (old_force);
-	free_object (old_force);
-	LOG (llevDebug,"\n");
+        if (exitmap) {
+            tmp=present_arch (perm_portal,exitmap,exitx,exity);
+            while (tmp) {
+                if (tmp->name == old_force->name) {
+                    remove_ob (tmp);
+                    free_object (tmp);
+                    break;
+                } else {
+                    tmp = tmp->above;
+                }
+            }
+        }
+        remove_ob (old_force);
+        free_object (old_force);
+        LOG (llevDebug,"\n");
     }
     free_object (dummy);
 
@@ -1012,16 +1032,24 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 
     /* Ensure exit map is loaded*/
     if (!strncmp(force->name, settings.localdir, strlen(settings.localdir)))
-	exitmap = ready_map_name(force->name, MAP_PLAYER_UNIQUE);
+        exitmap = ready_map_name(force->name, MAP_PLAYER_UNIQUE);
     else
-	exitmap = ready_map_name(force->name, 0);
+        exitmap = ready_map_name(force->name, 0);
 
     /* If we were unable to load (ex. random map deleted), warn player*/
     if (exitmap==NULL) {
-	new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"Something strange happens.\nYou can't remember where to go!?");
-	remove_ob(force);
-	free_object(force);
-	return 1;
+        new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,
+            "Something strange happens.\nYou can't remember where to go!?");
+        remove_ob(force);
+        free_object(force);
+        return 1;
+    }
+    else if (exitmap->last_reset_time.tv_sec != force->weapontype)
+    {
+        new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"The spell effect has expired.");
+        remove_ob(force);
+        free_object(force);
+        return 1;
     }
 
     op_level = caster_level(caster, spell);
@@ -1041,9 +1069,11 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
     snprintf (portal_name,1024,"%s's portal to %s",op->name,force->name);
     dummy=create_archetype(spell->slaying); /*The portal*/
     if(dummy == NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     EXIT_PATH(dummy) = add_string (force->name);
     EXIT_X(dummy)=EXIT_X(force);
@@ -1060,9 +1090,11 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      */
     tmp=create_archetype(spell->race);
     if(tmp == NULL){
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (force) in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     tmp->race=add_string (op->map->path);
     FREE_AND_COPY(tmp->name, portal_name);
@@ -1079,9 +1111,11 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
     snprintf (portal_name,1024,"%s's portal to %s",op->name,op->map->path);
     dummy=create_archetype (spell->slaying); /*The portal*/
     if(dummy == NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     EXIT_PATH(dummy) = add_string (op->map->path);
     EXIT_X(dummy)=op->x;
@@ -1099,9 +1133,11 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      */
     tmp=create_archetype(spell->race);
     if(tmp == NULL){
-	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
-	return 0;
+        new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
+        LOG(llevError,
+            "get_object failed (force) in cast_create_town_portal for %s!\n",
+            op->name);
+        return 0;
     }
     tmp->race=add_string(force->name);
     FREE_AND_COPY(tmp->name, portal_name);
@@ -1497,7 +1533,7 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int
     object *tmp, *tmp2=NULL;
     object *force=NULL;
     int i;
-  
+
     /* if dir = 99 op defaults to tmp, eat_special_food() requires this. */
     if(dir!=0) {
 	tmp=find_target_for_friendly_spell(op,dir);
@@ -1547,6 +1583,8 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int
 	return 1;
     }
     force->duration = spell_ob->duration + SP_level_duration_adjust(caster, spell_ob) * 50;
+    if (op->type == PLAYER)
+        store_spell_expiry(force);
     force->speed = 1.0;
     force->speed_left = -1.0;
     SET_FLAG(force, FLAG_APPLIED);
@@ -1703,9 +1741,9 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
  *
  * This code adds a new spell, called alchemy.  Alchemy will turn
  * objects to gold nuggets, the value of the gold nuggets being
- * about 90% of that of the item itself.  It uses the value of the
- * object before charisma adjustments, because the nuggets themselves
- * will be will be adjusted by charisma when sold.
+ * from 5% to 40% of that of the item itself depending on casting level.
+ * It uses the value of the object before charisma adjustments, because
+ * the nuggets themselves will be will be adjusted by charisma when sold.
  *
  * Large nuggets are worth 25 gp each (base).  You will always get
  * the maximum number of large nuggets you could get.
@@ -1728,25 +1766,26 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
 static object *small, *large;
 static uint64 small_value, large_value;
 
-static void alchemy_object(object *obj, int *small_nuggets,
+static void alchemy_object(float value_adj, object *obj, int *small_nuggets,
 	 int *large_nuggets, int *weight)
 {
     uint64 value=query_cost(obj, NULL, F_TRUE);
 
-    /* Give third price when we alchemy money (This should hopefully
-     * make it so that it isn't worth it to alchemy money, sell
-     * the nuggets, alchemy the gold from that, etc.
-     * Otherwise, give 9 silver on the gold for other objects,
-     * so that it would still be more affordable to haul
-     * the stuff back to town.
+    /* Multiply the value of the object by value_adj, which should range
+     * from 0.05 to 0.40. Set value to 0 instead if unpaid.
      */
-
     if (QUERY_FLAG(obj, FLAG_UNPAID))
 	value=0;
-    else if (obj->type==MONEY || obj->type==GEM)
-	value /=3;
     else
-	value = (value*9)/10;
+        value *= value_adj;
+
+    
+    /* Give half of what value_adj says when we alchemy money (This should
+     * hopefully make it so that it isn't worth it to alchemy money, sell
+     * the nuggets, alchemy the gold from that, etc.
+     */
+    if (value && (obj->type==MONEY || obj->type==GEM))
+	value /=2;
 
     if ((obj->value>0) && rndm(0, 29)) {
 	int count;
@@ -1773,7 +1812,7 @@ static void alchemy_object(object *obj, int *small_nuggets,
     free_object(obj);
 }
 
-static void update_map(object *op, mapstruct *m, int small_nuggets, int large_nuggets,
+static void place_alchemy_objects(object *op, mapstruct *m, int small_nuggets, int large_nuggets,
 	int x, int y)
 {
     object *tmp;
@@ -1806,6 +1845,7 @@ int alchemy(object *op, object *caster, object *spell_ob)
 {
     int x,y,weight=0,weight_max,large_nuggets,small_nuggets, mflags;
     sint16 nx, ny;
+    float value_adj;
     object *next,*tmp;
     mapstruct *mp;
 
@@ -1822,6 +1862,15 @@ int alchemy(object *op, object *caster, object *spell_ob)
     large=create_archetype("largenugget");
     small_value = query_cost(small, NULL, F_TRUE);
     large_value = query_cost(large, NULL, F_TRUE);
+    
+    /* Set value_adj to be a multiplier for how much of the original value
+     * will be in the nuggets. Starts at 0.05, increasing by 0.01 per casting
+     * level, maxing out at 0.40.
+     */
+    value_adj = (SP_level_dam_adjust(caster, spell_ob) / 
+        100.00) + 0.05;
+        
+    if (value_adj > 0.40) value_adj = 0.40;
 
     for(y= op->y-1;y<=op->y+1;y++) {
 	for(x= op->x-1;x<=op->x+1;x++) {
@@ -1858,14 +1907,14 @@ int alchemy(object *op, object *caster, object *spell_ob)
 			    if (tmp1->weight>0 && !QUERY_FLAG(tmp1, FLAG_NO_PICK) &&
 				!QUERY_FLAG(tmp1, FLAG_ALIVE) &&
 				!QUERY_FLAG(tmp1, FLAG_IS_CAULDRON))
-				alchemy_object(tmp1, &small_nuggets, &large_nuggets,
+				alchemy_object(value_adj, tmp1, &small_nuggets, &large_nuggets,
 					   &weight);
 			}
 		    }
-		    alchemy_object(tmp, &small_nuggets, &large_nuggets, &weight);
+		    alchemy_object(value_adj, tmp, &small_nuggets, &large_nuggets, &weight);
 	    
 		    if (weight>weight_max) {
-			update_map(op, mp, small_nuggets, large_nuggets, nx, ny);
+			place_alchemy_objects(op, mp, small_nuggets, large_nuggets, nx, ny);
 			free_object(large);
 			free_object(small);
 			return 1;
@@ -1877,7 +1926,7 @@ int alchemy(object *op, object *caster, object *spell_ob)
 	     * it also prevents us from alcheming nuggets that were just created
 	     * with this spell.
 	     */
-	    update_map(op, mp, small_nuggets, large_nuggets, nx, ny);
+	    place_alchemy_objects(op, mp, small_nuggets, large_nuggets, nx, ny);
 	}
     }
     free_object(large);
@@ -2041,7 +2090,7 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 		     tmp->type==TRIGGER_PEDESTAL || tmp->type==SPECIAL_KEY ||
 		     tmp->type==TREASURE || tmp->type==BOOK ||
 		     tmp->type==HOLY_ALTAR))) {
-			if(random_roll(0, skill->level-1, op, PREFER_HIGH) > level/4) {
+			if(random_roll(0, level-1, op, PREFER_HIGH) > tmp->level) {
 			    tmp->invisible=0;
 			    done_one = 1;
 			}
@@ -2075,7 +2124,7 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 		}
 		/* detect monster */
 		if (QUERY_FLAG(spell, FLAG_MONSTER) &&
-		    (QUERY_FLAG(tmp, FLAG_MONSTER) || tmp->type==PLAYER)) {
+		    (QUERY_FLAG(tmp, FLAG_MONSTER) || (tmp->type==PLAYER && !QUERY_FLAG(tmp, FLAG_WIZ)))) {
 			done_one = 2;
 			if (!detect) detect=tmp;
 		}
@@ -2297,7 +2346,7 @@ void counterspell(object *op,int dir)
 		free_object(head);
 	} else switch(head->type) {
 	    case SPELL_EFFECT: 
-		if(op->level > head->level) {
+		if((op->level > head->level) && !op->stats.food && !op->speed_left) {
 		    remove_ob(head);
 		    free_object(head);
 		}
@@ -2328,32 +2377,47 @@ int cast_consecrate(object *op, object *caster, object *spell) {
     object *tmp, *god=find_god(determine_god(op));
 
     if(!god) {
-	new_draw_info(NDI_UNIQUE, 0,op,
-		      "You can't consecrate anything if you don't worship a god!");
+        new_draw_info(NDI_UNIQUE, 0,op,"You can't consecrate anything if you don't worship a god!");
         return 0;
     }
- 
+
     for(tmp=op->below;tmp;tmp=tmp->below) {
-	if(QUERY_FLAG(tmp,FLAG_IS_FLOOR)) break;
+        if(QUERY_FLAG(tmp,FLAG_IS_FLOOR)) break;
         if(tmp->type==HOLY_ALTAR) {
 
-	    if(tmp->level > caster_level(caster, spell)) {
-		new_draw_info_format(NDI_UNIQUE, 0,op,
-		    "You are not powerful enough to reconsecrate the %s", tmp->name);
-		return 0;
-	    } else {
-		/* If we got here, we are consecrating an altar */
-		if(tmp->name)	free_string(tmp->name);
-		sprintf(buf,"Altar of %s",god->name);
-		tmp->name = add_string(buf);
-		tmp->level = caster_level(caster, spell);
-		tmp->other_arch = god->arch;
-		if(op->type==PLAYER) esrv_update_item(UPD_NAME, op, tmp);
-		new_draw_info_format(NDI_UNIQUE,0, op,
-				     "You consecrated the altar to %s!",god->name);
-		return 1;
-	    }
-	}
+            if(tmp->level > caster_level(caster, spell)) {
+                new_draw_info_format(NDI_UNIQUE, 0,op,"You are not powerful enough to reconsecrate the %s", tmp->name);
+                return 0;
+            } else {
+                /* If we got here, we are consecrating an altar */
+                object* new_altar;
+                int letter;
+                archetype* altar_arch;
+                snprintf(buf,MAX_BUF,"altar_");
+                letter=strlen(buf);
+                strncpy(buf+letter,god->name,MAX_BUF-letter);
+                for (; letter < strlen(buf); letter++)
+                    buf[letter] = tolower(buf[letter]);
+                altar_arch = find_archetype(buf);
+                if (!altar_arch) {
+                    new_draw_info_format(NDI_UNIQUE, 0, op, "You fail to consecrate the altar.");
+                    LOG(llevError, "cast_consecrate: can't find altar %s for god %s", buf, god->name);
+                    return 0;
+                }
+                new_altar = arch_to_object(altar_arch);
+                new_altar->x = tmp->x;
+                new_altar->y = tmp->y;
+                new_altar->level = tmp->level;
+                insert_ob_in_map(new_altar,tmp->map,tmp,INS_BELOW_ORIGINATOR);
+                if(op->type==PLAYER) {
+                    esrv_del_item(op->contr, tmp->count);
+                    esrv_send_item(op, new_altar);
+                }
+                remove_ob(tmp);
+                new_draw_info_format(NDI_UNIQUE,0, op,"You consecrated the altar to %s!",god->name);
+                return 1;
+            }
+        }
     }
     new_draw_info(NDI_UNIQUE, 0,op,"You are not standing over an altar!");
     return 0;
@@ -2595,6 +2659,8 @@ int create_aura(object *op, object *caster, object *spell)
 
     new_aura->duration  = spell->duration + 
                   10* SP_level_duration_adjust(caster,spell);
+    if (op->type == PLAYER)
+        store_spell_expiry(new_aura);
 
     new_aura->stats.dam = spell->stats.dam
                   +SP_level_dam_adjust(caster,spell);
@@ -2680,6 +2746,7 @@ void move_aura(object *aura) {
     /* put the aura back in the player's inventory */
     remove_ob(aura);
     insert_ob_in_ob(aura, env);
+    check_spell_expiry(aura);
 }
 
 /**
