@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_util_c =
- *   "$Id: spell_util.c,v 1.106 2006/06/05 06:28:56 mwedel Exp $";
+ *   "$Id: spell_util.c 5704 2007-03-06 19:41:03Z ryo_saeba $";
  */
 
 
@@ -167,7 +167,7 @@ void spell_effect (object *spob, int x, int y, mapstruct *map,
  * spell is the spell object.
  * Returns modified level.
  */
-static int min_casting_level(object *caster, object *spell)
+int min_casting_level(object *caster, object *spell)
 {
     int new_level;
 
@@ -520,7 +520,9 @@ int fire_arch_from_position (object *op, object *caster, sint16 x, sint16 y,
 	return 0;
 
     if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(m, x, y))) {
-	new_draw_info(NDI_UNIQUE, 0, op, "You can't cast the spell on top of a wall!\n");
+        if ( caster->type == PLAYER )
+            /* If caster is not player, it's for instance a swarm, so don't say there's an issue. */
+            new_draw_info(NDI_UNIQUE, 0, op, "You can't cast the spell on top of a wall!\n");
 	free_object(tmp);
 	return 0;
     }
@@ -600,33 +602,50 @@ object *find_target_for_friendly_spell(object *op,int dir) {
      * we then make the owner of this object the target.
      * The owner could very well be no where near op.
      */
-    if(op->type!=PLAYER && op->type!=RUNE) {
-	tmp=get_owner(op);
-	/* If the owner does not exist, or is not a monster, than apply the spell
-	 * to the caster.
-	 */
-	if(!tmp || !QUERY_FLAG(tmp,FLAG_MONSTER)) tmp=op;
+    if(op->type!=PLAYER && op->type!=RUNE)
+    {
+        tmp=get_owner(op);
+        /* If the owner does not exist, or is not a monster, than apply the spell
+         * to the caster.
+         */
+        if(!tmp || !QUERY_FLAG(tmp,FLAG_MONSTER)) tmp=op;
     }
-    else {
-	m = op->map;
-	x =  op->x+freearr_x[dir];
-	y =  op->y+freearr_y[dir];
+    else
+    {
+        m = op->map;
+        x =  op->x+freearr_x[dir];
+        y =  op->y+freearr_y[dir];
 
-	mflags = get_map_flags(m, &m, x, y, &x, &y);
+        mflags = get_map_flags(m, &m, x, y, &x, &y);
 
-	if (mflags & P_OUT_OF_MAP)
-	    tmp=NULL;
-	else  {
-	    for(tmp=get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above)
-		if(tmp->type==PLAYER)
-		    break;
-	}
+        if (mflags & P_OUT_OF_MAP)
+            tmp=NULL;
+        else
+        {
+            for(tmp=get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above)
+            {
+                if(tmp->type==PLAYER)
+                    break;
+            }
+        }
     }
     /* didn't find a player there, look in current square for a player */
     if(tmp==NULL)
-	for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
-	    if(tmp->type==PLAYER)
-		break;
+        for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
+        {
+            if(tmp->type==PLAYER)
+                break;
+            /* Don't forget to browse inside transports ! - gros 2006/07/25 */
+            if(tmp->type==TRANSPORT)
+            {
+                object* inv;
+                for (inv=tmp->inv; inv; inv=inv->below)
+                {
+                    if ((inv->type == PLAYER)&&(op == inv))
+                        return inv;
+                }
+            }
+        }
     return tmp;
 }
 
@@ -639,6 +658,7 @@ object *find_target_for_friendly_spell(object *op,int dir) {
  * It will not consider the object given as exclude (= caster) among possible
  * live objects. If the caster is a player, the spell will go after
  * monsters/generators only. If not, the spell will hunt players only.
+ * Exception is player on a battleground, who will be targeted unless excluded.
  * It returns the direction toward the first/closest live object if it finds
  * any, otherwise -1.
  * note that exclude can be NULL, in which case all bets are off.
@@ -665,10 +685,10 @@ int spell_find_dir(mapstruct *m, int x, int y, object *exclude) {
 
 	tmp=get_map_ob(mp,nx,ny);
 
-	while(tmp!=NULL && (((owner_type==PLAYER &&
-	    !QUERY_FLAG(tmp,FLAG_MONSTER) && !QUERY_FLAG(tmp,FLAG_GENERATOR)) ||
-            (owner_type!=PLAYER && tmp->type!=PLAYER)) ||
-	    (tmp == exclude || (tmp->head && tmp->head == exclude))))
+	while(tmp!=NULL && (
+             ((owner_type==PLAYER && !QUERY_FLAG(tmp,FLAG_MONSTER) && !QUERY_FLAG(tmp,FLAG_GENERATOR) && !(tmp->type == PLAYER && op_on_battleground(tmp, NULL, NULL)))
+              || (owner_type!=PLAYER && tmp->type!=PLAYER))
+             || (tmp == exclude || (tmp->head && tmp->head == exclude))))
 	    tmp=tmp->above;
 
 	if(tmp!=NULL && can_see_monsterP(m,x,y,i))
@@ -995,7 +1015,7 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
     /* if caster is a spell casting object, this normally shouldn't be 
      * an issue, because they don't have any spellpaths set up.
      */
-    if (caster->path_denied & spell_ob->path_attuned) {
+    if ((caster->path_denied & spell_ob->path_attuned) && !QUERY_FLAG(caster,FLAG_WIZ)) {
 	new_draw_info(NDI_UNIQUE, 0,op, "That spell path is denied to you.");
 	return 0;
     }
@@ -1083,7 +1103,7 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
      * doing.
      */
     if (spell_ob->type == SPELL && caster->type != POTION && !QUERY_FLAG(op, FLAG_WIZCAST) &&
-	(QUERY_FLAG(caster, FLAG_ALIVE) || QUERY_FLAG(op, FLAG_ALIVE)) &&
+        (QUERY_FLAG(caster, FLAG_ALIVE) || QUERY_FLAG(op, FLAG_ALIVE)) && !QUERY_FLAG(op,FLAG_MONSTER) &&
 	(((mflags & P_NO_MAGIC) && spell_ob->stats.sp) ||
 	   ((mflags & P_NO_CLERIC) && spell_ob->stats.grace))) {
 
@@ -1175,6 +1195,16 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 	change_skill(op, skill, 0);    /* needed for proper exp credit */
     }
 
+    /* Need to get proper ownership for spells cast via runes - these are not
+     * the normal 'rune of fire', but rather the magic runes that let the player
+     * put some other spell into the rune (glyph, firetrap, magic rune, etc)
+     */
+    if (caster->type == RUNE) {
+	object *owner = get_owner(caster);
+
+	if (owner) skill = find_skill_by_name(owner, caster->skill);
+    }
+
     switch(spell_ob->subtype) {
 	/* The order of case statements is same as the order they show up
 	 * in in spells.h.
@@ -1525,3 +1555,54 @@ void apply_spell_effect(object *spell, object *victim)
 	    break;
     }
 }
+
+/**
+ * Stores in the spell when to warn player of expiration.
+ *
+ * @param spell
+ * spell we're considering.
+ */
+void store_spell_expiry(object* spell) {
+    /* Keep when to warn the player of expiration */
+    char dur[10];
+    int i = spell->duration / 5;
+    if (!i)
+        i = 1;
+    snprintf(dur, sizeof(dur), "%d", i);
+    set_ob_key_value(spell, "spell_expiry_warn_1", dur, 1);
+    i = i / 5;
+    if (i > 0) {
+        snprintf(dur, sizeof(dur), "%d", i);
+        set_ob_key_value(spell, "spell_expiry_warn_2", dur, 1);
+    }
+}
+
+/**
+ * Checks if player should be warned of soon expiring spell.
+ *
+ * Should be called at each move of the spell. Will use key stored by store_spell_expiry().
+ * If the spell effect/force isn't in a player's inventory, won't do anything.
+ *
+ * @param spell
+ * force or spell whose effects will expire.
+ */
+void check_spell_expiry(object* spell) {
+    const char* key;
+
+    if (!spell->env || !spell->env->type == PLAYER)
+        return;
+
+    if ((key = get_ob_key_value(spell, "spell_expiry_warn_1")) != NULL) {
+        if (spell->duration == atoi(key)) {
+            new_draw_info_format(NDI_UNIQUE | NDI_NAVY, 0, spell->env, "The effects of your %s are draining out.", spell->name);
+            return;
+        }
+    }
+    if ((key = get_ob_key_value(spell, "spell_expiry_warn_2")) != NULL) {
+        if (spell->duration == atoi(key)) {
+            new_draw_info_format(NDI_UNIQUE | NDI_NAVY, 0, spell->env, "The effects of your %s are about to expire.", spell->name);
+            return;
+        }
+    }
+}
+
