@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_swap_c =
- *    "$Id: swap.c,v 1.18 2006/05/05 09:26:36 ryo_saeba Exp $";
+ *    "$Id: swap.c 4961 2006-09-21 05:14:18Z mwedel $";
  */
 
 /*
@@ -106,6 +106,7 @@ void read_map_log(void)
 
 	map->in_memory=MAP_SWAPPED;
 	map->darkness=darkness;
+	map->timeout=0;
 
 	/* When the reset time is saved out, it is adjusted so that
 	 * the current time is subtracted (thus, it is saved as number
@@ -152,7 +153,7 @@ void swap_map(mapstruct *map) {
 
 	LOG(llevDebug,"Resetting map %s.\n",map->path);
         /* Lauwenmark : Here we handle the MAPRESET global event */
-        execute_global_event(EVENT_MAPRESET, map->path);
+        execute_global_event(EVENT_MAPRESET, map);
 	map = map->next;
 	delete_map(oldmap);
 	return;
