@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.89 2006/05/05 09:26:36 ryo_saeba Exp $";
+ *    "$Id: time.c 5704 2007-03-06 19:41:03Z ryo_saeba $";
  */
 
 /*
@@ -91,6 +91,7 @@ void remove_door2(object *op) {
  */
 static void generate_monster_inv(object *gen) {
     int i;
+    int nx, ny;
     object *op,*head=NULL;
 
     int qty=0;
@@ -112,7 +113,7 @@ static void generate_monster_inv(object *gen) {
     qty=rndm(0,qty-1);
     for (op=gen->inv;qty;qty--)
         op=op->below;
-    i=find_free_spot(op,gen->map,gen->x,gen->y,1,9);
+    i=find_multi_free_spot_around(op, gen, &nx, &ny);
     if (i==-1)
         return;
     head=object_create_clone(op);
@@ -120,8 +121,9 @@ static void generate_monster_inv(object *gen) {
     unflag_inv (head,FLAG_IS_A_TEMPLATE);
     if (rndm(0, 9))
         generate_artifact(head, gen->map->difficulty);
-    insert_ob_in_map_at(head,gen->map,gen,0,gen->x+freearr_x[i],gen->y+freearr_y[i]);
+    insert_ob_in_map_at(head,gen->map,gen,0,nx,ny);
     if (QUERY_FLAG(head, FLAG_FREED)) return;
+    fix_multipart_object(head);
     if(HAS_RANDOM_ITEMS(head))
         create_treasure(head->randomitems,head,GT_APPLY,
                       gen->map->difficulty,0);
@@ -129,6 +131,7 @@ static void generate_monster_inv(object *gen) {
 
 static void generate_monster_arch(object *gen) {
     int i;
+    int nx, ny;
     object *op,*head=NULL,*prev=NULL;
     archetype *at=gen->other_arch;
 
@@ -144,12 +147,12 @@ static void generate_monster_arch(object *gen) {
 	LOG(llevError,"Generator (%s) not on a map?\n", gen->name);
 	return;
     }
-    i=find_free_spot(&at->clone,gen->map,gen->x,gen->y,1,9);
+    i=find_multi_free_spot_around(&at->clone, gen, &nx, &ny);
     if (i==-1) return;
     while(at!=NULL) {
 	op=arch_to_object(at);
-	op->x=gen->x+freearr_x[i]+at->clone.x;
-	op->y=gen->y+freearr_y[i]+at->clone.y;
+	op->x=nx+at->clone.x;
+	op->y=ny+at->clone.y;
 
 	if(head!=NULL)
 	    op->head=head,prev->more=op;
@@ -179,7 +182,10 @@ static void generate_monster(object *gen) {
 }
 
 static void remove_force(object *op) {
-    if (--op->duration > 0) return;
+    if (--op->duration > 0) {
+        check_spell_expiry(op);
+        return;
+    }
 
     switch (op->subtype) {
 	case FORCE_CONFUSION:
@@ -829,7 +835,7 @@ static void change_object(object *op) { /* Doesn`t handle linked objs yet */
 	 * client of the change.  Insert_ob_in_map takes care of the
 	 * updating the client, so we don't need to do that below.
 	 */
-	if ((pl=is_player_inv(env))!=NULL) {
+	if ((pl=get_player_container(env))!=NULL) {
 	    esrv_del_item(pl->contr, op->count);
 	    esrv_send_item(pl, tmp);
 	}
@@ -1058,6 +1064,11 @@ void move_player_mover(object *op) {
 void move_duplicator(object *op) {
     object *tmp;
 
+    if ( !op->other_arch ) {
+        LOG(llevInfo, "Duplicator with no other_arch! %d %d %s\n", op->x, op->y, op->map ? op->map->path : "nullmap");
+        return;
+    }
+
     if (op->above == NULL)
 	return;
     for (tmp=op->above; tmp != NULL; tmp=tmp->above) {
@@ -1129,6 +1140,9 @@ void move_creator(object *creator) {
 	return;
     }
 
+    if (creator->level != 0)
+        new_ob->level = creator->level;
+
     insert_ob_in_map_at(new_ob, creator->map, creator, 0, creator->x, creator->y);
     if (QUERY_FLAG(new_ob, FLAG_FREED))
 	return;
@@ -1237,7 +1251,7 @@ int process_object(object *op) {
 	    remove_force(op);
 	else {
 	    /* IF necessary, delete the item from the players inventory */
-	    object *pl=is_player_inv(op);
+	    object *pl=get_player_container(op);
 	    if (pl)
 		esrv_del_item(pl->contr, op->count);
 	    remove_ob(op);
