@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c,v 1.46 2006/06/05 06:28:57 mwedel Exp $";
+ *    "$Id: loop.c 5074 2006-11-04 07:37:28Z mwedel $";
  */
 
 /*
@@ -166,7 +166,8 @@ void request_info_cmd(char *buf, int len, socket_struct *ns)
     else if (!strcmp(buf,"image_sums")) send_image_sums(ns, params);
     else if (!strcmp(buf,"skill_info")) send_skill_info(ns, params);
     else if (!strcmp(buf,"spell_paths")) send_spell_paths(ns, params);
-    else Write_String_To_Socket(ns, bigbuf, len);
+    else if (!strcmp(buf,"exp_table")) send_exp_table(ns, params);
+    else Write_String_To_Socket(ns, bigbuf, slen);
 }
 
 /**
