@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: request.c,v 1.94 2006/05/30 07:16:23 mwedel Exp $";
+ *    "$Id: request.c 5149 2006-12-02 19:17:49Z ryo_saeba $";
  */
 
 /*
@@ -327,13 +327,7 @@ void toggle_extended_infos_cmd(char *buf, int len, socket_struct *ns)
      }
      Write_String_To_Socket(ns, cmdback,strlen(cmdback));
 }
-/*
-#define MSG_TYPE_BOOK            1
-#define MSG_TYPE_CARD            2
-#define MSG_TYPE_PAPER           3
-#define MSG_TYPE_SIGN            4
-#define MSG_TYPE_MONUMENT        5
-#define MSG_TYPE_SCRIPTED_DIALOG 6*/
+
 /** Reply to ExtendedInfos command */
 void toggle_extended_text_cmd(char *buf, int len, socket_struct *ns)
 {
@@ -589,6 +583,9 @@ void reply_cmd(char *buf, int len, player *pl)
 
 	case ST_GET_PASSWORD:
 	case ST_CONFIRM_PASSWORD:
+	case ST_CHANGE_PASSWORD_OLD:
+	case ST_CHANGE_PASSWORD_NEW:
+	case ST_CHANGE_PASSWORD_CONFIRM:
 	    receive_player_password(pl->ob,13);
 	    break;
 
@@ -2043,6 +2040,13 @@ void draw_client_map2(object *pl)
 
 		    for (layer=0; layer < MAP_LAYERS; layer++) {
 			ob = GET_MAP_FACE_OBJ(m, nx, ny, layer);
+            
+            /* Special case: send player itself if invisible */
+            if ( !ob && x == pl->x && y == pl->y &&
+                  ( pl->invisible & (pl->invisible < 50 ? 4 : 1) ) &&
+                  ( layer == MAP_LAYER_LIVING1 || layer == MAP_LAYER_LIVING2 ) )
+                ob = pl;
+
 			if (ob)  {
 			    g1 = has_obj;
 			    got_one += map2_add_ob(ax, ay, layer, ob, &sl, &pl->contr->socket, &has_obj, 0);
@@ -2229,6 +2233,30 @@ void send_plugin_custom_message(object *pl, char *buf)
     cs_write_string(&pl->contr->socket,buf,strlen(buf));
 }
 
+/**
+ * This sends the experience table the sever is using
+ */
+void send_exp_table(socket_struct *ns, char *params)
+{
+    SockList sl;
+    int i;
+    extern sint64 *levels;
+
+    sl.buf = malloc(MAXSOCKSENDBUF);
+    strcpy((char*)sl.buf,"replyinfo exp_table\n");
+    sl.len = strlen((char*)sl.buf);
+    SockList_AddShort(&sl, settings.max_level+1);
+    for (i=1; i<= settings.max_level; i++) {
+	if (sl.len+8 > MAXSOCKSENDBUF) {
+	    LOG(llevError, "Buffer overflow in send_exp_table, not sending all information\n");
+	    break;
+	}
+	SockList_AddInt64(&sl, levels[i]);
+    }
+    Send_With_Handling(ns, &sl);
+    free(sl.buf);
+}
+
 /**
  * This sends the skill number to name mapping.  We ignore
  * the params - we always send the same info no matter what.
@@ -2292,21 +2320,23 @@ void esrv_update_spells(player *pl) {
     SockList sl;
     int flags=0;
     object *spell;
+    client_spell* spell_info;
     if (!pl->socket.monitor_spells) return;
     for (spell=pl->ob->inv; spell!=NULL; spell=spell->below) {
 	if (spell->type == SPELL) {
+        spell_info = get_client_spell_state(pl, spell);
 	    /* check if we need to update it*/
-	    if (spell->last_sp != SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA)) {
-		spell->last_sp = SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA);
-		flags |= UPD_SP_MANA;
+	    if (spell_info->last_sp != SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA)) {
+            spell_info->last_sp = SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA);
+			flags |= UPD_SP_MANA;
 	    }
-	    if (spell->last_grace != SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE)) {
-		spell->last_grace = SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE);
-		flags |= UPD_SP_GRACE;
+        if (spell_info->last_grace != SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE)) {
+            spell_info->last_grace = SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE);
+			flags |= UPD_SP_GRACE;
 	    }
-	    if (spell->last_eat != spell->stats.dam+SP_level_dam_adjust(pl->ob, spell)) {
-		spell->last_eat = spell->stats.dam+SP_level_dam_adjust(pl->ob, spell);
-		flags |= UPD_SP_DAMAGE;
+        if (spell_info->last_dam != spell->stats.dam+SP_level_dam_adjust(pl->ob, spell)) {
+            spell_info->last_dam = spell->stats.dam+SP_level_dam_adjust(pl->ob, spell);
+			flags |= UPD_SP_DAMAGE;
 	    }
 	    if (flags !=0) {
 		sl.buf = malloc(MAXSOCKSENDBUF);
@@ -2314,9 +2344,9 @@ void esrv_update_spells(player *pl) {
 		sl.len=strlen((char*)sl.buf);
 		SockList_AddChar(&sl, flags);
 		SockList_AddInt(&sl, spell->count);
-		if (flags & UPD_SP_MANA) SockList_AddShort(&sl, spell->last_sp);
-		if (flags & UPD_SP_GRACE) SockList_AddShort(&sl, spell->last_grace);
-		if (flags & UPD_SP_DAMAGE) SockList_AddShort(&sl, spell->last_eat);
+        if (flags & UPD_SP_MANA) SockList_AddShort(&sl, spell_info->last_sp);
+        if (flags & UPD_SP_GRACE) SockList_AddShort(&sl, spell_info->last_grace);
+        if (flags & UPD_SP_DAMAGE) SockList_AddShort(&sl, spell_info->last_dam);
 		flags = 0;
 		Send_With_Handling(&pl->socket, &sl);
 		free(sl.buf);
@@ -2330,7 +2360,7 @@ void esrv_remove_spell(player *pl, object *spell) {
 
     if (!pl->socket.monitor_spells) return;
     if (!pl || !spell || spell->env != pl->ob) {
-	LOG(llevError, "Invalid call to esrv_remove_spell");
+	LOG(llevError, "Invalid call to esrv_remove_spell\n");
 	return;
     }
     sl.buf = malloc(MAXSOCKSENDBUF);
@@ -2343,23 +2373,25 @@ void esrv_remove_spell(player *pl, object *spell) {
 
 /** appends the spell *spell to the Socklist we will send the data to. */
 static void append_spell (player *pl, SockList *sl, object *spell) {
+    client_spell* spell_info;
     int len, i, skill=0; 
 
     if (!(spell->name)) {
         LOG(llevError, "item number %d is a spell with no name.\n", spell->count);
         return;
     }
+    spell_info = get_client_spell_state(pl, spell);
     SockList_AddInt(sl, spell->count);
     SockList_AddShort(sl, spell->level);
     SockList_AddShort(sl, spell->casting_time);
     /* store costs and damage in the object struct, to compare to later */
-    spell->last_sp = SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA);
-    spell->last_grace = SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE);
-    spell->last_eat = spell->stats.dam+SP_level_dam_adjust(pl->ob, spell);
+    spell_info->last_sp = SP_level_spellpoint_cost(pl->ob, spell, SPELL_MANA);
+    spell_info->last_grace = SP_level_spellpoint_cost(pl->ob, spell, SPELL_GRACE);
+    spell_info->last_dam = spell->stats.dam+SP_level_dam_adjust(pl->ob, spell);
     /* send the current values */
-    SockList_AddShort(sl, spell->last_sp);
-    SockList_AddShort(sl, spell->last_grace);
-    SockList_AddShort(sl, spell->last_eat);
+    SockList_AddShort(sl, spell_info->last_sp);
+    SockList_AddShort(sl, spell_info->last_grace);
+    SockList_AddShort(sl, spell_info->last_dam);
 
     /* figure out which skill it uses, if it uses one */
     if (spell->skill) {
@@ -2397,7 +2429,7 @@ static void append_spell (player *pl, SockList *sl, object *spell) {
 void esrv_add_spells(player *pl, object *spell) {
     SockList sl;
     if (!pl) {
-	LOG(llevError, "esrv_add_spells, tried to add a spell to a NULL player");
+	LOG(llevError, "esrv_add_spells, tried to add a spell to a NULL player\n");
 	return;
     }
     if (!pl->socket.monitor_spells) return;
@@ -2425,7 +2457,7 @@ void esrv_add_spells(player *pl, object *spell) {
 	}
     }
     else if (spell->type != SPELL) {
-	LOG(llevError, "Asked to send a non-spell object as a spell");
+	LOG(llevError, "Asked to send a non-spell object as a spell\n");
 	return;
     }
     else append_spell(pl, &sl, spell);
